package com.mh.web.controller.alarm;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.AlarmCode;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.operation.IAlarmCodeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 仪表编码页面请求
 * @date 2025-01-14 16:40:58
 */
@RestController
@RequestMapping("/alarm/ac")
public class AlarmCodeController extends BaseController {

    @Autowired
    private IAlarmCodeService alarmCodeService;

    /**
     * 获取报警编码记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('alarm:ac:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmCode alarmCode)
    {
        startPage();
        List<AlarmCode> list = alarmCodeService.selectAlarmCodeList(alarmCode);
        return getDataTable(list);
    }

    /**
     * 根据报警编码记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('alarm:ac:query')")
    @GetMapping(value = "/{acId}")
    public AjaxResult getInfo(@PathVariable String acId)
    {
        return success(alarmCodeService.selectAlarmCodeById(acId));
    }

    /**
     * 新增报警编码记录
     */
    @PreAuthorize("@ss.hasPermi('alarm:ac:add')")
    @Log(title = "报警编码记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody AlarmCode alarmCode)
    {
        alarmCode.setCreateBy(getUsername());
        alarmCode.setCreateTime(new Date());
        return toAjax(alarmCodeService.insertAlarmCode(alarmCode));
    }

    /**
     * 修改报警编码记录信息
     */
    @PreAuthorize("@ss.hasPermi('alarm:ac:edit')")
    @Log(title = "报警编码记录管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody AlarmCode alarmCode)
    {
        alarmCode.setUpdateBy(getUsername());
        alarmCode.setUpdateTime(new Date());
        return toAjax(alarmCodeService.updateAlarmCode(alarmCode));
    }

    /**
     * 删除报警编码记录管理
     */
    @PreAuthorize("@ss.hasPermi('alarm:ac:remove')")
    @Log(title = "报警编码记录管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{acIds}")
    public AjaxResult remove(@PathVariable String[] acIds)
    {
        return toAjax(alarmCodeService.deleteAlarmCodeByIds(acIds));
    }

}
