package com.mh.web.controller.alarm;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.AlarmRecords;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.operation.IAlarmRecordsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 报警记录查询规则
 * @date 2025-01-14 17:32:55
 */
@RestController
@RequestMapping("/alarm/ard")
public class AlarmRecordsController extends BaseController {

    @Autowired
    private IAlarmRecordsService alarmRecordsService;

    /**
     * 获取报警规则记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('alarm:ard:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmRecords alarmRecords)
    {
        startPage();
        List<AlarmRecords> list = alarmRecordsService.selectAlarmRecordsList(alarmRecords);
        return getDataTable(list);
    }

    /**
     * 根据报警规则记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('alarm:ard:query')")
    @GetMapping(value = "/{acdId}")
    public AjaxResult getInfo(@PathVariable String acdId)
    {
        return success(alarmRecordsService.selectAlarmRecordsById(acdId));
    }

    /**
     * 删除报警规则记录管理
     */
    @PreAuthorize("@ss.hasPermi('alarm:ard:remove')")
    @Log(title = "报警规则记录管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{acdIds}")
    public AjaxResult remove(@PathVariable String[] acdIds)
    {
        return toAjax(alarmRecordsService.deleteAlarmRecordsByIds(acdIds));
    }
    
}
