package com.mh.web.controller.alarm;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.AlarmRules;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.common.utils.DateUtils;
import com.mh.system.service.operation.IAlarmRulesService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 仪表报警规则
 * @date 2025-01-14 17:32:55
 */
@RestController
@RequestMapping("/alarm/ar")
public class AlarmRulesController extends BaseController {

    @Autowired
    private IAlarmRulesService alarmRulesService;

    /**
     * 获取报警规则记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('alarm:ar:list')")
    @GetMapping("/list")
    public TableDataInfo list(AlarmRules alarmRules)
    {
        startPage();
        List<AlarmRules> list = alarmRulesService.selectAlarmRulesList(alarmRules);
        return getDataTable(list);
    }

    /**
     * 根据报警规则记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('alarm:ar:query')")
    @GetMapping(value = "/{acId}")
    public AjaxResult getInfo(@PathVariable String acId)
    {
        return success(alarmRulesService.selectAlarmRulesById(acId));
    }

    /**
     * 新增报警规则记录
     */
    @PreAuthorize("@ss.hasPermi('alarm:ar:add')")
    @Log(title = "报警规则记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody AlarmRules alarmRules)
    {
        alarmRules.setCreateBy(getUsername());
        alarmRules.setCreateTime(DateUtils.getNowDate());
        return toAjax(alarmRulesService.insertAlarmRules(alarmRules));
    }

    /**
     * 修改报警规则记录信息
     */
    @PreAuthorize("@ss.hasPermi('alarm:ar:edit')")
    @Log(title = "报警规则记录管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody AlarmRules alarmRules)
    {
        alarmRules.setUpdateBy(getUsername());
        alarmRules.setUpdateTime(DateUtils.getNowDate());
        return toAjax(alarmRulesService.updateAlarmRules(alarmRules));
    }

    /**
     * 删除报警规则记录管理
     */
    @PreAuthorize("@ss.hasPermi('alarm:ar:remove')")
    @Log(title = "报警规则记录管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{acIds}")
    public AjaxResult remove(@PathVariable String[] acIds)
    {
        return toAjax(alarmRulesService.deleteAlarmRulesByIds(acIds));
    }
    
}
