package com.mh.web.controller.comprehensive;

import com.mh.common.annotation.Log;
import com.mh.common.config.MHConfig;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.model.LoginUser;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.common.utils.file.FileUploadUtils;
import com.mh.common.utils.file.MimeTypeUtils;
import com.mh.system.service.ISysParamsService;
import com.mh.system.service.overview.IProOverviewService;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 项目总览
 * @date 2025-03-19 11:08:41
 */
@RestController
@RequestMapping("/pro/overview")
public class ProOverviewController extends BaseController {

    private final ISysParamsService sysParamsService;

    private final IProOverviewService proOverviewService;

    public ProOverviewController(ISysParamsService sysParamsService, IProOverviewService proOverviewService) {
        this.sysParamsService = sysParamsService;
        this.proOverviewService = proOverviewService;
    }

    /**
     * 获取项目简介
     * @return
     */
    @GetMapping("/introduction")
    public TableDataInfo introduction() {
        return getDataTable(sysParamsService.queryList());
    }

    /**
     * 头像上传
     */
    @Log(title = "项目图标", businessType = BusinessType.UPDATE)
    @PostMapping("/logo")
    public AjaxResult avatar(@RequestParam("logo") MultipartFile file, @RequestParam("proId") String proId) throws Exception
    {
        if (!file.isEmpty())
        {
            String avatar = FileUploadUtils.upload(MHConfig.getAvatarPath(), file, MimeTypeUtils.IMAGE_EXTENSION);
            String username = getUsername();
            if (sysParamsService.updateLogo(avatar, proId, username))
            {
                AjaxResult ajax = AjaxResult.success();
                ajax.put("imgUrl", avatar);
                return ajax;
            }
        }
        return error("上传图片异常，请联系管理员");
    }

    /**
     * 获取项目概况
     * @return
     */
    @GetMapping("/profile")
    public TableDataInfo profile() {
        return getDataTable(proOverviewService.getProProfile());
    }

    /**
     * 获取能耗分析：查询最近12个月的能耗数据
     * @return
     */
    @GetMapping("/energyAnalysis")
    public TableDataInfo energyAnalysis() {
        return getDataTable(proOverviewService.energyAnalysis());
    }

    /**
     * 各个系统主要的指标参数
     * @return
     */
    @GetMapping("/mainParams")
    public TableDataInfo mainParams() {
        return getDataTable(proOverviewService.mainParams());
    }

}
