package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.ICollectionParamsManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 仪表管理表控制类
 * @date 2025-01-13 16:10:17
 */
@RestController
@RequestMapping("/device/cpm")
public class CollectionParamsManageController extends BaseController {

    @Autowired
    private ICollectionParamsManageService iCollectionParamsManageService;

    /**
     * 获取列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:list')")
    @GetMapping("/list")
    public TableDataInfo list(CollectionParamsManage communicationParams)
    {
        startPage();
        List<CollectionParamsManage> list = iCollectionParamsManageService.selectCollectionParamsManageList(communicationParams);
        return getDataTable(list);
    }

    /**
     * 根据id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:query')")
    @GetMapping(value = "/{cpmId}")
    public AjaxResult getInfo(@PathVariable String cpmId)
    {
        return success(iCollectionParamsManageService.selectCollectionParamsManageById(cpmId));
    }

    /**
     * 新增设备采集参数
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:add')")
    @Log(title = "设备采集参数管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody CollectionParamsManage communicationParams)
    {
        communicationParams.setCreateBy(getUsername());
        communicationParams.setCreateTime(new Date());
        return toAjax(iCollectionParamsManageService.insertCollectionParamsManage(communicationParams));
    }

    /**
     * 修改设备采集参数信息
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:edit')")
    @Log(title = "设备采集参数管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody CollectionParamsManage communicationParams)
    {
        communicationParams.setUpdateBy(getUsername());
        communicationParams.setUpdateTime(new Date());
        return toAjax(iCollectionParamsManageService.updateCollectionParamsManage(communicationParams));
    }

    /**
     * 删除设备采集参数管理
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:remove')")
    @Log(title = "设备采集参数管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{cpmIds}")
    public AjaxResult remove(@PathVariable String[] cpmIds)
    {
        return toAjax(iCollectionParamsManageService.deleteCommunicationByIds(cpmIds));
    }

}
