package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.CommunicationParams;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.ICommunicationParamsService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备通信采集参数管理
 * @date 2025-01-08 14:36:24
 */
@RestController
@RequestMapping("/device/cmp")
public class CommunicationParamsController extends BaseController {

    @Autowired
    private ICommunicationParamsService iCommunicationParamsService;

    /**
     * 获取列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:cmp:list')")
    @GetMapping("/list")
    public TableDataInfo list(CommunicationParams communicationParams)
    {
        startPage();
        List<CommunicationParams> list = iCommunicationParamsService.selectCmPList(communicationParams);
        return getDataTable(list);
    }

    /**
     * 根据id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:cmp:query')")
    @GetMapping(value = "/{cmpId}")
    public AjaxResult getInfo(@PathVariable String cmpId)
    {
        return success(iCommunicationParamsService.selectCommunicationParamsById(cmpId));
    }

    /**
     * 新增网关
     */
    @PreAuthorize("@ss.hasPermi('device:cmp:add')")
    @Log(title = "设备通信采集参数管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody CommunicationParams communicationParams)
    {
        communicationParams.setCreateBy(getUsername());
        communicationParams.setCreateTime(new Date());
        return toAjax(iCommunicationParamsService.insertCommunicationParams(communicationParams));
    }

    /**
     * 修改网关信息
     */
    @PreAuthorize("@ss.hasPermi('device:cmp:edit')")
    @Log(title = "设备通信采集参数管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody CommunicationParams communicationParams)
    {
        communicationParams.setUpdateBy(getUsername());
        communicationParams.setUpdateTime(new Date());
        return toAjax(iCommunicationParamsService.updateCommunicationParams(communicationParams));
    }

    /**
     * 删除设备通信采集参数管理
     */
    @PreAuthorize("@ss.hasPermi('device:cmp:remove')")
    @Log(title = "设备通信采集参数管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{cmpIds}")
    public AjaxResult remove(@PathVariable String[] cmpIds)
    {
        return toAjax(iCommunicationParamsService.deleteCommunicationByIds(cmpIds));
    }
}
