package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.DeviceInOutManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.IDeviceInOutManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备出入库流水表记录
 * @date 2025-01-09 15:24:24
 */
@RestController
@RequestMapping("/device/spareInOut")
public class DeviceInOutManageController extends BaseController {
    @Autowired
    private IDeviceInOutManageService deviceInOutManageService;

    /**
     * 获取备品/备件出入库管理流水记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:spareInOut:list')")
    @GetMapping("/list")
    public TableDataInfo list(DeviceInOutManage deviceInOutManage)
    {
        startPage();
        List<DeviceInOutManage> list = deviceInOutManageService.selectDeviceInOutManageList(deviceInOutManage);
        return getDataTable(list);
    }

    /**
     * 根据备品/备件出入库管理流水记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:spareInOut:query')")
    @GetMapping(value = "/{spareId}")
    public AjaxResult getInfo(@PathVariable String spareId)
    {
        return success(deviceInOutManageService.selectDeviceInOutManageById(spareId));
    }

    /**
     * 新增备品/备件出入库管理流水记录
     */
    @PreAuthorize("@ss.hasPermi('device:spareInOut:add')")
    @Log(title = "备品/备件出入库管理流水记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody List<DeviceInOutManage> deviceInOutManage)
    {
        return toAjax(deviceInOutManageService.insertDeviceInOutManage(deviceInOutManage, getUsername()));
    }

//    /**
//     * 修改备品/备件出入库管理流水记录信息
//     */
//    @PreAuthorize("@ss.hasPermi('device:spareInOut:edit')")
//    @Log(title = "备品/备件出入库管理流水记录管理", businessType = BusinessType.UPDATE)
//    @PutMapping
//    public AjaxResult edit(@Validated @RequestBody DeviceInOutManage deviceInOutManage)
//    {
//        deviceInOutManage.setUpdateBy(getUsername());
//        return toAjax(deviceInOutManageService.updateDeviceInOutManage(deviceInOutManage));
//    }
//
//    /**
//     * 删除备品/备件出入库管理流水记录管理
//     */
//    @PreAuthorize("@ss.hasPermi('device:spareInOut:remove')")
//    @Log(title = "备品/备件出入库管理流水记录管理", businessType = BusinessType.DELETE)
//    @DeleteMapping("/{spareIds}")
//    public AjaxResult remove(@PathVariable String[] spareIds)
//    {
//        return toAjax(deviceInOutManageService.deleteDeviceInOutManageByIds(spareIds));
//    }

}
