package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.DeviceLedger;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.IDeviceLedgerService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备台账管理
 * @date 2025-01-10 16:38:04
 */
@RestController
@RequestMapping("/device/ledger")
public class DeviceLedgerController extends BaseController {

    @Autowired
    private IDeviceLedgerService deviceLedgerService;

    /**
     * 获取设备台账列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:ledger:list')")
    @GetMapping("/list")
    public TableDataInfo list(DeviceLedger ledgerInfo)
    {
        startPage();
        List<DeviceLedger> list = deviceLedgerService.selectDeviceLedgerList(ledgerInfo);
        return getDataTable(list);
    }

    /**
     * 根据设备台账id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:ledger:query')")
    @GetMapping(value = "/{ledgerId}")
    public AjaxResult getInfo(@PathVariable String ledgerId)
    {
        return success(deviceLedgerService.selectDeviceLedgerById(ledgerId));
    }

    /**
     * 新增设备台账
     */
    @PreAuthorize("@ss.hasPermi('device:ledger:add')")
    @Log(title = "设备台账管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody DeviceLedger deviceLedger)
    {
        deviceLedger.setCreateBy(getUsername());
        deviceLedger.setCreateTime(new Date());
        return toAjax(deviceLedgerService.insertDeviceLedger(deviceLedger));
    }

    /**
     * 修改设备台账信息
     */
    @PreAuthorize("@ss.hasPermi('device:ledger:edit')")
    @Log(title = "设备台账管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody DeviceLedger deviceLedger)
    {
        deviceLedger.setUpdateBy(getUsername());
        deviceLedger.setUpdateTime(new Date());
        return toAjax(deviceLedgerService.updateDeviceLedger(deviceLedger));
    }

    /**
     * 删除设备台账管理
     */
    @PreAuthorize("@ss.hasPermi('device:ledger:remove')")
    @Log(title = "设备台账管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ledgerIds}")
    public AjaxResult remove(@PathVariable String[] ledgerIds)
    {
        return toAjax(deviceLedgerService.deleteDeviceLedgerByIds(ledgerIds));
    }
    
}
