package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.DeviceQrManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.IDeviceQrManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备二维码维护管理
 * @date 2025-01-09 16:48:30
 */
@RestController
@RequestMapping("/device/qr")
public class DeviceQrManageController extends BaseController {

    @Autowired
    private IDeviceQrManageService deviceQrManageService;

    @PreAuthorize("@ss.hasPermi('device:qr:create')")
    @GetMapping("/create")
    public AjaxResult create(@RequestParam Integer qrNum) throws Exception {
        return success(deviceQrManageService.createQrCode(qrNum, getUsername()));
    }


    /**
     * 获取二维码维护记录列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:qr:list')")
    @GetMapping("/list")
    public TableDataInfo list(DeviceQrManage deviceQrManage)
    {
        startPage();
        List<DeviceQrManage> list = deviceQrManageService.selectDeviceQrManageList(deviceQrManage);
        return getDataTable(list);
    }

    /**
     * 根据二维码维护记录id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:qr:query')")
    @GetMapping(value = "/{qrId}")
    public AjaxResult getInfo(@PathVariable String qrId)
    {
        return success(deviceQrManageService.selectDeviceQrManageById(qrId));
    }

    /**
     * 新增二维码维护记录
     */
    @PreAuthorize("@ss.hasPermi('device:qr:add')")
    @Log(title = "二维码维护记录管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody DeviceQrManage deviceQrManage)
    {
        deviceQrManage.setCreateBy(getUsername());
        deviceQrManage.setCreateTime(new Date());
        return toAjax(deviceQrManageService.insertDeviceQrManage(deviceQrManage));
    }

    /**
     * 修改二维码维护记录信息
     */
    @PreAuthorize("@ss.hasPermi('device:qr:edit')")
    @Log(title = "二维码维护记录管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody DeviceQrManage deviceQrManage)
    {
        deviceQrManage.setUpdateBy(getUsername());
        deviceQrManage.setUpdateTime(new Date());
        return toAjax(deviceQrManageService.updateDeviceQrManage(deviceQrManage));
    }

    /**
     * 删除二维码维护记录管理
     */
    @PreAuthorize("@ss.hasPermi('device:qr:remove')")
    @Log(title = "二维码维护记录管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{qrIds}")
    public AjaxResult remove(@PathVariable String[] qrIds)
    {
        return toAjax(deviceQrManageService.deleteDeviceQrManageByIds(qrIds));
    }

}
