package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.GatewayManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.domain.SysNotice;
import com.mh.system.service.ISysNoticeService;
import com.mh.system.service.device.IGatewayManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 网关设备管理
 * @date 2025-01-08 14:36:24
 */
@RestController
@RequestMapping("/device/gateway")
public class GatewayManageController extends BaseController {

    @Autowired
    private IGatewayManageService gatewayManageService;

    /**
     * 获取网关列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:gw:list')")
    @GetMapping("/list")
    public TableDataInfo list(GatewayManage gatewayManage)
    {
        startPage();
        List<GatewayManage> list = gatewayManageService.selectGwManageList(gatewayManage);
        return getDataTable(list);
    }

    /**
     * 根据网关id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:gw:query')")
    @GetMapping(value = "/{gwId}")
    public AjaxResult getInfo(@PathVariable String gwId)
    {
        return success(gatewayManageService.selectGwManageById(gwId));
    }

    /**
     * 新增网关
     */
    @PreAuthorize("@ss.hasPermi('device:gw:add')")
    @Log(title = "网关管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody GatewayManage gatewayManage)
    {
        gatewayManage.setCreateBy(getUsername());
        gatewayManage.setCreateTime(new Date());
        return toAjax(gatewayManageService.insertGatewayManage(gatewayManage));
    }

    /**
     * 修改网关信息
     */
    @PreAuthorize("@ss.hasPermi('device:gw:edit')")
    @Log(title = "网关管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody GatewayManage gatewayManage)
    {
        gatewayManage.setUpdateBy(getUsername());
        gatewayManage.setUpdateTime(new Date());
        return toAjax(gatewayManageService.updateGateway(gatewayManage));
    }

    /**
     * 删除网关管理
     */
    @PreAuthorize("@ss.hasPermi('device:gw:remove')")
    @Log(title = "网关管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{gatewayIds}")
    public AjaxResult remove(@PathVariable String[] gatewayIds)
    {
        return toAjax(gatewayManageService.deleteGatewayByIds(gatewayIds));
    }
}
