package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.MaintainInfo;
import com.mh.common.core.domain.entity.MaintainSum;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.IMaintainInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 维护设备设备管理
 * @date 2025-01-08 14:36:24
 */
@RestController
@RequestMapping("/device/maintain")
public class MaintainInfoController extends BaseController {

    @Autowired
    private IMaintainInfoService maintainInfoService;

    /**
     * 获取维护设备列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:list')")
    @GetMapping("/list")
    public TableDataInfo list(MaintainInfo maintainInfo)
    {
        startPage();
        List<MaintainInfo> list = maintainInfoService.selectMaintainInfoList(maintainInfo);
        return getDataTable(list);
    }

    /**
     * 根据维护设备id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:query')")
    @GetMapping(value = "/{maintainId}")
    public AjaxResult getInfo(@PathVariable String maintainId)
    {
        return success(maintainInfoService.selectMaintainInfoById(maintainId));
    }

    /**
     * 新增维护设备
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:add')")
    @Log(title = "维护设备管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody MaintainInfo maintainInfo)
    {
        maintainInfo.setCreateBy(getUsername());
        maintainInfo.setCreateTime(new Date());
        return toAjax(maintainInfoService.insertMaintainInfo(maintainInfo));
    }

    /**
     * 修改维护设备信息
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:edit')")
    @Log(title = "维护设备管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody MaintainInfo maintainInfo)
    {
        maintainInfo.setUpdateBy(getUsername());
        maintainInfo.setUpdateTime(new Date());
        return toAjax(maintainInfoService.updateMaintainInfo(maintainInfo));
    }

    /**
     * 删除维护设备管理
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:remove')")
    @Log(title = "维护设备管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{maintainIds}")
    public AjaxResult remove(@PathVariable String[] maintainIds)
    {
        return toAjax(maintainInfoService.deleteMaintainInfoByIds(maintainIds));
    }

    /**
     * 删除维护设备管理
     */
    @PreAuthorize("@ss.hasPermi('device:maintain:remove')")
    @Log(title = "维护设备管理", businessType = BusinessType.DELETE)
    @GetMapping("/maintainSum")
    public AjaxResult sumMaintain(@RequestParam(value= "buildingId", required=false)String buildingId,
                                  @RequestParam(value= "curDate", required=false)String curDate) {
        try {
            MaintainSum record;
            if (buildingId == null || buildingId.isEmpty()) {
                record = maintainInfoService.queryMaintainSum("所有", curDate);
            } else {
                record = maintainInfoService.queryMaintainSum(buildingId, curDate);
            }
            if (record == null) {
                record = new MaintainSum();
            }
            return AjaxResult.success(record);
        } catch (Exception e) {
            return AjaxResult.error();
        }
    }

}
