package com.mh.web.controller.device;

import com.mh.common.annotation.ControlDeviceAno;
import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.dto.DeviceMonitorDTO;
import com.mh.common.core.domain.entity.OrderEntity;
import com.mh.common.core.domain.vo.DeviceOperateMonitorVO;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.framework.mqtt.service.IMqttGatewayService;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.operation.IOperationDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备控制类
 * @date 2025-01-15 14:11:22
 */
@Slf4j
@RestController
@RequestMapping("/device")
public class OperationController extends BaseController {

    @Value("${control.topic}")
    String controlTopic;

    private final ICollectionParamsManageService iCollectionParamsManageService;

    private final IOperationDeviceService iOperationService;

    private final IMqttGatewayService iMqttGatewayService;

    @Autowired
    public OperationController(ICollectionParamsManageService iCollectionParamsManageService,
                               IOperationDeviceService iOperationService,
                               IMqttGatewayService iMqttGatewayService) {
        this.iCollectionParamsManageService = iCollectionParamsManageService;
        this.iOperationService = iOperationService;
        this.iMqttGatewayService = iMqttGatewayService;
    }

    /**
     * 获取监控列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:list')")
    @GetMapping("/monitor/list")
    public TableDataInfo list(@RequestParam(name = "systemType") String systemType)
    {
        List<DeviceMonitorDTO> list = iCollectionParamsManageService.selectMonitorListBySystemType(systemType);
        return getDataTable(list);
    }

    /**
     * 对设备进行操作处理获取对应的列表数据
     *
     * @param systemType
     * @return
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:operationList')")
    @GetMapping("/operation/list")
    public TableDataInfo operationDeviceList(@RequestParam(name = "systemType") String systemType) {
        try {
            List<DeviceOperateMonitorVO> deviceOperateMonitorVOS = iCollectionParamsManageService.operationDeviceList(systemType);
            return getDataTable(deviceOperateMonitorVOS);
        } catch (Exception e) {
            log.error("设备操作失败", e);
            return getDataTable(null);
        }
    }

    /**
     * 对设备进行操作处理
     *
     * @param changeValues
     * @return
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:operation')")
    @PostMapping("/operation")
    @Log(title = "设备指令操作操作", businessType = BusinessType.UPDATE)
    @ControlDeviceAno(value = "设备操作")
    public AjaxResult operationDevice(@RequestBody List<OrderEntity> changeValues) {
        try {
            String sendOrder = iOperationService.operationDevice(changeValues);
            // 获取mqtt操作队列(后期通过mqtt队列配置发送主题)
            log.info("发送主题：{},消息：{}", controlTopic, sendOrder);
//            iMqttGatewayService.publish(controlTopic, sendOrder, 1);
        } catch (Exception e) {
            log.error("设备操作失败", e);
            return AjaxResult.error();
        }
        return AjaxResult.success();
    }

}
