package com.mh.web.controller.device;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.SparePartsManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.device.ISparePartsManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 备品/备件出入库管理
 * @date 2025-01-09 11:40:25
 */
@RestController
@RequestMapping("/device/spare")
public class SparePartsManageController extends BaseController {

    @Autowired
    private ISparePartsManageService sparePartsManageService;

    /**
     * 获取备品/备件出入库管理列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:spare:list')")
    @GetMapping("/list")
    public TableDataInfo list(SparePartsManage sparePartsManage)
    {
        startPage();
        List<SparePartsManage> list = sparePartsManageService.selectSparePartsManageList(sparePartsManage);
        return getDataTable(list);
    }

    /**
     * 根据备品/备件出入库管理id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('device:spare:query')")
    @GetMapping(value = "/{spareId}")
    public AjaxResult getInfo(@PathVariable String spareId)
    {
        return success(sparePartsManageService.selectSparePartsManageById(spareId));
    }

    /**
     * 新增备品/备件出入库管理
     */
    @PreAuthorize("@ss.hasPermi('device:spare:add')")
    @Log(title = "备品/备件出入库管理管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SparePartsManage sparePartsManage)
    {
        sparePartsManage.setCreateBy(getUsername());
        return toAjax(sparePartsManageService.insertSparePartsManage(sparePartsManage));
    }

    /**
     * 修改备品/备件出入库管理信息
     */
    @PreAuthorize("@ss.hasPermi('device:spare:edit')")
    @Log(title = "备品/备件出入库管理管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SparePartsManage sparePartsManage)
    {
        sparePartsManage.setUpdateBy(getUsername());
        return toAjax(sparePartsManageService.updateSparePartsManage(sparePartsManage));
    }

    /**
     * 删除备品/备件出入库管理管理
     */
    @PreAuthorize("@ss.hasPermi('device:spare:remove')")
    @Log(title = "备品/备件出入库管理管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{spareIds}")
    public AjaxResult remove(@PathVariable String[] spareIds)
    {
        return toAjax(sparePartsManageService.deleteSparePartsManageByIds(spareIds));
    }

}
