package com.mh.web.controller.energy;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.utils.DateUtils;
import com.mh.system.service.energy.IComprehensiveEnergyConsumptionService;
import com.mh.system.service.energy.IHistoryDataPreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 综合能管管理
 * @date 2025-03-24 15:53:13
 */
@RestController
@RequestMapping("/energy/cec")
public class ComprehensiveEnergyConsumptionController extends BaseController {

    private final IComprehensiveEnergyConsumptionService comprehensiveEnergyConsumptionService;

    private final IHistoryDataPreService historyDataPreService;

    @Autowired
    public ComprehensiveEnergyConsumptionController(IComprehensiveEnergyConsumptionService comprehensiveEnergyConsumptionService, IHistoryDataPreService historyDataPreService) {
        this.comprehensiveEnergyConsumptionService = comprehensiveEnergyConsumptionService;
        this.historyDataPreService = historyDataPreService;
    }

    /**
     * 能耗结构图
     * @param vo
     * @return
     */
    @PostMapping("/struct")
    public TableDataInfo structure(@RequestBody EnergyQueryVO vo) {
        DateUtils.sysEnergyDateChange(vo);
        return getDataTable(comprehensiveEnergyConsumptionService.structure(vo));
    }

    /**
     * 能耗钻取:实际上获取每个系统的用电能耗数据
     * @param vo
     * @return
     */
    @PostMapping("/drilling")
    public TableDataInfo drilling(@RequestBody EnergyQueryVO vo) {
        return getDataTable(comprehensiveEnergyConsumptionService.drilling(vo));
    }

    @GetMapping("/pre/topData")
    public TableDataInfo getTopData(@RequestParam("systemType") String systemType,
                                    @RequestParam(value = "deviceType", required = false) String deviceType) {
        return getDataTable(historyDataPreService.getTopData(systemType, deviceType));
    }

    @GetMapping("/pre/echart")
    public TableDataInfo getEnergyPre(@RequestParam("systemType") String systemType,
                                      @RequestParam(value = "deviceType", required = false) String deviceType,
                                      @RequestParam(value = "beginTime", required = false) String beginDate,
                                      @RequestParam(value = "endTime", required = false) String endDate) {
        return getDataTable(historyDataPreService.getEnergyPre(systemType, beginDate, endDate, deviceType));
    }


}
