package com.mh.web.controller.energy;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.vo.EnergyConsumptionVO;
import com.mh.common.utils.DateUtils;
import com.mh.system.service.energy.IEnergyService;
import jakarta.annotation.Resource;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 系统能耗分析
 * @date 2023/7/18 14:00:09
 */
@RestController
@RequestMapping("/energy")
public class SysEnergyConsumptionController extends BaseController {

    @Resource
    private IEnergyService energyService;

    /**
     * 获取系统瞬时能耗分析
     * @param vo
     * @return
     */
    @PostMapping("/sys")
    public AjaxResult sys(@RequestBody EnergyConsumptionVO vo) {
        DateUtils.energyDateChange(vo);
        return energyService.sys(vo);
    }

    /**
     * 获取能耗同比分析
     * @param vo
     * @return
     */
    @PostMapping("/yoy")
    public AjaxResult yoy(@RequestBody EnergyConsumptionVO vo) {
        DateUtils.energyDateChange(vo);
        return energyService.yoy(vo);
    }

    /**
     * 获取能耗环比分析
     * @param vo
     * @return
     */
    @PostMapping("/mom")
    public AjaxResult mom(@RequestBody EnergyConsumptionVO vo) {
        DateUtils.energyDateChange(vo);
        return energyService.mom(vo);
    }

    /**
     * 获取能耗同比环比分析
     * @param vo
     * @return
     */
    @PostMapping("/yoyMom")
    public AjaxResult yoyMom(@RequestBody EnergyConsumptionVO vo) {
        DateUtils.energyDateChange(vo);
        return energyService.yoyMom(vo);
    }

    /**
     * 获取机房各个设备能耗
     * @param vo
     * @return
     */
    @PostMapping("/device")
    public AjaxResult device(@RequestBody EnergyConsumptionVO vo) {
        DateUtils.energyDateChange(vo);
        return energyService.device(vo);
    }

}
