package com.mh.web.controller.energy;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.ListUtils;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.system.service.energy.IEnergyQueryService;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 系统能源查询（中港项目）
 * @date 2023-12-13 09:39:02
 */
@Slf4j
@RestController
@RequestMapping("/energy")
public class SysEnergyQueryController {

    @Resource
    private IEnergyQueryService energyQueryService;

    /**
     * 整体机房图形，表格数据查询（公用一个接口）
     * @param page
     * @return
     */
    @PostMapping("/sys/query")
    public AjaxResult sysQuery(@RequestBody EnergyQueryVO page) {
        return energyQueryService.sysQuery(page);
    }

    /**
     * 整体机房表格数据查询
     * @param vo
     * @param response
     */
    @PostMapping("/sys/query/export")
    public void exportSysTable(@RequestBody EnergyQueryVO vo, HttpServletResponse response) {
        // 文件名
        try {
            String fileName =  "机房整体能耗表.xlsx";
            // 从数据库获取数据
            List<Map<String, Object>> dataList = (List<Map<String, Object>>) energyQueryService.sysQuery(vo).get("data");
            if (dataList != null) {
                // 设置响应格式
                response.setContentType("application/vdn.ms-excel;charset=utf-8");
                response.setHeader("Content-Disposition", "attachment; filename=\"" + URLEncoder.encode(fileName, "UTF-8") + "\"");
                response.setCharacterEncoding("UTF-8");

                List<List<String>> head = ListUtils.newArrayList();
                List<String> titleArr = new ArrayList<>();
                List<String> timeStrArr = new ArrayList<>();
                List<String> cold = new ArrayList<>();
                List<String> meter = new ArrayList<>();
                List<String> cop = new ArrayList<>();
                for (Map<String, Object> map : dataList) {
                    if (map.containsKey("titleArr")) {
                        titleArr = Arrays.asList((String[]) map.get("titleArr"));
                        List<String> head0 = ListUtils.newArrayList();
                        head0.add("日期");
                        List<String> head1 = ListUtils.newArrayList();
                        head1.add("制冷量");
                        List<String> head2 = ListUtils.newArrayList();
                        head2.add("耗电量");
                        List<String> head3= ListUtils.newArrayList();
                        head3.add("COP");
                        head.add(head0);
                        head.add(head1);
                        head.add(head2);
                        head.add(head3);
                    }
                    if (map.containsKey("timeStrArr")) {
                        timeStrArr = Arrays.asList((String[])map.get("timeStrArr"));
                    }
                    if (map.containsKey("cold")) {
                        cold = Arrays.asList((String[])map.get("cold"));
                    }
                    if (map.containsKey("meter")) {
                        meter = Arrays.asList((String[])map.get("meter"));
                    }
                    if (map.containsKey("cop")) {
                        cop = Arrays.asList((String[])map.get("cop"));
                    }
                }
                List<List<Object>> excelDataList = ListUtils.newArrayList();
                for (int i = 0; i < timeStrArr.size(); i++) {
                    List<Object> list1 = ListUtils.newArrayList();
                    list1.add(timeStrArr.get(i));
                    list1.add(cold.get(i));
                    list1.add(meter.get(i));
                    list1.add(cop.get(i));
                    excelDataList.add(list1);
                }
                // 内容格式
                EasyExcel.write(response.getOutputStream()).head(head).sheet("机房整体能耗表").doWrite(excelDataList);
            }
        } catch (Exception e) {
            log.error("下载报表异常", e);
            throw new RuntimeException("下载报表异常");
        }
    }

    /**
     * 各个设备类型图形，表格数据查询（公用一个接口）
     * @param page
     * @return
     */
    @PostMapping("/sys/deviceType/query")
    public AjaxResult deviceTypeQuery(@RequestBody EnergyQueryVO page) {
        return energyQueryService.deviceTypeQuery(page);
    }

}
