package com.mh.web.controller.monitor;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.dto.DeviceMonitorDTO;
import com.mh.common.core.domain.vo.DeviceOperateMonitorVO;
import com.mh.common.core.page.TableDataInfo;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.device.ICoolingSystemMonitorService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 冷源系统监测
 * @date 2025-03-17 17:36:44
 */
@Slf4j
@RestController
@RequestMapping("/device/cs")
public class CoolingSystemMonitorController extends BaseController {

    private final ICollectionParamsManageService iCollectionParamsManageService;

    private final ICoolingSystemMonitorService iCoolingSystemMonitorService;

    public CoolingSystemMonitorController(ICollectionParamsManageService iCollectionParamsManageService, ICoolingSystemMonitorService iCoolingSystemMonitorService) {
        this.iCollectionParamsManageService = iCollectionParamsManageService;
        this.iCoolingSystemMonitorService = iCoolingSystemMonitorService;
    }

    /**
     * 获取监控列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:list')")
    @GetMapping("/monitor/list")
    public TableDataInfo list(@RequestParam(name = "systemType") String systemType)
    {
        List<DeviceMonitorDTO> list = iCollectionParamsManageService.selectMonitorListBySystemType(systemType);
        return getDataTable(list);
    }

    /**
     * 对设备进行操作处理获取对应的列表数据
     *
     * @param systemType
     * @return
     */
    @PreAuthorize("@ss.hasPermi('device:cpm:operationList')")
    @GetMapping("/operation/list")
    public TableDataInfo operationDeviceList(@RequestParam(name = "systemType") String systemType) {
        try {
            List<DeviceOperateMonitorVO> deviceOperateMonitorVOS = iCollectionParamsManageService.operationDeviceList(systemType);
            return getDataTable(deviceOperateMonitorVOS);
        } catch (Exception e) {
            log.error("设备操作失败", e);
            return getDataTable(null);
        }
    }

    /**
     * 获取天气数据
     * @return
     */
    @GetMapping("/getWeatherData")
    public AjaxResult getWeatherData() {
        return AjaxResult.success(iCoolingSystemMonitorService.getWeatherData());
    }

    /**
     * 查询运行天数
     * @return
     */
    @GetMapping("/runTime")
    public AjaxResult getRunTime(){
        return AjaxResult.success(iCoolingSystemMonitorService.getRunTime());
    }

    @GetMapping("/sysPerformance")
    public AjaxResult getSysPerformance(){
        return AjaxResult.success(iCoolingSystemMonitorService.getSysPerformance());
    }

    /**
     * 启动按钮参数获取
     * @param systemType
     * @param paramType
     * @return
     */
    @GetMapping("/oneKeyButton")
    public AjaxResult getOneKeyButton(@RequestParam(name = "systemType") String systemType, @RequestParam(name = "paramType") String paramType) {
        return AjaxResult.success(iCoolingSystemMonitorService.getOneKeyButton(systemType, paramType));
    }

    /**
     * 获取当前系统模式以及现在的运行状态
     * @param systemType
     * @param paramType
     * @return
     */
    @GetMapping("/getSystemMode")
    public AjaxResult getSystemMode(@RequestParam(name = "systemType") String systemType, @RequestParam(name = "paramType") String paramType) {
        return AjaxResult.success(iCoolingSystemMonitorService.getSystemMode(systemType, paramType));
    }

}
