package com.mh.web.controller.monitor;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.ColumnData;
import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.dto.HotWaterControlDTO;
import com.mh.common.core.domain.dto.HotWaterNowDataDTO;
import com.mh.common.core.domain.dto.PumpInfoDTO;
import com.mh.common.core.domain.entity.ChillersEntity;
import com.mh.common.core.page.TableDataInfo;
import com.mh.system.service.device.ICollectionParamsManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 生活热水监控
 * @date 2025-02-17 14:23:33
 */
@RestController
@RequestMapping("/device/hotWater")
public class HotWaterMonitorController extends BaseController {

    private final ICollectionParamsManageService iCollectionParamsManageService;

    @Autowired
    public HotWaterMonitorController(ICollectionParamsManageService iCollectionParamsManageService) {
        this.iCollectionParamsManageService = iCollectionParamsManageService;
    }

    /**
     * 获取生活热水监控公共信息内容
     * @param systemType
     * @param floorId
     * @return
     */
    @GetMapping("/monitorPublic")
    public TableDataInfo monitorPublic(@RequestParam("systemType") String systemType, @RequestParam("floorId") String floorId) {
        List<ColumnFilter> list = iCollectionParamsManageService.monitorPublic(systemType, floorId);
        return getDataTable(list);
    }

    /**
     * 获取生活热水监控公共信息列表
     * @param systemType
     * @param floorId
     * @return
     */
    @GetMapping("/monitorList")
    public TableDataInfo monitorList(@RequestParam("systemType") String systemType, @RequestParam("floorId") String floorId) {
        List<HotWaterNowDataDTO> list = iCollectionParamsManageService.monitorList(systemType, floorId);
        return getDataTable(list);
    }

    /**
     * 获取生活热水监控操作信息内容
     * @param systemType
     * @param floorId
     * @return
     */
    @GetMapping("/operateList")
    public TableDataInfo operateList(@RequestParam("systemType") String systemType, @RequestParam("floorId") String floorId) {
        List<HotWaterControlDTO> list = iCollectionParamsManageService.operateList(systemType, floorId);
        return getDataTable(list);
    }

    @GetMapping("/pumpListInfo")
    public TableDataInfo list(@RequestParam("registerAddr") String registerAddr,
                              @RequestParam("mtType") String mtType) {
        List<PumpInfoDTO> list = iCollectionParamsManageService.selectPumpListInfo(registerAddr, mtType);
        return getDataTable(list);
    }

    /**
     * 获取水温，压力曲线数据
     * @return
     */
    @GetMapping("/lineData")
    public TableDataInfo lineData(@RequestParam("registerAddr") String registerAddr,
                                  @RequestParam("mtType") String mtType) {
        List<ChillersEntity> list = iCollectionParamsManageService.lineData(registerAddr, mtType);
        return getDataTable(list);
    }

}
