package com.mh.web.controller.monitor;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.dto.HouseGroupDTO;
import com.mh.common.core.domain.dto.IndoorTempDTO;
import com.mh.common.core.domain.dto.TempHisRequest;
import com.mh.common.core.domain.vo.IndoorTempVO;
import com.mh.common.core.page.TableDataInfo;
import com.mh.system.service.device.IIndoorTempMonitorService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 室内温度监测
 * @date 2025-02-18 15:43:47
 */
@RestController
@RequestMapping("/device/indoor")
public class IndoorTempMonitorController extends BaseController {

    private final IIndoorTempMonitorService indoorTempMonitorService;

    @Autowired
    public IndoorTempMonitorController(IIndoorTempMonitorService indoorTempMonitorService) {
        this.indoorTempMonitorService = indoorTempMonitorService;
    }

    /**
     * 根据楼层查询每一个房间的温度
     */
    @GetMapping("/tempList")
    public TableDataInfo getIndoorTempByFloor(@RequestParam(value = "systemType") String systemType,
                                              @RequestParam(value = "floorId") String floorId) throws JsonProcessingException {
        List<HouseGroupDTO> indoorTempByFloor = indoorTempMonitorService.getIndoorTempByFloor(systemType, floorId);
        return getDataTable(indoorTempByFloor);
    }

    @PostMapping("/tempHis")
    public TableDataInfo getIndoorTempHisByHouseId(@RequestBody TempHisRequest tempHisRequest) {
        List<IndoorTempDTO> indoorTempHisByHouseId = indoorTempMonitorService.getIndoorTempHisByHouseId(tempHisRequest);
        return getDataTable(indoorTempHisByHouseId);
    }

}
