package com.mh.web.controller.mqtt;

import com.mh.framework.mqtt.service.IMqttMsgSenderService;
import com.mh.framework.mqtt.service.IMqttTopicService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description 主题消息类
 * @date 2024-10-30 15:05:24
 */
@RestController
@RequestMapping("/topic")
public class MqttTopicController {

    @Autowired
    private IMqttTopicService mqttTopicService;

    @Autowired
    private IMqttMsgSenderService mqttMsgSenderService;

    @GetMapping("/add")
    public String add(String topic) {
        mqttTopicService.subscribe(topic);
        return "添加主题：" + topic + "成功";
    }

    @GetMapping("/del")
    public String del(String topic) {
        mqttTopicService.unsubscribe(topic);
        return "删除主题：" + topic + "成功";
    }

    @GetMapping("/list")
    public String list() {
        return "已订阅的主题：" + String.join(",", mqttTopicService.getSubscribedTopics());
    }

    @GetMapping("/publish")
    public String publish(String topic, String pushMessage) {
        String[] subscribedTopics = mqttTopicService.getSubscribedTopics();
        boolean exists = Arrays.stream(subscribedTopics).anyMatch(val -> val.equals(topic));
        if (!exists) {
            return "主题不存在";
        }
        try {
            mqttMsgSenderService.publish(topic, pushMessage);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return "发布主题：" + topic + "成功";
    }

    @GetMapping("/reply")
    public String reply(String topic, String pushMessage) {
        mqttMsgSenderService.publish(topic, pushMessage);
        return "回复主题：" + topic + "成功";
    }

}
