package com.mh.web.controller.space;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.AreaInfo;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.space.IAreaInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 区域管理
 * @date 2025-02-19 16:35:22
 */
@RestController
@RequestMapping("/space/area")
public class AreaInfoController extends BaseController {

    @Autowired
    private IAreaInfoService areaInfoService;

    /**
     * 获取区域管理列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('space:area:list')")
    @GetMapping("/list")
    public TableDataInfo list(AreaInfo areaInfo)
    {
        startPage();
        List<AreaInfo> list = areaInfoService.selectAreaInfoList(areaInfo);
        return getDataTable(list);
    }

    /**
     * 根据区域管理id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('space:area:query')")
    @GetMapping(value = "/{areaId}")
    public AjaxResult getInfo(@PathVariable String areaId)
    {
        return success(areaInfoService.selectAreaInfoById(areaId));
    }

    /**
     * 新增区域管理
     */
    @PreAuthorize("@ss.hasPermi('space:area:add')")
    @Log(title = "区域管理管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody AreaInfo AreaInfo)
    {
        AreaInfo.setCreateBy(getUsername());
        return toAjax(areaInfoService.insertAreaInfo(AreaInfo));
    }

    /**
     * 修改区域管理信息
     */
    @PreAuthorize("@ss.hasPermi('space:area:edit')")
    @Log(title = "区域管理管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody AreaInfo AreaInfo)
    {
        AreaInfo.setUpdateBy(getUsername());
        return toAjax(areaInfoService.updateAreaInfo(AreaInfo));
    }

    /**
     * 删除区域管理管理
     */
    @PreAuthorize("@ss.hasPermi('space:area:remove')")
    @Log(title = "区域管理管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{areaIds}")
    public AjaxResult remove(@PathVariable String[] areaIds)
    {
        return toAjax(areaInfoService.deleteAreaInfoByIds(areaIds));
    }

}
