package com.mh.web.controller.space;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.BuildingInfo;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.space.IBuildingInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 楼栋管理
 * @date 2025-02-19 16:35:22
 */
@RestController
@RequestMapping("/space/building")
public class BuildingInfoController extends BaseController {

    @Autowired
    private IBuildingInfoService buildingInfoService;

    /**
     * 获取楼栋管理列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('space:building:list')")
    @GetMapping("/list")
    public TableDataInfo list(BuildingInfo buildingInfo)
    {
        startPage();
        List<BuildingInfo> list = buildingInfoService.selectBuildingInfoList(buildingInfo);
        return getDataTable(list);
    }

    /**
     * 根据楼栋管理id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('space:building:query')")
    @GetMapping(value = "/{buildingId}")
    public AjaxResult getInfo(@PathVariable String buildingId)
    {
        return success(buildingInfoService.selectBuildingInfoById(buildingId));
    }

    /**
     * 新增楼栋管理
     */
    @PreAuthorize("@ss.hasPermi('space:building:add')")
    @Log(title = "楼栋管理管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody BuildingInfo BuildingInfo)
    {
        BuildingInfo.setCreateBy(getUsername());
        return toAjax(buildingInfoService.insertBuildingInfo(BuildingInfo));
    }

    /**
     * 修改楼栋管理信息
     */
    @PreAuthorize("@ss.hasPermi('space:building:edit')")
    @Log(title = "楼栋管理管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody BuildingInfo BuildingInfo)
    {
        BuildingInfo.setUpdateBy(getUsername());
        return toAjax(buildingInfoService.updateBuildingInfo(BuildingInfo));
    }

    /**
     * 删除楼栋管理管理
     */
    @PreAuthorize("@ss.hasPermi('space:building:remove')")
    @Log(title = "楼栋管理管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{buildingIds}")
    public AjaxResult remove(@PathVariable String[] buildingIds)
    {
        return toAjax(buildingInfoService.deleteBuildingInfoByIds(buildingIds));
    }

}
