package com.mh.web.controller.space;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.CpmSpaceRelation;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.space.ICpmSpaceRelationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 采集参数和区域、楼栋、楼层、房间关联控制类
 * @date 2025-02-20 15:13:10
 */
@RestController
@RequestMapping("/space/relation")
public class CpmSpaceRelationController extends BaseController {

    private final ICpmSpaceRelationService cpmSpaceRelationService;

    @Autowired
    public CpmSpaceRelationController(ICpmSpaceRelationService cpmSpaceRelationService) {
        this.cpmSpaceRelationService = cpmSpaceRelationService;
    }

    /**
     * 获取采集参数、区域等关联管理列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('space:cpmSpaceRelation:list')")
    @GetMapping("/list")
    public TableDataInfo list(CpmSpaceRelation cpmSpaceRelation)
    {
        startPage();
        List<CollectionParamsManage> list = cpmSpaceRelationService.selectCpmSpaceRelationList(cpmSpaceRelation);
        return getDataTable(list);
    }

    /**
     * 根据区域管理id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('space:cpmSpaceRelation:query')")
    @GetMapping(value = "/{cpmSpaceRelationId}")
    public AjaxResult getInfo(@PathVariable String cpmSpaceRelationId)
    {
        return success(cpmSpaceRelationService.selectCpmSpaceRelationById(cpmSpaceRelationId));
    }

    /**
     * 新增采集参数、区域等关联管理
     */
    @PreAuthorize("@ss.hasPermi('space:cpmSpaceRelation:add')")
    @Log(title = "采集参数、区域等关联管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody CpmSpaceRelation CpmSpaceRelation)
    {
        CpmSpaceRelation.setCreateBy(getUsername());
        return toAjax(cpmSpaceRelationService.insertCpmSpaceRelation(CpmSpaceRelation));
    }

    /**
     * 修改采集参数、区域等关联管理
     */
    @PreAuthorize("@ss.hasPermi('space:cpmSpaceRelation:edit')")
    @Log(title = "采集参数、区域等关联管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody CpmSpaceRelation CpmSpaceRelation)
    {
        CpmSpaceRelation.setUpdateBy(getUsername());
        return toAjax(cpmSpaceRelationService.updateCpmSpaceRelation(CpmSpaceRelation));
    }

    /**
     * 删除采集参数、区域等关联管理
     */
    @PreAuthorize("@ss.hasPermi('space:cpmSpaceRelation:remove')")
    @Log(title = "采集参数、区域等关联管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{cpmSpaceRelationIds}")
    public AjaxResult remove(@PathVariable String[] cpmSpaceRelationIds)
    {
        return toAjax(cpmSpaceRelationService.deleteCpmSpaceRelationByIds(cpmSpaceRelationIds));
    }


}
