package com.mh.web.controller.space;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.FloorInfo;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.space.IFloorInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 楼层管理
 * @date 2025-02-19 16:35:22
 */
@RestController
@RequestMapping("/space/floor")
public class FloorInfoController extends BaseController {

    @Autowired
    private IFloorInfoService floorInfoService;

    /**
     * 获取楼层管理列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('space:floor:list')")
    @GetMapping("/list")
    public TableDataInfo list(FloorInfo floorInfo)
    {
        startPage();
        List<FloorInfo> list = floorInfoService.selectFloorInfoList(floorInfo);
        return getDataTable(list);
    }

    /**
     * 根据楼层管理id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('space:floor:query')")
    @GetMapping(value = "/{floorId}")
    public AjaxResult getInfo(@PathVariable String floorId)
    {
        return success(floorInfoService.selectFloorInfoById(floorId));
    }

    /**
     * 新增楼层管理
     */
    @PreAuthorize("@ss.hasPermi('space:floor:add')")
    @Log(title = "楼层管理管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody FloorInfo FloorInfo)
    {
        FloorInfo.setCreateBy(getUsername());
        return toAjax(floorInfoService.insertFloorInfo(FloorInfo));
    }

    /**
     * 修改楼层管理信息
     */
    @PreAuthorize("@ss.hasPermi('space:floor:edit')")
    @Log(title = "楼层管理管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody FloorInfo FloorInfo)
    {
        FloorInfo.setUpdateBy(getUsername());
        return toAjax(floorInfoService.updateFloorInfo(FloorInfo));
    }

    /**
     * 删除楼层管理管理
     */
    @PreAuthorize("@ss.hasPermi('space:floor:remove')")
    @Log(title = "楼层管理管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{floorIds}")
    public AjaxResult remove(@PathVariable String[] floorIds)
    {
        return toAjax(floorInfoService.deleteFloorInfoByIds(floorIds));
    }

}
