package com.mh.web.controller.space;

import com.mh.common.annotation.Log;
import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.HouseInfo;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.enums.BusinessType;
import com.mh.system.service.space.IHouseInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 房间管理
 * @date 2025-02-19 16:35:22
 */
@RestController
@RequestMapping("/space/house")
public class HouseInfoController extends BaseController {

    @Autowired
    private IHouseInfoService houseInfoService;

    /**
     * 获取房间列表内容数据
     */
    @PreAuthorize("@ss.hasPermi('space:house:list')")
    @GetMapping("/list")
    public TableDataInfo list(HouseInfo houseInfo)
    {
        startPage();
        List<HouseInfo> list = houseInfoService.selectHouseInfoList(houseInfo);
        return getDataTable(list);
    }

    /**
     * 根据房间id获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('space:house:query')")
    @GetMapping(value = "/{houseId}")
    public AjaxResult getInfo(@PathVariable String houseId)
    {
        return success(houseInfoService.selectHouseInfoById(houseId));
    }

    /**
     * 新增房间
     */
    @PreAuthorize("@ss.hasPermi('space:house:add')")
    @Log(title = "房间信息管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody HouseInfo HouseInfo)
    {
        HouseInfo.setCreateBy(getUsername());
        return toAjax(houseInfoService.insertHouseInfo(HouseInfo));
    }

    /**
     * 修改房间信息
     */
    @PreAuthorize("@ss.hasPermi('space:house:edit')")
    @Log(title = "房间信息管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody HouseInfo HouseInfo)
    {
        HouseInfo.setUpdateBy(getUsername());
        return toAjax(houseInfoService.updateHouseInfo(HouseInfo));
    }

    /**
     * 删除房间信息管理
     */
    @PreAuthorize("@ss.hasPermi('space:house:remove')")
    @Log(title = "房间信息管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{houseIds}")
    public AjaxResult remove(@PathVariable String[] houseIds)
    {
        return toAjax(houseInfoService.deleteHouseInfoByIds(houseIds));
    }

}
