package com.mh.common.core.domain;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.mh.common.constant.UserConstants;
import com.mh.common.core.domain.entity.SysDept;
import com.mh.common.core.domain.entity.SysMenu;
import com.mh.common.utils.StringUtils;

import java.io.Serializable;
import java.util.List;
import java.util.stream.Collectors;

/**
 * Treeselect树结构实体类
 * 
 * @author mh
 */
public class SpaceTreeSelect implements Serializable
{
    private static final long serialVersionUID = 1L;

    /** 节点ID */
    private String id;

    /** 节点名称 */
    private String label;

    /**
     * 节点类型（0：区域 1：楼栋 2：楼层 3：房屋）
     */
    private int nodeType;

    /** 节点禁用 */
    private boolean disabled = false;

    /** 子节点 */
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private List<SpaceTreeSelect> children;

    // 用于构建关系的临时字段（不序列化）
    @JsonIgnore
    private String parentId;

    public SpaceTreeSelect()
    {

    }

    public SpaceTreeSelect(String id, String houseName, boolean b, String floorId, int nodeType) {
        this.id = id;
        this.label = houseName;
        this.disabled = b;
        this.parentId = floorId;
        this.nodeType = nodeType;
    }

    public int getNodeType() {
        return nodeType;
    }

    public void setNodeType(int nodeType) {
        this.nodeType = nodeType;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getParentId() {
        return parentId;
    }

    public void setParentId(String parentId) {
        this.parentId = parentId;
    }

    public String getLabel()
    {
        return label;
    }

    public void setLabel(String label)
    {
        this.label = label;
    }

    public boolean isDisabled()
    {
        return disabled;
    }

    public void setDisabled(boolean disabled)
    {
        this.disabled = disabled;
    }

    public List<SpaceTreeSelect> getChildren()
    {
        return children;
    }

    public void setChildren(List<SpaceTreeSelect> children)
    {
        this.children = children;
    }

}
