package com.mh.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.mh.common.core.domain.BaseEntity;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 楼栋信息
 * @date 2025-02-19 15:49:33
 */
@TableName("building_info")
public class BuildingInfo extends BaseEntity {

    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    private String id; // 楼栋唯一标识符，作为主键使用。
    private String buildingName; // 楼栋名称，用于标识具体的楼栋。
    private String parentId; // 父级楼栋id，默认值为"0"，表示顶级楼栋。
    private Integer levelsCount; // 楼层数量，表示该楼栋有多少层。
    private Integer beginLevel; // 起始楼层号，表示楼栋的第一层编号。
    private Integer houseCount; // 每层房间数，记录每层楼有多少个房间。
    private Integer bedCount; // 床位总数，记录楼栋内的总床位数量。
    private Integer checkInCount; // 实际入住数，记录当前已入住的人数或床位数。
    private String areaId; // 区域id，标识楼栋所属的地理或管理区域。
    private Integer orderNum; // 排序号，用于定义楼栋显示顺序。
    private BigDecimal tankHeight; // 水箱高度，记录楼栋内水箱的高度信息。
    private Integer pumpCount; // 水泵数量，记录楼栋内安装的水泵数量。
    private BigDecimal lowTankHeight; // 低区水箱高度，记录低区水箱的具体高度。

    @JsonIgnore
    @TableField(exist = false)
    private String searchValue;

    /** 请求参数 */
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params;

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getBuildingName() {
        return buildingName;
    }

    public void setBuildingName(String buildingName) {
        this.buildingName = buildingName;
    }

    public String getParentId() {
        return parentId;
    }

    public void setParentId(String parentId) {
        this.parentId = parentId;
    }

    public Integer getLevelsCount() {
        return levelsCount;
    }

    public void setLevelsCount(Integer levelsCount) {
        this.levelsCount = levelsCount;
    }

    public Integer getBeginLevel() {
        return beginLevel;
    }

    public void setBeginLevel(Integer beginLevel) {
        this.beginLevel = beginLevel;
    }

    public Integer getHouseCount() {
        return houseCount;
    }

    public void setHouseCount(Integer houseCount) {
        this.houseCount = houseCount;
    }

    public Integer getBedCount() {
        return bedCount;
    }

    public void setBedCount(Integer bedCount) {
        this.bedCount = bedCount;
    }

    public Integer getCheckInCount() {
        return checkInCount;
    }

    public void setCheckInCount(Integer checkInCount) {
        this.checkInCount = checkInCount;
    }

    public String getAreaId() {
        return areaId;
    }

    public void setAreaId(String areaId) {
        this.areaId = areaId;
    }

    public Integer getOrderNum() {
        return orderNum;
    }

    public void setOrderNum(Integer orderNum) {
        this.orderNum = orderNum;
    }

    public BigDecimal getTankHeight() {
        return tankHeight;
    }

    public void setTankHeight(BigDecimal tankHeight) {
        this.tankHeight = tankHeight;
    }

    public Integer getPumpCount() {
        return pumpCount;
    }

    public void setPumpCount(Integer pumpCount) {
        this.pumpCount = pumpCount;
    }

    public BigDecimal getLowTankHeight() {
        return lowTankHeight;
    }

    public void setLowTankHeight(BigDecimal lowTankHeight) {
        this.lowTankHeight = lowTankHeight;
    }

    @Override
    public String getSearchValue() {
        return searchValue;
    }

    @Override
    public void setSearchValue(String searchValue) {
        this.searchValue = searchValue;
    }

    @Override
    public Map<String, Object> getParams() {
        return params;
    }

    @Override
    public void setParams(Map<String, Object> params) {
        this.params = params;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", id)
                .append("buildingName", buildingName)
                .append("parentId", parentId)
                .append("levelsCount", levelsCount)
                .append("beginLevel", beginLevel)
                .append("houseCount", houseCount)
                .append("bedCount", bedCount)
                .append("checkInCount", checkInCount)
                .append("areaId", areaId)
                .append("orderNum", orderNum)
                .append("tankHeight", tankHeight)
                .append("pumpCount", pumpCount)
                .append("lowTankHeight", lowTankHeight)
                .append("searchValue", searchValue)
                .append("params", params)
                .toString();
    }
}
