package com.mh.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.mh.common.core.domain.BaseEntity;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.util.Date;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备二维码维护管理
 * @date 2025-01-09 16:42:51
 */
@TableName("device_ledger")
public class DeviceLedger extends BaseEntity {

    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    private String id;

    /**
     * 设备名称
     */
    private String deviceName;

    /**
     * 设备规格型号
     */
    private String modelSpecs;

    /**
     * 设备资产编号
     */
    private String assetNum;

    /**
     * 设备二维码id
     */
    private String qrCodeId;

    /**
     * 设备的预计使用寿命
     */
    private String serviceLife;

    /**
     * 设备所属类型（字典值获取）
     */
    private String deviceType;

    /**
     * 设备的技术参数描述
     */
    private String techParams;

    /**
     * 设备相关的技术文档链接或文件名
     */
    private String techDoc;

    /**
     * 设备的安装日期和时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date installTime;

    /**
     * 二维码状态：0：已绑定，1：未绑定
     */
    private Integer status;

    /**
     * 是否是需要采集（0：是，1：不采集）
     */
    private Integer isCollection;

    /**
     * 是否计算能效(0:计算，1不计算)
     */
    private Integer isCalcEnergy;

    @JsonIgnore
    @TableField(exist = false)
    private String searchValue;

    /** 备注 */
    @TableField(exist = false)
    private String remark;

    /** 请求参数 */
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params;

    /**
     * 系统类型
     */
    private String systemType;

    /**
     * 排序
     */
    private Integer orderNum;

    public Integer getOrderNum() {
        return orderNum;
    }

    public void setOrderNum(Integer orderNum) {
        this.orderNum = orderNum;
    }

    public String getSystemType() {
        return systemType;
    }

    public void setSystemType(String systemType) {
        this.systemType = systemType;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getDeviceName() {
        return deviceName;
    }

    public void setDeviceName(String deviceName) {
        this.deviceName = deviceName;
    }

    public String getModelSpecs() {
        return modelSpecs;
    }

    public void setModelSpecs(String modelSpecs) {
        this.modelSpecs = modelSpecs;
    }

    public String getAssetNum() {
        return assetNum;
    }

    public void setAssetNum(String assetNum) {
        this.assetNum = assetNum;
    }

    public String getQrCodeId() {
        return qrCodeId;
    }

    public void setQrCodeId(String qrCodeId) {
        this.qrCodeId = qrCodeId;
    }

    public String getServiceLife() {
        return serviceLife;
    }

    public void setServiceLife(String serviceLife) {
        this.serviceLife = serviceLife;
    }

    public String getDeviceType() {
        return deviceType;
    }

    public void setDeviceType(String deviceType) {
        this.deviceType = deviceType;
    }

    public String getTechParams() {
        return techParams;
    }

    public void setTechParams(String techParams) {
        this.techParams = techParams;
    }

    public String getTechDoc() {
        return techDoc;
    }

    public void setTechDoc(String techDoc) {
        this.techDoc = techDoc;
    }

    public Date getInstallTime() {
        return installTime;
    }

    public void setInstallTime(Date installTime) {
        this.installTime = installTime;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getIsCollection() {
        return isCollection;
    }

    public void setIsCollection(Integer isCollection) {
        this.isCollection = isCollection;
    }

    public Integer getIsCalcEnergy() {
        return isCalcEnergy;
    }

    public void setIsCalcEnergy(Integer isCalcEnergy) {
        this.isCalcEnergy = isCalcEnergy;
    }

    @Override
    public String getSearchValue() {
        return searchValue;
    }

    @Override
    public void setSearchValue(String searchValue) {
        this.searchValue = searchValue;
    }

    @Override
    public String getRemark() {
        return remark;
    }

    @Override
    public void setRemark(String remark) {
        this.remark = remark;
    }

    @Override
    public Map<String, Object> getParams() {
        return params;
    }

    @Override
    public void setParams(Map<String, Object> params) {
        this.params = params;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", id)
                .append("deviceName", deviceName)
                .append("modelSpecs", modelSpecs)
                .append("assetNum", assetNum)
                .append("qrCodeId", qrCodeId)
                .append("serviceLife", serviceLife)
                .append("deviceType", deviceType)
                .append("techParams", techParams)
                .append("techDoc", techDoc)
                .append("installTime", installTime)
                .append("status", status)
                .append("isCollection", isCollection)
                .append("isCalcEnergy", isCalcEnergy)
                .append("searchValue", searchValue)
                .append("remark", remark)
                .append("params", params)
                .append("systemType", systemType)
                .append("orderNum", orderNum)
                .toString();
    }
}
