package com.mh.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.mh.common.core.domain.BaseEntity;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 房间信息表
 * @date 2025-02-19 16:13:15
 */
@Setter
@Getter
@TableName("house_info")
public class HouseInfo extends BaseEntity {

    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    private String id;

    /**
     * 区域id
     */
    private String areaId;

    /**
     * 楼栋id
     */
    private String buildingId;

    /**
     * 楼层id
     */
    private String floorId;

    /**
     * 房间名称
     */
    private String houseName;

    /**
     * 房屋详细地址
     */
    private String address;

    /**
     * 房屋价格，保留两位小数
     */
    private BigDecimal price;

    /**
     * 房屋总面积
     */
    private BigDecimal totalArea;

    /**
     * 房屋使用面积（可居住部分）
     */
    private BigDecimal usableArea;

    /**
     * 建造年份
     */
    private String builtYear;

    /**
     * 0:正常，1：禁用
     */
    private Integer status;

    @JsonIgnore
    @TableField(exist = false)
    private String searchValue;

    /**
     * 请求参数
     */
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params;

    private int orderNum;

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", id)
                .append("areaId", areaId)
                .append("buildingId", buildingId)
                .append("floorId", floorId)
                .append("houseName", houseName)
                .append("address", address)
                .append("price", price)
                .append("totalArea", totalArea)
                .append("usableArea", usableArea)
                .append("builtYear", builtYear)
                .append("status", status)
                .append("searchValue", searchValue)
                .append("params", params)
                .toString();
    }

}
