package com.mh.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.mh.common.core.domain.BaseEntity;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.math.BigDecimal;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 备品/备件出入库管理表，用于记录备件的出入库操作及当前库存情况
 * @date 2025-01-09 11:24:02
 */
@TableName("spare_parts_manage")
public class SparePartsManage extends BaseEntity {

    /**
     * id
     */
    @TableId(value = "id", type = IdType.ASSIGN_UUID)
    private String id;

    /**
     * 备件的序列号，用于唯一识别每个备件
     */
    private String serialNum;

    /**
     * 备件的名称，便于快速识别
     */
    private String spareName;

    /**
     * 备件的规格和型号
     */
    private String modelSpecs;

    /**
     * 关联到设备类型的ID，标识备件适用于哪种设备类型
     */
    private String deviceTypeId;

    /**
     * 当前备件的库存数量，在每次出入库操作后更新
     */
    private Integer inventoryLevels;

    /**
     * 最低库存量阈值，当库存低于此值时需要补货
     */
    private Integer minInventoryLevels;

    /**
     * 备件的预计使用年限，单位为年
     */
    private Integer serviceLife;

    /**
     * 备件的单价，保留三位小数
     */
    private BigDecimal unitPrice;

    @JsonIgnore
    @TableField(exist = false)
    private String searchValue;

    /**
     * 备注
     */
    @TableField(exist = false)
    private String remark;

    /**
     * 请求参数
     */
    @TableField(exist = false)
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private Map<String, Object> params;

    @Override
    public String getSearchValue() {
        return searchValue;
    }

    @Override
    public void setSearchValue(String searchValue) {
        this.searchValue = searchValue;
    }

    @Override
    public String getRemark() {
        return remark;
    }

    @Override
    public void setRemark(String remark) {
        this.remark = remark;
    }

    @Override
    public Map<String, Object> getParams() {
        return params;
    }

    @Override
    public void setParams(Map<String, Object> params) {
        this.params = params;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getSerialNum() {
        return serialNum;
    }

    public void setSerialNum(String serialNum) {
        this.serialNum = serialNum;
    }

    public String getSpareName() {
        return spareName;
    }

    public void setSpareName(String spareName) {
        this.spareName = spareName;
    }

    public String getModelSpecs() {
        return modelSpecs;
    }

    public void setModelSpecs(String modelSpecs) {
        this.modelSpecs = modelSpecs;
    }

    public String getDeviceTypeId() {
        return deviceTypeId;
    }

    public void setDeviceTypeId(String deviceTypeId) {
        this.deviceTypeId = deviceTypeId;
    }

    public Integer getInventoryLevels() {
        return inventoryLevels;
    }

    public void setInventoryLevels(Integer inventoryLevels) {
        this.inventoryLevels = inventoryLevels;
    }

    public Integer getMinInventoryLevels() {
        return minInventoryLevels;
    }

    public void setMinInventoryLevels(Integer minInventoryLevels) {
        this.minInventoryLevels = minInventoryLevels;
    }

    public Integer getServiceLife() {
        return serviceLife;
    }

    public void setServiceLife(Integer serviceLife) {
        this.serviceLife = serviceLife;
    }

    public BigDecimal getUnitPrice() {
        return unitPrice;
    }

    public void setUnitPrice(BigDecimal unitPrice) {
        this.unitPrice = unitPrice;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", id)
                .append("serialNum", serialNum)
                .append("spareName", spareName)
                .append("modelSpecs", modelSpecs)
                .append("deviceTypeId", deviceTypeId)
                .append("inventoryLevels", inventoryLevels)
                .append("minInventoryLevels", minInventoryLevels)
                .append("serviceLife", serviceLife)
                .append("unitPrice", unitPrice)
                .append("searchValue", searchValue)
                .append("remark", remark)
                .append("params", params)
                .toString();
    }
}
