package com.mh.common.core.domain.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.util.Date;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 存储天气信息表数据
 * @date 2025-03-30 10:54:46
 */
@Setter
@Getter
@TableName("weather_data")
public class WeatherData {

    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 省份名称，如"广东省"
     */
    private String province;

    /**
     * 城市名称，如"广州市"
     */
    private String city;

    /**
     * 行政区划代码，如"440100"
     */
    private String adCode;

    /**
     * 天气现象，如"晴"、"多云"、"小雨"等
     */
    private String weather;

    /**
     * 温度，单位：摄氏度
     */
    private Integer temperature;

    /**
     * 相对湿度，单位：百分比(0-100)
     */
    private Integer humidity;

    /**
     * 风向描述，如"东北风"、"西南风"
     */
    private String windDirection;

    /**
     * 风力描述，如"3-4级"、"5级"
     */
    private String windPower;

    /**
     * 天气数据报告时间，记录数据采集的具体时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date reportTime;

    /**
     * 记录创建时间，自动设置为当前时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", id)
                .append("province", province)
                .append("city", city)
                .append("adCode", adCode)
                .append("weather", weather)
                .append("temperature", temperature)
                .append("humidity", humidity)
                .append("windDirection", windDirection)
                .append("windPower", windPower)
                .append("reportTime", reportTime)
                .append("createdTime", createdTime)
                .toString();
    }
}
