package com.mh.framework.aspectj;

import com.alibaba.fastjson2.JSON;
import com.mh.common.annotation.ControlDeviceAno;
import com.mh.common.core.domain.entity.DeviceControlLog;
import com.mh.common.core.domain.entity.OrderEntity;
import com.mh.common.enums.ControlTypeEnum;
import com.mh.common.utils.SecurityUtils;
import com.mh.system.service.device.IDeviceControlLogService;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.Method;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 设备控制详情内容
 * @date 2023/8/8 11:55:54
 */
@Slf4j
@Aspect
@Component
public class ControlDeviceAspect {

    @Autowired
    private IDeviceControlLogService deviceControlLogService;

    @Pointcut(value = "execution(* com.mh.*.controller..*(..)) && @annotation(com.mh.common.annotation.ControlDeviceAno)")
    public void executionMethod(){}

    @Pointcut(value = "execution(* com.mh.*.controller..*(..)) && @annotation(com.mh.common.annotation.ControlDeviceAno)")
    public void executionNote(){}

    @Before("executionMethod()")
    public void controlBefore(JoinPoint joinPoint) {
        log.info("前置通知");
        Object[] args = joinPoint.getArgs();
        for (Object arg : args) {
            log.info("获取到对应的参数==>{}",arg);
        }
    }

    @Around(value = "execution(* com.mh.*.controller..*(..)) && @annotation(com.mh.common.annotation.ControlDeviceAno)")
    public Object controlBeforeNote(ProceedingJoinPoint joinPoint) throws Throwable {
        // 通知签名
        Signature signature = joinPoint.getSignature();
        MethodSignature msg = (MethodSignature) signature;
        Object target = joinPoint.getTarget();
        // 获取注解标注的方法
        Method method = target.getClass().getMethod(msg.getName(), msg.getParameterTypes());
        // 通过方法获取注解
        ControlDeviceAno annotation = method.getAnnotation(ControlDeviceAno.class);
//        Object proceed;
        // 获取参数
        Object[] args = joinPoint.getArgs();
        log.info("注解方法标注值=={}", annotation.value());
        log.info("注解方法标注=={}", annotation.isAop());
        for (Object arg : args) {
            log.info("方法内的参数==>{}", arg);
            if (Objects.isNull(arg)) {
                continue;
            }
            String string = JSON.toJSON(arg).toString();
            List<OrderEntity> collect = JSON.parseArray(string, OrderEntity.class);
            if (!CollectionUtils.isEmpty(collect)) {
                for (OrderEntity orderEntity : collect) {
//                    DeviceMessageDTO deviceMessageDTO = deviceDisplayService.queryDeviceStatusByDeviceId(String.valueOf(orderEntity.getId()));
                    DeviceControlLog deviceControlLog = new DeviceControlLog();
//                    if (null != deviceMessageDTO) {
//                        deviceControlLogEntity.setDeviceName(deviceMessageDTO.getLabel());
//                    }
                    deviceControlLog.setDeviceName(orderEntity.getOtherName());
                    deviceControlLog.setControlType(ControlTypeEnum.getByType(orderEntity.getType()));
                    // 根据id查询对应的注册地址信息
                    if (null != orderEntity.getType() && orderEntity.getType() == 1) {
                        // 修改开关状态
                        deviceControlLog.setControlContent(orderEntity.getParam().equals("0")?"关闭":"开启");
                    } else if (null != orderEntity.getType() && orderEntity.getType() == 0) {
                        // 修改频率
                        deviceControlLog.setControlContent(orderEntity.getParam());
                    } else if (null != orderEntity.getType() && orderEntity.getType() == 3) {
                        // 修改手动获取全自动
                        deviceControlLog.setControlContent(orderEntity.getParam().equals("0")?"关闭全自动":"开启全自动");
                    }
                    deviceControlLog.setCreateUser(SecurityUtils.getUsername());
                    deviceControlLog.setCreateTime(new Date());
                    deviceControlLogService.insertData(deviceControlLog);
                }
            }
        }
//        if (!annotation.isAop()) {
//            log.info("你无需处理当前注解内容");
//            proceed = joinPoint.proceed();
//        } else {
//            log.info("进入aop判断");
//            proceed = joinPoint.proceed();
//
//        }
        return joinPoint.proceed();
    }

}
