package com.mh.framework.dealdata;

import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceReport;
import com.mh.common.model.request.AdvantechReceiver;
import com.mh.common.model.request.OneTwoThreeTempData;

import java.util.List;
import java.util.Map;

/**
 * 数据解析，处理，入库操作
 * @Author : Rainbow
 * @date : 2023/5/26
 */
public interface DataProcessService {
    /**
     * 主机数据解析入库
     * @param data 未解析数据
     */
    void insertChillerData(AdvantechReceiver data);

    /**
     * 其他数据解析入库
     * @param data 未解析数据
     */
    void insertOtherData(AdvantechReceiver data);

    /**
     * 电表等其他设备数据入库
     * @param data
     */
    void insertDeviceData(AdvantechReceiver data);

    /**
     * 查询初始值
     * @param deviceNum
     * @return
     */
    String queryInitValue(String deviceNum);

    /**
     * 查询上一次采集数据、时间等参数
     * @param deviceNum
     * @param type min hour day month year
     * @return
     */
    DeviceReport queryLastValue(String deviceNum, String type);

    /**
     * 批量插入data_min表中
     * @param dataMinList
     */
    void insertDatabase(List<DeviceReport> dataMinList);

    /**
     * 查询data_ + type + year 表中未处理的数据
     * @param type min hour day month year
     * @return
     */
    Map<String, Object> queryUntreatedData(String type);

    /**
     * 批量插入报表中
     * @param hourList 报表数据
     * @param type 报表类型
     */
    void batchInsertTable(List<DeviceReport> hourList, String type);


    /**
     * 根据Id批量修改grade = 1
     * 需要根据IdMap来判断修改哪个表的id
     * @param idMap 主键ID集合
     * @param type 报表类型
     */
    void batchUpdateGrade(Map<String, List<Long>> idMap, String type);

    /**
     * 批量插入或者更新
     * @param dataList 数据集
     * @param tableType
     */
    void batchInsertOrUpdate(List<DeviceReport> dataList, String tableType);

    /**
     * 查询倍率
     * @param deviceNum
     * @return
     */
    Integer queryRatio(String deviceNum);

    /**
     * 插入主机历史流水表
     * @param cacheList
     */
    void insertChillerReport(List<CollectionParamsManage> cacheList);

    /**
     * 批量更新运行时长
     * @param runTimeList
     */
    void batchUpdateRunTime(List<Map<String, String>> runTimeList);

    /**
     * 计算COP
     * @param time
     */
    void calculateCopByTime(String time);

    /**
     * 插入温度数据
     * @param oneTwoThreeTempData
     */
    void insertTempData(OneTwoThreeTempData oneTwoThreeTempData);
}
