package com.mh.framework.mqtt.handler;

import com.mh.common.config.MHConfig;
import com.mh.common.constant.ChannelName;
import com.mh.common.constant.TopicEnum;
import com.mh.common.utils.spring.SpringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.annotation.Router;
import org.springframework.integration.mqtt.support.MqttHeaders;
import org.springframework.integration.router.AbstractMessageRouter;
import org.springframework.messaging.Message;
import org.springframework.messaging.MessageChannel;
import org.springframework.messaging.MessageHeaders;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.Collections;
import java.util.Objects;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description 入站消息路由分发中心
 * @date 2024-10-29 17:04:17
 */
@Slf4j
@Component
public class InboundMessageRouter extends AbstractMessageRouter {

    /** 系统基础配置 */
    @Autowired
    private MHConfig mHConfig;

    /**
     * 目前只需要这个方式，后期在拓展使用IntegrationFlow方式
     * @param message
     * @return
     */
    @Router(inputChannel = ChannelName.INBOUND)
    @Override
    protected Collection<MessageChannel> determineTargetChannels(Message<?> message) {
        MessageHeaders headers = message.getHeaders();
        String topic = Objects.requireNonNull(headers.get(MqttHeaders.RECEIVED_TOPIC)).toString();
//        byte[] payload = (byte[]) message.getPayload();
//        log.info("从当前主题 topic: {}, 接收到的消息：{}", topic, new String(payload));
        // 判断当前主题是否是当前项目的,温湿度目前写死的
        if (!topic.startsWith(mHConfig.getName()) && !topic.contains("/temp")) {
            log.info("当前主题 topic: {} 不是当前项目的，直接丢弃", topic);
            return Collections.singleton(SpringUtils.getBean(ChannelName.DEFAULT_BOUND));
        }
        // 找到对应的主题消息通道
        if (topic.contains("/temp")) {
            return Collections.singleton(SpringUtils.getBean(ChannelName.EVENTS_UPLOAD_INBOUND));
        } else {
            TopicEnum topicEnum = TopicEnum.find(mHConfig.getName() + "/", topic);
            MessageChannel bean = SpringUtils.getBean(topicEnum.getBeanName());
            return Collections.singleton(bean);
        }
    }
}
