package com.mh.framework.mqtt.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.mh.common.model.response.CommonTopicResponse;
import com.mh.common.model.response.ServiceReply;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description Mqtt发送消息
 * @date 2024-10-29 17:31:40
 */
public interface IMqttMsgSenderService {

    void publish(String topic, String pushMessage);

    /**
     * 发布消息
     * @param topic    target
     * @param response message
     */
    void publish(String topic, CommonTopicResponse response);

    /**
     * 使用qos发布消息
     *
     * @param topic    target
     * @param qos      qos
     * @param response message
     */
    void publish(String topic, int qos, CommonTopicResponse response);

    /**
     * 发送消息并同时接收响应
     * @param clazz
     * @param topic
     * @param response 通知启动是否成功
     * @return
     */
    <T> T publishWithReply(Class<T> clazz, String topic, CommonTopicResponse response);

    /**
     * 发送消息并同时接收响应
     * @param clazz
     * @param topic
     * @param response
     * @param retryTime
     * @param <T>
     * @return
     */
    <T> T publishWithReply(Class<T> clazz, String topic, CommonTopicResponse response, int retryTime);

    /**
     * 专门用于发送服务消息
     * @param clazz  The generic class for ServiceReply.
     * @param sn
     * @param method
     * @param data
     * @param bid
     * @param <T>
     * @return
     */
    <T> ServiceReply<T> publishServicesTopic(TypeReference<T> clazz, String sn, String method, Object data, String bid);

    /**
     * 仅用于为服务发送消息，不设置接收到的子类型
     * @param sn
     * @param method
     * @param data
     * @param bid
     * @return
     */
    ServiceReply publishServicesTopic(String sn, String method, Object data, String bid);

    /**
     * 专门用于发送服务消息
     * @param clazz  The generic class for ServiceReply.
     * @param sn
     * @param method
     * @param data
     * @param <T>
     * @return
     */
    <T> ServiceReply<T> publishServicesTopic(TypeReference<T> clazz, String sn, String method, Object data);

    /**
     * 仅用于为服务发送消息，不设置接收到的子类型
     * @param sn
     * @param method
     * @param data
     * @return
     */
    ServiceReply publishServicesTopic(String sn, String method, Object data);

}
