package com.mh.framework.mqtt.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.mh.common.constant.ChannelName;
import com.mh.common.constant.Constants;
import com.mh.common.model.request.AdvantechDatas;
import com.mh.common.model.request.AdvantechReceiver;
import com.mh.common.model.request.OneTwoThreeTempData;
import com.mh.framework.mqtt.service.IEventsService;
import com.mh.framework.rabbitmq.producer.SendMsgByTopic;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.annotation.ServiceActivator;
import org.springframework.messaging.MessageHeaders;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.Objects;

/**
 * @author LJF
 * @version 1.0
 * @project springboot-mqtt-demo
 * @description 通道消息处理
 * @date 2024-11-05 11:42:30
 */
@Slf4j
@Service
public class EventsServiceImpl implements IEventsService {

    @Autowired
    private ObjectMapper mapper;

    @Autowired
    private SendMsgByTopic sendMsgByTopic;

    @ServiceActivator(inputChannel = ChannelName.EVENTS_UPLOAD_INBOUND)
    @Override
    public void handleInboundUpload(byte[] receiver, MessageHeaders headers) {
        // 获取当前的主题
        String topic = Objects.requireNonNull(headers.get("mqtt_receivedTopic")).toString();
        handleInboundData(receiver, topic, "主动上报数据");
    }

    @ServiceActivator(inputChannel = ChannelName.EVENTS_COLLECTION_INBOUND)
    @Override
    public void handleInboundCollection(byte[] receiver, MessageHeaders headers) {
        // 获取当前的主题
        String topic = Objects.requireNonNull(headers.get("mqtt_receivedTopic")).toString();
        handleInboundData(receiver, topic, "主动下发采集数据");
    }

    @ServiceActivator(inputChannel = ChannelName.EVENTS_CONTROL_INBOUND)
    @Override
    public void handleInboundControl(byte[] receiver, MessageHeaders headers) {
        // 获取当前的主题
        String topic = Objects.requireNonNull(headers.get("mqtt_receivedTopic")).toString();
        handleInboundData(receiver, topic, "控制指令下发");
    }

    @ServiceActivator(inputChannel = ChannelName.EVENTS_SEND_INBOUND)
    @Override
    public void handleInboundSend(byte[] receiver, MessageHeaders headers) throws IOException {
        String sendStr = new String(receiver, CharsetUtil.UTF_8);
        log.info("接收到控制指令下发=>{}", sendStr);
    }

    private void handleInboundData(byte[] receiver,String topic, String logMessage) {
        try {
            AdvantechReceiver commonTopicReceiver = new AdvantechReceiver();
            if (!topic.contains(Constants.TEMP)) {
                commonTopicReceiver = mapper.readValue(receiver, AdvantechReceiver.class);
                log.info("主题：{}，类型：{}: ，数据：{}", topic, logMessage, commonTopicReceiver.toString());
            }
            // 接入消息队列，利用消息对接进行数据处理
            // 判断当前主题属于哪种主动上报数据
            if (topic.contains(Constants.CHILLERS)) {
                // 主机参数数据
                sendMsgByTopic.sendToChillerMQ(JSONObject.toJSONString(commonTopicReceiver));
            } else if (topic.contains(Constants.OTHER)) {
                // 其他数据
                sendMsgByTopic.sendToOtherMQ(JSONObject.toJSONString(commonTopicReceiver));
            } else if (topic.contains(Constants.DEVICE)) {
                // 设备数据
                sendMsgByTopic.sendToDeviceMQ(JSONObject.toJSONString(commonTopicReceiver));
            } else if (topic.contains(Constants.TEMP)) {
                // 温湿度数据
                OneTwoThreeTempData oneTwoThreeTempData = mapper.readValue(receiver, OneTwoThreeTempData.class);
                log.info("主题：{}，类型：{}: ，数据：{}", topic, logMessage, oneTwoThreeTempData.toString());
                sendMsgByTopic.sendToTempMQ(JSONObject.toJSONString(oneTwoThreeTempData));
            } else {
                // 非本地主题处理
                log.info("非本地主题处理: {}", topic);
            }
        } catch (IOException e) {
            log.error("处理数据时发生错误: ", e);
        }
    }


}
