package com.mh.framework.mqtt.service.impl;

import com.mh.framework.mqtt.service.IMqttGatewayService;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.integration.mqtt.support.MqttHeaders;
import org.springframework.integration.support.MessageBuilder;
import org.springframework.messaging.MessageChannel;
import org.springframework.stereotype.Service;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 网关实现类
 * @date 2025-02-07 08:44:55
 */
@Service
public class MqttGatewayServiceImpl implements IMqttGatewayService {

    private final MessageChannel outboundChannel;

    public MqttGatewayServiceImpl(@Qualifier("outbound") MessageChannel outboundChannel) {
        this.outboundChannel = outboundChannel;
    }

    @Override
    public synchronized void publish(String topic, String payload, int qos) {
        outboundChannel.send(
                MessageBuilder
                        .withPayload(payload)
                        .setHeader(MqttHeaders.TOPIC, topic)
                        .setPriority(qos)
                        .build());
    }

    @Override
    public void publish(String topic, byte[] payload) {
        outboundChannel.send(
                MessageBuilder
                        .withPayload(payload)
                        .setHeader(MqttHeaders.TOPIC, topic)
                        .build());
    }

    @Override
    public void publish(String topic, byte[] payload, int qos) {
        outboundChannel.send(
                MessageBuilder
                        .withPayload(payload)
                        .setHeader(MqttHeaders.TOPIC, topic)
                        .setPriority(qos)
                        .build());
    }
}
