package com.mh.quartz.task;

import com.mh.common.core.domain.entity.AlarmCode;
import com.mh.common.core.domain.entity.AlarmRecords;
import com.mh.common.core.domain.entity.AlarmRules;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.utils.BigDecimalUtils;
import com.mh.common.utils.DateUtils;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.operation.IAlarmCodeService;
import com.mh.system.service.operation.IAlarmRecordsService;
import com.mh.system.service.operation.IAlarmRulesService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;
import java.util.List;


/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 创建报警记录
 * @date 2025-02-21 08:48:48
 */
@Slf4j
@Component("createAlarmTask")
public class CreateAlarmTask {

    private final IAlarmRulesService alarmRulesService;

    private final IAlarmCodeService alarmCodeService;

    private final ICollectionParamsManageService collectionParamsManageService;

    private final IAlarmRecordsService alarmRecordService;

    @Autowired
    public CreateAlarmTask(IAlarmRulesService alarmRulesService, IAlarmCodeService alarmCodeService, ICollectionParamsManageService collectionParamsManageService, IAlarmRecordsService alarmRecordService) {
        this.alarmRulesService = alarmRulesService;
        this.alarmCodeService = alarmCodeService;
        this.collectionParamsManageService = collectionParamsManageService;
        this.alarmRecordService = alarmRecordService;
    }

    public void createAlarmTask() {
        log.info("创建报警记录");
        // 查询仪表报警规则记录，查看哪些规则启用了
        List<AlarmRules> alarmRules = alarmRulesService.selectAlarmRulesListByStatus(0);
        // 循环查询报警规则，判断是否满足报警条件
        for (AlarmRules alarmRule : alarmRules) {
            // 判断报警类型
            if ("0".equals(alarmRule.getAlarmType())) {
                // 当前是越限事件
                // 查询事件类型查询对应的报警模板内容
                AlarmCode alarmCode = alarmCodeService.selectAlarmCodeByAlarmType(alarmRule.getEventType());
                // 获取当前采集参数值
                CollectionParamsManage collectionParamsManage = collectionParamsManageService.selectCollectionParamsManageById(alarmRule.getCpmId());
                // 判断当前值是否是当前事件
                AlarmRecords alarmRecords = new AlarmRecords();
                BigDecimal curValue = collectionParamsManage.getCurValue();
                Date curTime = collectionParamsManage.getCurTime();
                // 阈值
                String threshold1 = alarmRule.getThreshold1();
                if (alarmRule.getTimePeriodSet() == 0 && DateUtils.isSameDay(curTime, new Date())) {
                    // 执行相关操作
                    insertOrUpdateRecord(alarmRule, curValue, threshold1, alarmCode, alarmRecords, collectionParamsManage);
                } else if (alarmRule.getTimePeriodSet() == 1
                        && DateUtils.isSameDay(collectionParamsManage.getCurTime(), new Date())
                        && DateUtils.isCurrentTimeInRange(alarmRule.getBeginTime(), alarmRule.getEndTime(), curTime)
                ) {
                    // 执行相关操作
                    insertOrUpdateRecord(alarmRule, curValue, threshold1, alarmCode, alarmRecords, collectionParamsManage);
                }
            }
        }
    }

    private void insertOrUpdateRecord(AlarmRules alarmRule, BigDecimal curValue, String threshold1, AlarmCode alarmCode, AlarmRecords alarmRecords, CollectionParamsManage collectionParamsManage) {
        boolean compare = BigDecimalUtils.compare(alarmRule.getCondition1(), curValue, new BigDecimal(threshold1));
        if (compare) {
            // 创建报警记录
            String content = alarmCode.getMsgContent();
            content = content.replace("#{curValue}", curValue.setScale(1, RoundingMode.HALF_UP).toString());
            content = content.replace("#{setValue}", alarmRule.getCondition1() + threshold1);
            alarmRecords.setContent(content);
            alarmRecords.setAlarmType(alarmRule.getAlarmType());
            alarmRecords.setEventType(alarmRule.getEventType());
            alarmRecords.setAlarmLevel(alarmRule.getAlarmLevel());
            alarmRecords.setLedgerId(alarmRule.getLedgerId());
            alarmRecords.setCpmId(alarmRule.getCpmId());
            alarmRecords.setDeviceName(alarmRule.getDeviceName());
            alarmRecords.setCpmName(alarmRule.getCpmName());
            alarmRecords.setCreateTime(collectionParamsManage.getCurTime());
            // 判断报警记录是否已经存在
            AlarmRecords isExits = alarmRecordService.selectIsExist(alarmRecords);
            if (isExits == null) {
                alarmRecordService.insertAlarmRecord(alarmRecords);
            } else {
                // 更新报警记录
                isExits.setContent(content);
                isExits.setCreateTime(collectionParamsManage.getCurTime());
                alarmRecordService.updateAlarmRecord(isExits);
            }
        }
    }

}
