package com.mh.quartz.task;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.mh.common.core.domain.entity.SysParams;
import com.mh.common.core.domain.entity.WeatherData;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.StringUtils;
import com.mh.system.service.ISysParamsService;
import com.mh.system.service.IWeatherDataService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 获取天气信息任务
 * @date 2025-03-30 11:31:08
 */
@Slf4j
@Component("getWeatherDataTask")
public class GetWeatherDataTask {

    @Value("${amap.key}")
    String amapKey;

    @Autowired
    private ISysParamsService sysParamsService;

    @Autowired
    private IWeatherDataService weatherDataService;

    public void getWeatherInfo() {
        // 从系统参数中获取对应的项目区域
        List<SysParams> sysParams = sysParamsService.queryList();
        if (null != sysParams && !sysParams.isEmpty()) {
            String url = "https://restapi.amap.com/v3/weather/weatherInfo?key="+amapKey+"&city="+sysParams.get(0).getProArea();
            RestTemplate restTemplate = new RestTemplate();
            String returnResult = restTemplate.getForObject(url, String.class);
            if (!StringUtils.isBlank(returnResult)) {
                JSONObject jsonObject = JSON.parseObject(returnResult);
                //System.out.println(jsonObject);
                if ("1".equals(jsonObject.get("status"))) {
                    try {
                        WeatherData weatherData = new WeatherData();
                        weatherData.setProvince(jsonObject.getJSONArray("lives").getJSONObject(0).getString("province"));
                        weatherData.setCity(jsonObject.getJSONArray("lives").getJSONObject(0).getString("city"));
                        weatherData.setAdCode(jsonObject.getJSONArray("lives").getJSONObject(0).getString("adcode"));
                        weatherData.setWeather(jsonObject.getJSONArray("lives").getJSONObject(0).getString("weather"));
                        weatherData.setTemperature(jsonObject.getJSONArray("lives").getJSONObject(0).getInteger("temperature"));
                        weatherData.setHumidity(jsonObject.getJSONArray("lives").getJSONObject(0).getInteger("humidity"));
                        weatherData.setWindDirection(jsonObject.getJSONArray("lives").getJSONObject(0).getString("winddirection"));
                        weatherData.setWindPower(jsonObject.getJSONArray("lives").getJSONObject(0).getString("windpower"));
                        weatherData.setReportTime(DateUtils.stringToDate(jsonObject.getJSONArray("lives").getJSONObject(0).getString("reporttime"), "yyyy-MM-dd HH:mm:ss"));
                        weatherData.setCreatedTime(new Date());
                        weatherDataService.insertWeatherData(weatherData);
                    } catch (Exception e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }
    }

}
