package com.mh.system.mapper.device;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.common.core.domain.dto.TempHumidityDTO;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.vo.CollectionParamsManageVO;
import com.mh.common.core.domain.vo.DeviceMonitorVO;
import com.mh.common.core.domain.vo.HotWaterControlListVO;
import org.apache.ibatis.annotations.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备采集参数配置mapper
 * @date 2025-01-13 16:34:52
 */
@Mapper
public interface CollectionParamsManageMapper extends BaseMapper<CollectionParamsManage> {

    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "device_rate", property = "deviceRate"),
            @Result(column = "device_caliber", property = "deviceCaliber"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDates"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "create_time", property = "createTime"),
            @Result(column = "update_time", property = "updateTime"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    @Select("<script>" +
            "select * from collection_params_manage" +
            "<where>" +
            "<if test='deviceNum != null'>" +
            "and mt_num = #{deviceNum}" +
            "</if>" +
            "</where>" +
            "</script>")
    List<CollectionParamsManage> getDeviceByOther(@Param("deviceNum") String deviceNum);

    @Select("select " +
            " count(1) " +
            "from " +
            " collection_params_manage cpm  " +
            " join device_ledger dl  " +
            " on cpm.device_ledger_id = dl.id  " +
            " and dl.device_type = #{deviceType} " +
            " and cpm.grade = #{grade} " +
            " and cpm.mt_is_sum = 0")
    int selectSummary(@Param("grade") int grade,
                      @Param("deviceType") String deviceType);

    @Select("select id from collection_params_manage where mt_code = #{registerAddr} and mt_type = #{mtType} limit 1 ")
    String selectIdByMtCodeAndType(@Param("registerAddr") String registerAddr,
                                   @Param("mtType") String mtType);

    @Select("select " +
            " cpm.* " +
            "from " +
            " collection_params_manage cpm " +
            "join cpm_space_relation csr  " +
            "on " +
            " cpm.id = csr.cpm_id " +
            " and csr.house_id = #{houseId} ")
    List<CollectionParamsManage> selectByHouseId(@Param("houseId") String houseId);

    @Select("select " +
            " cpm.* " +
            "from " +
            " collection_params_manage cpm " +
            "join cpm_space_relation csr  " +
            "on " +
            " cpm.id = csr.cpm_id " +
            " and csr.floor_id = #{floorId} ")
    List<CollectionParamsManage> selectByFloorId(@Param("floorId") String floorId);

    @Select("select " +
            " cpm.* " +
            "from " +
            " collection_params_manage cpm " +
            "join cpm_space_relation csr  " +
            "on " +
            " cpm.id = csr.cpm_id " +
            " and csr.building_id = #{buildingId} ")
    List<CollectionParamsManage> selectByBuildingId(@Param("buildingId") String buildingId);

    @Select("<script>" +
            "select " +
            " cpm.id as \"cpmId\"," +
            " hi.id as \"houseId\"," +
            " hi.house_name as \"houseName\", " +
            " hi.order_num as \"orderNum\" " +
            "from " +
            " cpm_space_relation csr " +
            "join collection_params_manage cpm " +
            "on " +
            " csr.cpm_id = cpm.id " +
            "join house_info hi  " +
            "on " +
            " csr.house_id = hi.id " +
            "WHERE " +
            " cpm.mt_type = #{mtType}" +
            " and hi.id IN " +
            " <foreach collection='houseIds' item='item' open='(' separator=',' close=')'>" +
            "   #{item}" +
            " </foreach>" +
            " and cpm.param_type = #{paramType}" +
            " and cpm.system_type = #{systemType}  " +
            "</script>")
    List<Map<String, String>> selectByHouseIdAndMtType(@Param("houseIds") List<String> houseIds,
                                                       @Param("systemType") String systemType,
                                                       @Param("mtType") String mtType,
                                                       @Param("paramType") String paramType);

    @Select("select " +
            " cpm.* " +
            "from " +
            " collection_params_manage cpm " +
            "join cpm_space_relation csr  " +
            "on " +
            " cpm.id = csr.cpm_id " +
            " and csr.area_id = #{areaId} ")
    List<CollectionParamsManage> selectByAreaId(@Param("areaId") String areaId);

    @Select("WITH ranked_data AS ( " +
            "    SELECT  " +
            "        csr.house_id,  " +
            "        hi.house_name,  " +
            "        cpm.other_name,  " +
            "        cpm.cur_value,  " +
            "        cpm.cur_time,  " +
            "        cpm.id as cpm_id,  " +
            "        dl.status,  " +
            "        hi.order_num,  " +
            "        cpm.param_type,  " +
            "        CASE  " +
            "            WHEN ar.create_time >= CURRENT_DATE AND ar.create_time < CURRENT_DATE + interval '1 day'  " +
            "            THEN 1  " +
            "            ELSE 0  " +
            "        END as alarm_status, " +
            "        ROW_NUMBER() OVER ( " +
            "            PARTITION BY csr.house_id  " +
            "            ORDER BY  " +
            "                CASE WHEN ar.create_time >= CURRENT_DATE AND ar.create_time < CURRENT_DATE + interval '1 day' THEN 0 ELSE 1 END, " +
            "                cpm.cur_time DESC " +
            "        ) as row_num " +
            "    FROM  " +
            "        cpm_space_relation csr  " +
            "        JOIN collection_params_manage cpm ON csr.cpm_id = cpm.id  " +
            "        RIGHT JOIN house_info hi ON csr.house_id = hi.id  " +
            "        JOIN device_ledger dl ON dl.id = cpm.device_ledger_id  " +
            "        LEFT JOIN alarm_records ar ON ar.cpm_id = cpm.id  " +
            "    WHERE  " +
            "        csr.floor_id = #{floorId}  " +
            "        AND hi.floor_id = #{floorId}  " +
            "        AND cpm.param_type = #{paramType}  " +
            "        AND cpm.system_type = #{systemType} " +
            ") " +
            "SELECT  " +
            "    house_id,  " +
            "    house_name,  " +
            "    other_name,  " +
            "    cur_value,  " +
            "    cur_time,  " +
            "    cpm_id,  " +
            "    status,  " +
            "    order_num,  " +
            "    param_type,  " +
            "    alarm_status " +
            "FROM  " +
            "    ranked_data " +
            "WHERE  " +
            "    row_num = 1 " +
            "ORDER BY  " +
            "    order_num; ")
    List<TempHumidityDTO> selectByParamType(@Param("systemType") String systemType,
                                            @Param("floorId") String floorId,
                                            @Param("paramType") String paramType);


    @Select("select  " +
            " cpm.id,  " +
            " dl.device_type as mt_type,  " +
            " cpm.device_ledger_id,  " +
            " dl.device_name, " +
            " cpm.other_name,  " +
            " cpm.cur_value,  " +
            " cpm.cur_time,  " +
            " cpm.param_type," +
            " dl.order_num as dl_order_num," +
            " cpm.order_num as cpm_order_num   " +
            "from  " +
            " collection_params_manage cpm  " +
            "left join   " +
            "device_ledger dl on  " +
            " cpm.device_ledger_id = dl.id  " +
            " where cpm.system_type = #{systemType}  " +
            " and cpm.param_type != '0'  " +
            " and cpm.param_type != '16' " +
            " and cpm.is_use = 0 " +
            " order by dl.device_type ,dl.order_num ,cpm.order_num  ")
    List<CollectionParamsManageVO> selectBySystemType(@Param("systemType") String systemType);

    @Select("select " +
            " hi.house_name || right(cpm.other_name, " +
            " 4) as name, " +
            " cpm.cur_value as value, " +
            " cpm.cur_time as curTime " +
            "from " +
            " collection_params_manage cpm " +
            "left join device_ledger dl on " +
            " cpm.device_ledger_id = dl.id " +
            "left join cpm_space_relation csr on " +
            " cpm.id = csr.cpm_id " +
            "left join house_info hi on " +
            " csr.house_id = hi.id " +
            "where " +
            " (cpm.param_type = '12' " +
            "  or cpm.param_type = '13') " +
            " and (dl.device_type is null)" +
            " and csr.floor_id = #{floorId} " +
            " and cpm.system_type = #{systemType}" +
            " order by hi.house_name, cpm.param_type ")
    List<Map<String, Object>> selectBySystemTypeAndBuildingId(@Param("systemType") String systemType, 
                                                              @Param("floorId") String floor_id);

    @Select("select  " +
            "  dl.id,  " +
            "  dl.device_name,  " +
            "  cpm.cur_value as collectValue,  " +
            "  cpm.cur_time as collectTime,  " +
            "  cpm.param_type as paramType,  " +
            "  cpm.other_name as collectName,  " +
            "  dl.order_num as orderNum  " +
            "from  " +
            "  collection_params_manage cpm  " +
            "left join device_ledger dl on  " +
            "  cpm.device_ledger_id = dl.id  " +
            "left join cpm_space_relation csr on  " +
            "  cpm.id = csr.cpm_id  " +
            "left join house_info hi on  " +
            "  csr.house_id = hi.id  " +
            "where  " +
            "  (dl.device_type = #{deviceType})  " +
            "  and csr.floor_id = #{floorId}   " +
            "  and cpm.system_type = #{systemType}  " +
            "  order by cpm.param_type,dl.order_num    ")
    List<DeviceMonitorVO> selectHotWaterBySystemTypeAndBuildingIdAndDeviceType(@Param("systemType") String systemType,
                                                                               @Param("floorId") String floorId,
                                                                               @Param("deviceType") String deviceType);

    /**
     * 查询热s热水控制列表
     * @param systemType
     * @param floorId
     * @return
     */
    @Select("select " +
            " cpm.id as cpm_id, " +
            " hi.id as house_id, " +
            " hi.house_name, " +
            " dl.device_name, " +
            " COALESCE(dl.id, '0') as dl_id, " +
            " cpm.cur_value, " +
            " cpm.cur_time, " +
            " cpm.param_type, " +
            " cpm.other_name,  " +
            " hi.order_num as hi_order_num,  " +
            " dl.order_num as dl_order_num,  " +
            " cpm.order_num as cpm_order_num  " +
            "from " +
            " collection_params_manage cpm " +
            "left join device_ledger dl on " +
            " cpm.device_ledger_id = dl.id " +
            "left join cpm_space_relation csr on " +
            " cpm.id = csr.cpm_id " +
            "left join house_info hi on " +
            " csr.house_id = hi.id " +
            "where " +
//            " (dl.device_type = '10' or dl.device_type = '16' or dl.device_type = '17' ) " +
            " csr.floor_id = #{floorId}  " +
            " and cpm.system_type = #{systemType} " +
            " order by dl.order_num   ")
    List<HotWaterControlListVO> selectHotWaterBySystemTypeAndBuildingId(@Param("systemType") String systemType,
                                                                        @Param("floorId") String floorId);

    @Select("select " +
            " cpm.id,  " +
            " dl.device_type as mt_type,  " +
            " cpm.device_ledger_id,  " +
            " dl.device_name, " +
            " cpm.other_name,  " +
            " cpm.cur_value,  " +
            " cpm.cur_time,  " +
            " cpm.param_type   " +
            "from " +
            " collection_params_manage cpm " +
            "left join device_ledger dl on " +
            " cpm.device_ledger_id = dl.id " +
            "left join cpm_space_relation csr on " +
            " cpm.id = csr.cpm_id " +
            "left join house_info hi on " +
            " csr.house_id = hi.id " +
            "where " +
            " csr.house_id = #{houseId}  " +
            " and cpm.system_type = #{systemType} " +
            " order by dl.order_num   ")
    List<CollectionParamsManageVO> selectBySystemTypeAndHouseId(@Param("systemType") String systemType,
                                                                @Param("houseId") String houseId);

    @Update("update collection_params_manage set cur_value = #{curValue}, cur_time = #{curTime} where id = #{id} ")
    void updateCurValueById(@Param("id") String id, @Param("curValue") BigDecimal curValue, @Param("curTime") Date curTime);

    @Select("select" +
            " device_ledger_id " +
            "from" +
            " collection_params_manage cpm " +
            "where" +
            " is_use = 0 " +
            " and date(cur_time) != date(current_timestamp) " +
            " and device_ledger_id is not null " +
            " group by device_ledger_id;")
    List<String> OffLine();
}
