package com.mh.system.mapper.device;

import com.mh.common.core.domain.entity.ChillersEntity;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceReport;
import com.mh.common.core.domain.vo.IndoorTempVO;
import org.apache.ibatis.annotations.*;

import java.util.Date;
import java.util.List;

/**
 * @Author : ljf
 * @date : 2025-02-10
 */
@Mapper
public interface DataProcessMapper {

    /**
     * 批量插入冷水机组分钟记录保存表
     *
     * @param batchList
     * @param tableName
     */
    @Select("<script>" +
            "insert into ${tableName}(device_num, device_code, register_address,register_name, cur_value, calc_value ,cur_time,grade,register_id)" +
            "values " +
            "<foreach collection='batchList' item='item' index='index' separator=','>" +
            "(#{item.mtNum}," +
            "#{item.mtCode}," +
            "#{item.registerAddr}," +
            "#{item.otherName}," +
            "cast(#{item.curValue} as numeric(24,2))," +
            "cast(#{item.curValue} as numeric(24,2))," +
            "#{item.curTime}," +
            "#{item.grade}," +
            "#{item.id})" +
            "</foreach>" +
            "</script>")
    void batchInsertChiller(@Param("batchList") List<CollectionParamsManage> batchList, @Param("tableName") String tableName);

    /**
     * 查询上一次数据
     *
     * @param deviceNum
     * @param tableName
     * @return
     */
    @Select("select * from ${tableName} where device_num = #{deviceNum} order by cur_time desc limit 1")
    @Results({
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "cur_time", property = "lastTime"),
            @Result(column = "cur_value", property = "lastValue"),
            @Result(column = "ratio", property = "ratio")
    })
    DeviceReport queryLastValue(@Param("deviceNum") String deviceNum, @Param("tableName") String tableName);

    @Select("select * from ${tableName} where device_num = #{entity.deviceNum} and cur_time = #{entity.curTime} limit 1")
    DeviceReport isHaveData(@Param("entity") DeviceReport deviceReport, @Param("tableName") String tableName);

    @Update("update ${tableName} set " +
            " cur_value = cast(#{entity.curValue} as numeric(24,2))," +
            " used_value= cast(#{entity.usedValue} as numeric(24,2)), " +
            " calc_value = cast(#{entity.calcValue} as numeric(24,2)), " +
            " cur_time = #{entity.curTime}," +
            " grade = #{entity.grade} where id = #{entity.id} ")
    void updateDataMinByTime(@Param("entity") DeviceReport deviceReport, @Param("tableName") String tableName);

    /**
     * 批量插入数据表中
     *
     * @param batchList
     * @param tableName
     */
    @Select("<script>" +
            "<foreach collection='batchList' item='item' index='index' separator=';'>" +
            "INSERT INTO ${tableName} (device_num, device_code, device_type, last_value, last_time, cur_value, cur_time, used_value, calc_value, grade, ratio) " +
            "SELECT " +
            " #{item.deviceNum}, " +
            " #{item.deviceCode}, " +
            " #{item.deviceType}, " +
            " cast(#{item.lastValue} as numeric(24,2)), " +
            " #{item.lastTime}, " +
            " cast(#{item.curValue} as decimal(24,2)),  " +
            " #{item.curTime}, " +
            " cast(#{item.usedValue} as decimal(24,2)), " +
            " cast(#{item.calcValue} as numeric(24,2)), " +
            " #{item.grade}, " +
            " #{item.ratio} " +
            "WHERE NOT EXISTS (" +
            "  SELECT 1 FROM ${tableName} WHERE device_num = #{item.deviceNum} AND cur_time = #{item.curTime} " +
            ")" +
            "</foreach>" +
            "</script>")
    void insertDataMin(@Param("batchList") List<DeviceReport> batchList,
                       @Param("tableName") String tableName);

    /**
     * 查询未处理的数据
     *
     * @param tableName
     * @return
     */
    @Select("select * from ${tableName} where grade = 0 order by cur_time desc")
    List<DeviceReport> queryUntreatedData(String tableName);

    @Select("select count(1) from ${tableName} where device_num = #{deviceNum} and EXTRACT(YEAR FROM cur_time) = EXTRACT(YEAR FROM #{curTime}::timestamp);")
    Integer selectDataByYY(@Param("tableName") String tableName,@Param("curTime") Date curTime,@Param("deviceNum")String deviceNum);

    @Select("select count(1) from ${tableName} where device_num = #{deviceNum} and DATE_TRUNC('month', cur_time) = DATE_TRUNC('month', #{curTime}::timestamp);")
    Integer selectDataByMM(@Param("tableName") String tableName,@Param("curTime") Date curTime,@Param("deviceNum")String deviceNum);

    @Select("select count(1) from ${tableName} where device_num = #{deviceNum} and DATE(cur_time) = DATE(#{curTime}::timestamp)")
    Integer selectDataByDD(@Param("tableName") String tableName,@Param("curTime") Date curTime,@Param("deviceNum")String deviceNum);

    @Select("select count(1) from ${tableName} where device_num = #{deviceNum} and DATE_TRUNC('hour', cur_time) = DATE_TRUNC('hour', #{curTime}::timestamp); ")
    Integer selectDataByHH(@Param("tableName") String tableName,@Param("curTime") Date curTime,@Param("deviceNum")String deviceNum);

    @Insert("insert into ${tableName}(device_num, device_code, device_type, last_value, last_time, cur_value,cur_time,calc_value,used_value,ratio,grade) " +
            "  values(" +
            " #{data.deviceNum}," +
            " #{data.deviceCode}," +
            " #{data.deviceType}," +
            " cast(#{data.lastValue} as numeric(24,2))," +
            " #{data.lastTime}," +
            " cast(#{data.curValue} as numeric(24,2))," +
            " #{data.curTime}," +
            " cast(#{data.calcValue} as numeric(24,2))," +
            " cast(#{data.usedValue} as numeric(24,2))," +
            " #{data.ratio}," +
            " 0)")
    void insertTable(@Param("data") DeviceReport data, @Param("tableName") String tableName);

    @Update("update ${tableName} set cur_time = #{data.curTime},cur_value = #{data.curValue}::numeric,calc_value = (#{data.curValue}::numeric - last_value)*ratio,used_value = #{data.curValue}::numeric - last_value " +
            "where device_num = #{data.deviceNum} and DATE_TRUNC('month', cur_time) = DATE_TRUNC('month', #{data.curTime}::timestamp); ")
    void updateTableMM(@Param("data") DeviceReport data, @Param("tableName") String tableName);

    @Update("update ${tableName} set cur_time = #{data.curTime},cur_value = #{data.curValue}::numeric,calc_value = (#{data.curValue}::numeric - last_value)*ratio,used_value = #{data.curValue}::numeric - last_value " +
            "where device_num = #{data.deviceNum} and EXTRACT(YEAR FROM cur_time) = EXTRACT(YEAR FROM #{data.curTime}::timestamp); ")
    void updateTableYY(@Param("data") DeviceReport data, @Param("tableName") String tableName);

    @Update("update ${tableName} set cur_time = #{data.curTime},cur_value = #{data.curValue}::numeric,calc_value = (#{data.curValue}::numeric - last_value)*ratio,used_value = #{data.curValue}::numeric - last_value " +
            "where device_num = #{data.deviceNum} and DATE(cur_time) = DATE(#{data.curTime}::timestamp) ")
    void updateTableDD(@Param("data") DeviceReport data, @Param("tableName") String tableName);

    @Update("update ${tableName} set cur_time = #{data.curTime},cur_value = #{data.curValue}::numeric,calc_value = (#{data.curValue}::numeric - last_value)*ratio,used_value = #{data.curValue}::numeric - last_value " +
            "where device_num = #{data.deviceNum} and DATE_TRUNC('hour', cur_time) = DATE_TRUNC('hour', #{data.curTime}::timestamp); ")
    void updateTableHH(@Param("data") DeviceReport data, @Param("tableName") String tableName);

    /**
     * 批量修改grade
     *
     * @param value     主键ID
     * @param tableName 表名
     * @param grade
     */
    @Update("<script>" +
            "update ${tableName} set grade = #{grade} where id in(" +
            "<foreach collection ='ids' item='id' index = 'index' separator=','>" +
            "#{id}" +
            "</foreach>" +
            ")" +
            "</script>")
    void batchUpdateGrade(@Param("ids") List<Long> value, @Param("tableName") String tableName, @Param("grade") int grade);

    /**
     * 查询初始数据
     *
     * @param deviceNum
     * @return
     */
    @Select("select mt_init_value from collection_params_manage where mt_num = #{deviceNum} and is_use = '0'")
    String queryInitValue(@Param("deviceNum") String deviceNum);

    @Select("select * from ${tableName} where register_id = #{registerId} and cur_time >= CURRENT_TIMESTAMP - INTERVAL '24 hours' ")
    List<ChillersEntity> selectLineDataByCodeAndRegisterAddr(@Param("tableName") String tableName,
                                                             @Param("registerId") String registerId);

    @Select("<script>" +
            "select to_char(cur_time, 'YYYY-MM-DD HH24:00:00') as curTime, cur_value as indoorTemp from ${tableName} " +
            "<where>" +
            "  <if test='registerIds != null and registerIds.size() > 0'>" +
            "    register_id in (" +
            "    <foreach collection='registerIds' item='id' separator=','>" +
            "      #{id}" +
            "    </foreach>" +
            "    )" +
            "  </if>" +
            "   <if test='startTime != null and endTime != null'>" +
            "       and cur_time between #{startTime}::timestamp and #{endTime}::timestamp " +
            "   </if>" +
            "</where>" +
            " order by cur_time desc" +
            "</script>")
    List<IndoorTempVO> selectHisDataByCodeAndRegisterAddr(@Param("tableName") String tableName,
                                                          @Param("registerIds") List<String> registerIds,
                                                          @Param("startTime") String startTime,
                                                          @Param("endTime") String endTime);

}
