package com.mh.system.mapper.energy;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;
import java.util.Map;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 综合能耗mapper
 * @date 2025-03-24 17:24:45
 */
@Mapper
public interface ComprehensiveEnergyConsumptionMapper {

    /**
     * 查询单个表数据
     * @param curTableName
     * @param isSum
     * @param startTime
     * @param endTime
     * @param paramType
     * @param systemType
     * @return
     */
    @Select("SELECT  " +
            "    sdd.dict_label as \"label\",  " +
            "    cpm.system_type as \"systemType\",  " +
            "    SUM(COALESCE(dd.calc_value, 0)) as \"calcValue\" " +
            "FROM  " +
            "    ${tableName} dd  " +
            "JOIN  " +
            "    collection_params_manage cpm  " +
            "ON  " +
            "    cpm.mt_num = dd.device_num  " +
            "JOIN " +
            "    sys_dict_data sdd " +
            "ON " +
            "    cpm.terminal_device_type = sdd.dict_value " +
            "WHERE  " +
            "    cpm.grade = #{grade}  " +
            "    AND cpm.mt_is_sum = #{isSum}  " +
            "    AND cpm.param_type = #{paramType}  " +
            "    AND cpm.system_type = #{systemType} " +
            "    AND dd.cur_time BETWEEN #{startTime}::timestamp AND #{endTime}::timestamp " +
            "    AND sdd.dict_type = 'sys_device_type' " +
            "GROUP BY  " +
            "    sdd.dict_label,  " +
            "    cpm.system_type; ")
    List<Map<String, Object>> selectOneTableData(@Param("tableName") String curTableName,
                                                 @Param("isSum") int isSum,
                                                 @Param("startTime")String startTime,
                                                 @Param("endTime") String endTime,
                                                 @Param("paramType") String paramType,
                                                 @Param("systemType") String systemType,
                                                 @Param("grade") int grade);

    /**
     * 查询多个表数据
     * @param curTableName
     * @param lastTableName
     * @param isSum
     * @param startTime
     * @param endTime
     * @param paramType
     * @param systemType
     * @return
     */
    @Select("SELECT  " +
            "    sdd.dict_label as \"label\",  " +
            "    cpm.system_type as \"systemType\",  " +
            "    SUM(COALESCE(dd.calc_value, 0)) as \"calcValue\" " +
            "FROM  " +
            "    ( " +
            "        SELECT device_num, calc_value, cur_time FROM ${curTableName} " +
            "           where cur_time BETWEEN #{startTime}::timestamp AND #{endTime}::timestamp  " +
            "        UNION ALL " +
            "        SELECT device_num, calc_value, cur_time FROM ${lastTableName} " +
            "           where cur_time BETWEEN #{startTime}::timestamp AND #{endTime}::timestamp  " +
            "    ) dd " +
            "JOIN  " +
            "    collection_params_manage cpm  " +
            "ON  " +
            "    cpm.mt_num = dd.device_num  " +
            "JOIN " +
            "    sys_dict_data sdd " +
            "ON " +
            "    cpm.terminal_device_type = sdd.dict_value " +
            "WHERE  " +
            "    cpm.grade = #{grade}  " +
            "    AND cpm.mt_is_sum = #{isSum}  " +
            "    AND cpm.param_type = #{paramType}  " +
            "    AND cpm.system_type = #{systemType} " +
            "    AND dd.cur_time BETWEEN #{startTime}::timestamp AND #{endTime}::timestamp " +
            "    AND sdd.dict_type = 'sys_device_type' " +
            "GROUP BY  " +
            "    sdd.dict_label,  " +
            "    cpm.system_type; ")
    List<Map<String, Object>> selectMultiTableData(@Param("curTableName") String curTableName,
                                                   @Param("lastTableName") String lastTableName,
                                                   @Param("isSum") int isSum,
                                                   @Param("startTime") String startTime,
                                                   @Param("endTime") String endTime,
                                                   @Param("paramType") String paramType,
                                                   @Param("systemType") String systemType,
                                                   @Param("grade") int grade);
}
