package com.mh.system.mapper.energy;

import com.mh.common.core.domain.entity.DeviceTypeEnergy;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 能耗分析mapper
 * @date 2023-12-14 14:04:48
 */
@Mapper
public interface EnergyAnalyzeMapper {

    /**
     * 跨表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select " +
            " dm.calc_value, " +
            " dm.device_num, " +
            " cpm.other_name as remark, " +
            " cpm.terminal_device_type as device_type " +
            "from " +
            " ${lastTableName} dm " +
            "join collection_params_manage cpm " +
            "            on " +
            " cpm.mt_num = dm.device_num " +
            " and cpm.mt_code = dm.device_code " +
            " and cpm.register_addr = dm.register_addr " +
            "join device_ledger dl  " +
            "            on " +
            " cpm.device_ledger_id = dl.id " +
            "where " +
            " dm.cur_time >= #{startTime} " +
            " and dm.cur_time &lt;= #{endTime} " +
            " and cpm.grade = 40 " +
            " and cpm.is_use = 0 " +
            "<if test='paramType != null and paramType != \"\"'>" +
            "   and dl.device_type = #{paramType}" +
            "</if>" +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and cpm.system_type = #{systemType}" +
            "</if>" +
            "union all " +
            "select " +
            " dm.calc_value, " +
            " dm.device_num, " +
            " cpm.other_name as remark, " +
            " cpm.terminal_device_type as device_type " +
            "from " +
            " ${curTableName} dm " +
            "join collection_params_manage cpm " +
            "            on " +
            " cpm.mt_num = dm.device_num " +
            " and cpm.mt_code = dm.device_code " +
            " and cpm.register_addr = dm.register_addr " +
            "join device_ledger dl  " +
            "            on " +
            " cpm.device_ledger_id = dl.id " +
            "where " +
            " dm.cur_time >= #{startTime} " +
            " and dm.cur_time &lt;= #{endTime} " +
            " and cpm.grade = 40 " +
            " and cpm.is_use = 0 " +
            "<if test='paramType != null and paramType != \"\"'>" +
            "   and dl.device_type = #{paramType}" +
            "</if>" +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and cpm.system_type = #{systemType}" +
            "</if>" +
            "</script>")
    List<DeviceTypeEnergy> queryManyTable(@Param("startTime") String startTime,
                                          @Param("endTime") String endTime,
                                          @Param("lastTableName") String lastTableName,
                                          @Param("curTableName") String curTableName,
                                          @Param("len") String dateLen,
                                          @Param("paramType") String paramType,
                                          @Param("systemType") String systemType);

    /**
     * 单表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select " +
            " dm.calc_value, " +
            " dm.device_num, " +
            " cpm.other_name as remark, " +
            " cpm.terminal_device_type as device_type " +
            "from " +
            " ${lastTableName} dm " +
            "join collection_params_manage cpm " +
            "            on " +
            " cpm.mt_num = dm.device_num " +
            " and cpm.mt_code = dm.device_code " +
            "join device_ledger dl  " +
            "            on " +
            " cpm.device_ledger_id = dl.id " +
            "where " +
            " dm.cur_time >= #{startTime}::timestamp " +
            " and dm.cur_time &lt;= #{endTime}::timestamp " +
            " and cpm.grade = 40 " +
            " and cpm.is_use = 0 " +
            "<if test='paramType != null and paramType != \"\"'>" +
            "   and dl.device_type = #{paramType}" +
            "</if>" +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and cpm.system_type = #{systemType}" +
            "</if>" +
            "</script>")
    List<DeviceTypeEnergy> queryOneTable(@Param("startTime") String startTime,
                                         @Param("endTime") String endTime,
                                         @Param("lastTableName") String lastTableName,
                                         @Param("curTableName") String curTableName,
                                         @Param("len") String dateLen,
                                         @Param("paramType") String paramType,
                                         @Param("systemType") String systemType);

    @Select("<script>" +
            "SELECT " +
            "  SUM(COALESCE(dh.calc_value, 0)) AS calcValue, " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr, " +
            "  dh.device_num AS deviceNum, " +
            "  dm.other_name as remark " +
            "FROM " +
            "  ${lastTableName} dh " +
            "JOIN collection_params_manage dm ON dh.device_num = dm.mt_num " +
            "WHERE " +
            "  dh.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dh.cur_time &lt;= #{endTime}::timestamp " +
            "  AND dm.id IN " +
            "  <foreach collection='params' item='item' open='(' separator=',' close=')'>" +
            "    #{item} " +
            "  </foreach> " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and dm.system_type = #{systemType}" +
            "</if>" +
            "GROUP BY " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ), " +
            "  dh.device_num, " +
            "  dm.other_name " +
            "</script>")
    List<DeviceTypeEnergy> queryDeviceOneTable(@Param("startTime") String startTime,
                                               @Param("endTime") String endTime,
                                               @Param("lastTableName") String lastTableName,
                                               @Param("curTableName") String curTableName,
                                               @Param("len") String dateLen,
                                               @Param("params") List<Object> params,
                                               @Param("systemType") String systemType);

    @Select("<script>" +
            "SELECT " +
            "  SUM(COALESCE(dh.calc_value, 0)) AS calcValue, " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr, " +
            "  dh.device_num AS deviceNum, " +
            "  dm.other_name as remark " +
            "FROM " +
            "  ${lastTableName} dh " +
            "JOIN collection_params_manage dm ON dh.device_num = dm.mt_num " +
            "WHERE " +
            "  dh.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dh.cur_time &lt;= #{endTime}::timestamp " +
            "  AND dm.id IN " +
            "  <foreach collection='params' item='item' open='(' separator=',' close=')'>" +
            "    #{item} " +
            "  </foreach> " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and dm.system_type = #{systemType}" +
            "</if>" +
            "GROUP BY " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ), " +
            "  dh.device_num, " +
            "  dm.other_name " +
            "union all " +
            "SELECT " +
            "  SUM(COALESCE(dh.calc_value, 0)) AS calcValue, " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr, " +
            "  dh.device_num AS deviceNum, " +
            "  dm.other_name as remark " +
            "FROM " +
            "  ${curTableName} dh " +
            "JOIN collection_params_manage dm ON dh.device_num = dm.mt_num " +
            "WHERE " +
            "  dh.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dh.cur_time &lt;= #{endTime}::timestamp " +
            "  AND dm.id IN " +
            "  <foreach collection='params' item='item' open='(' separator=',' close=')'>" +
            "    #{item} " +
            "  </foreach> " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "   and dm.system_type = #{systemType}" +
            "</if>" +
            "GROUP BY " +
            "  TO_CHAR(dh.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ), " +
            "  dh.device_num, " +
            "  dm.other_name " +
            "</script>" )
    List<DeviceTypeEnergy> queryDeviceManyTable(@Param("startTime") String startTime,
                                                @Param("endTime") String endTime,
                                                @Param("lastTableName") String lastTableName,
                                                @Param("curTableName") String curTableName,
                                                @Param("len") String dateLen,
                                                @Param("params") List<Object> params,
                                                @Param("systemType") String systemType);
}
