package com.mh.system.mapper.energy;

import com.mh.common.core.domain.entity.ConsumptionAnalyze;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 系统能耗分析
 * @date 2023/7/18 16:54:37
 */
@Mapper
public interface EnergyMapper {

    /**
     * 跨表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud' " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${lastTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dm.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.grade = 40 and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "union all " +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud' " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${curTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dm.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.grade = 40 and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "</script>")
    List<ConsumptionAnalyze> queryManyTable(@Param("startTime") String startTime,
                                            @Param("endTime") String endTime,
                                            @Param("lastTableName") String lastTableName,
                                            @Param("curTableName") String curTableName,
                                            @Param("len") String dateLen,
                                            @Param("paramType") String paramType,
                                            @Param("systemType") String systemType);

    /**
     * 单表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud' " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${lastTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dm.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.grade = 40 " +
            "    and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'>" +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "order by  " +
            "  timeStr " +
            "</script>")
    List<ConsumptionAnalyze> queryOneTable(@Param("startTime") String startTime,
                                           @Param("endTime") String endTime,
                                           @Param("lastTableName") String lastTableName,
                                           @Param("curTableName") String curTableName,
                                           @Param("len") String dateLen,
                                           @Param("paramType") String paramType,
                                           @Param("systemType") String systemType);

    /**
     * 查询能耗分析同比
     * @param startTime
     * @param endTime
     * @param lastTableName
     * @param curTableName
     * @param dateLen
     * @param paramType
     * @return
     */
    List<ConsumptionAnalyze> queryManyTableYoy(@Param("startTime") String startTime,
                                               @Param("endTime") String endTime,
                                               @Param("lastTableName") String lastTableName,
                                               @Param("curTableName") String curTableName,
                                               @Param("yoyStartTime") String yoyStartTime,
                                               @Param("yoyEndTime") String yoyEndTime,
                                               @Param("yoyLastTableName") String yoyLastTableName,
                                               @Param("yoyCurTableName") String yoyCurTableName,
                                               @Param("len") String dateLen,
                                               @Param("paramType") String paramType,
                                               @Param("timeType") String timeType,
                                               @Param("systemType") String systemType);


    /**
     * 查询能耗分析同比
     * @param startTime
     * @param endTime
     * @param lastTableName
     * @param curTableName
     * @param yoyStartTime
     * @param yoyEndTime
     * @param dateLen
     * @param paramType
     * @param timeType
     * @return
     */
    List<ConsumptionAnalyze> queryOneTableYoy(@Param("startTime") String startTime,
                                              @Param("endTime") String endTime,
                                              @Param("lastTableName") String lastTableName,
                                              @Param("curTableName") String curTableName,
                                              @Param("yoyStartTime") String yoyStartTime,
                                              @Param("yoyEndTime") String yoyEndTime,
                                              @Param("len") String dateLen,
                                              @Param("paramType") String paramType,
                                              @Param("timeType") String timeType,
                                              @Param("systemType") String systemType);

    /**
     * 查询能耗分析环比（查询类型是month，year）
     * @param startTime
     * @param endTime
     * @param yoyStartTime
     * @param yoyEndTime
     * @param tableName
     * @param dateLen
     * @param paramType
     * @param timeType
     * @return
     */
    List<ConsumptionAnalyze> queryMonthAndYearMom(@Param("startTime") String startTime,
                                                  @Param("endTime") String endTime,
                                                  @Param("momStartTime") String yoyStartTime,
                                                  @Param("momEndTime") String yoyEndTime,
                                                  @Param("tableName") String tableName,
                                                  @Param("len") String dateLen,
                                                  @Param("paramType") String paramType,
                                                  @Param("timeType") String timeType,
                                                  @Param("systemType") String systemType);

    /**
     * 单表查询能耗分析环比（查询类型是hour，day）
     * @param startTime
     * @param endTime
     * @param yoyStartTime
     * @param yoyEndTime
     * @param tableName
     * @param dateLen
     * @param paramType
     * @param timeType
     * @return
     */
    List<ConsumptionAnalyze> queryOneTableMom(@Param("startTime") String startTime,
                                              @Param("endTime") String endTime,
                                              @Param("momStartTime") String yoyStartTime,
                                              @Param("momEndTime") String yoyEndTime,
                                              @Param("tableName") String tableName,
                                              @Param("len") String dateLen,
                                              @Param("paramType") String paramType,
                                              @Param("timeType") String timeType,
                                              @Param("systemType") String systemType);

    /**
     * 多表查询能耗分析环比（查询类型是hour，day）
     * @param startTime
     * @param endTime
     * @param yoyStartTime
     * @param yoyEndTime
     * @param tableName1 上一年的表
     * @param tableName2 今年的表
     * @param dateLen
     * @param paramType
     * @param timeType
     * @return
     */
    List<ConsumptionAnalyze> queryManyTableMom(@Param("startTime") String startTime,
                                               @Param("endTime") String endTime,
                                               @Param("momStartTime") String yoyStartTime,
                                               @Param("momEndTime") String yoyEndTime,
                                               @Param("tableName1") String tableName1,
                                               @Param("tableName2") String tableName2,
                                               @Param("len") String dateLen,
                                               @Param("paramType") String paramType,
                                               @Param("timeType") String timeType,
                                               @Param("systemType") String systemType);

    /**
     * 查询各个机房设备的用电量（查询月，年）
     * @param startTime
     * @param endTime
     * @param lastTableName
     * @param curTableName
     * @param dateLen
     * @param deviceType
     * @return
     */
    List<ConsumptionAnalyze> queryDeviceOneTable(@Param("startTime") String startTime,
                                                 @Param("endTime") String endTime,
                                                 @Param("lastTableName") String lastTableName,
                                                 @Param("curTableName") String curTableName,
                                                 @Param("len") String dateLen,
                                                 @Param("deviceType") String deviceType);

    /**
     * 查询各个机房设备的用电量（查询小时，天）
     * @param startTime
     * @param endTime
     * @param lastTableName
     * @param curTableName
     * @param dateLen
     * @param deviceType
     * @return
     */
    List<ConsumptionAnalyze> queryDeviceManyTable(@Param("startTime") String startTime,
                                                  @Param("endTime") String endTime,
                                                  @Param("lastTableName") String lastTableName,
                                                  @Param("curTableName") String curTableName,
                                                  @Param("len") String dateLen,
                                                  @Param("deviceType") String deviceType,
                                                  @Param("systemType") String systemType);

    /**
     * 查询系统分析折线图
     * @param curTable
     * @param curCopTable
     * @param startTime
     * @param endTime
     * @return
     */
    List<ConsumptionAnalyze> queryLineDataSysByOne(@Param("curTable") String curTable,
                                                   @Param("curCopTable") String curCopTable,
                                                   @Param("startTime") String startTime,
                                                   @Param("endTime") String endTime);

    /**
     * 查询系统分析折线图
     * @param lastTable
     * @param curTable
     * @param lastCopTable
     * @param curCopTable
     * @param startTime
     * @param endTime
     * @return
     */
    List<ConsumptionAnalyze> queryLineDataSysByMany(@Param("lastTable") String lastTable,
                                                    @Param("curTable") String curTable,
                                                    @Param("lastCopTable") String lastCopTable,
                                                    @Param("curCopTable") String curCopTable,
                                                    @Param("startTime") String startTime,
                                                    @Param("endTime") String endTime);

    /**
     * 查询各个设备的折线图数据
     * @param startTime
     * @param endTime
     * @param lastTableName
     * @param curTableName
     * @param deviceType
     * @return
     */
    List<ConsumptionAnalyze> queryDeviceLineManyTable(@Param("startTime") String startTime,
                                                      @Param("endTime") String endTime,
                                                      @Param("lastTableName") String lastTableName,
                                                      @Param("curTableName") String curTableName,
                                                      @Param("deviceType") String deviceType,
                                                      @Param("systemType") String systemType);

    @Select("select isnull(sum(calc_value),0) from ${tableName} " +
            " where device_type = 'cloud' " +
            " and device_code in (select device_code from device_manage where grade >=0 and grade < 10) " +
            " and cur_time >= concat(#{timeStr},':00:00') and cur_time <= concat(#{timeStr},':59:59') ")
    String queryCoolingData(@Param("tableName") String tableName, @Param("timeStr") String timeString);
}
