package com.mh.system.mapper.energy;

import com.mh.common.core.domain.entity.ConsumptionAnalyze;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 能耗查询
 * @date 2023-12-13 16:00:01
 */
@Mapper
public interface EnergyQueryMapper {

    /**
     * 跨表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud'  " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${lastTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dh.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'> " +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "union all " +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud'  " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${curTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dm.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'> " +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "</script>")
    List<ConsumptionAnalyze> queryManyTable(@Param("startTime") String startTime,
                                            @Param("endTime") String endTime,
                                            @Param("lastTableName") String lastTableName,
                                            @Param("curTableName") String curTableName,
                                            @Param("len") String dateLen,
                                            @Param("paramType") String paramType,
                                            @Param("haveMeter") boolean haveMeter,
                                            @Param("haveCloud") boolean haveCloud,
                                            @Param("systemType") String systemType);

    /**
     * 单表查询操作
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param lastTableName 上一个表名
     * @param curTableName 当前表名
     * @return
     */
    @Select("<script>" +
            "select  " +
            "  sum(dm.calc_value) as curValue,  " +
            "  CASE dm.device_type  " +
            "       WHEN '5' THEN 'meter'  " +
            "       WHEN '2' THEN 'cloud'  " +
            "       ELSE 'Unknown'  " +
            "  END as deviceType," +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) AS timeStr " +
            "from  " +
            "  ${lastTableName} dm  " +
            "where  " +
            "  dm.cur_time &gt;= #{startTime}::timestamp " +
            "  AND dm.cur_time &lt;= #{endTime}::timestamp " +
            "  and dm.device_num in (  " +
            "  select  " +
            "    cpm.mt_num  " +
            "  from  " +
            "    collection_params_manage cpm  " +
            "  join device_ledger dl   " +
            "on  " +
            "    cpm.device_ledger_id = dl.id  " +
            "    and dl.device_type in ('5', '6') and cpm.terminal_device_type = '15' " +
            "<if test='systemType != null and systemType != \"\"'> " +
            "    and cpm.system_type = #{systemType} " +
            "</if>" +
            "<if test='paramType != null and paramType != \"\"'>" +
            "    and cpm.param_type = #{paramType} " +
            "</if>" +
            ")  " +
            "group by  " +
            "  device_type,  " +
            "  TO_CHAR(dm.cur_time, " +
            "    <choose>" +
            "      <when test='len == 4'>'YYYY'</when>" +
            "      <when test='len == 7'>'YYYY-MM'</when>" +
            "      <when test='len == 10'>'YYYY-MM-DD'</when>" +
            "      <when test='len == 13'>'YYYY-MM-DD HH24'</when>" +
            "      <when test='len == 16'>'YYYY-MM-DD HH24:MI'</when>" +
            "      <otherwise>'YYYY-MM-DD HH24:MI:SS'</otherwise>" +
            "    </choose>" +
            "  ) " +
            "order by  " +
            "  timeStr " +
            "</script>")
    List<ConsumptionAnalyze> queryOneTable(@Param("startTime") String startTime,
                                           @Param("endTime") String endTime,
                                           @Param("lastTableName") String lastTableName,
                                           @Param("curTableName") String curTableName,
                                           @Param("len") String dateLen,
                                           @Param("paramType") String paramType,
                                           @Param("haveMeter") boolean haveMeter,
                                           @Param("haveCloud") boolean haveCloud,
                                           @Param("systemType") String systemType);

}
