package com.mh.system.mapper.energy;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.common.core.domain.dto.EnergyPreEchartDataDTO;
import com.mh.common.core.domain.dto.EnergyPreTopDataDTO;
import com.mh.common.core.domain.entity.HistoryDataPre;
import org.apache.ibatis.annotations.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 用能预测
 * @date 2025-03-25 16:55:58
 */
@Mapper
public interface HistoryDataPreMapper extends BaseMapper<HistoryDataPre> {

    @Results({
            @Result(column = "pre_cur_data", property = "preCurData"),
            @Result(column = "yes_data", property = "yesData"),
            @Result(column = "pre_yes_data", property = "preYesData"),
            @Result(column = "error_data", property = "errorData")
    })
    @Select("<script>" +
            "SELECT " +
            "  <choose>" +
            "    <when test='type == \"2\"'>" +
            "      COALESCE(t.water_value_pre, 0) as pre_cur_data, " +
            "      COALESCE(t1.water_value, 0) as yes_data, " +
            "      COALESCE(t1.water_value_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.water_value_pre > 0 THEN (ABS(COALESCE(t1.water_value_pre, 0) - COALESCE(t1.water_value, 0)) / t1.water_value_pre * 100)::numeric(18, 2) ELSE 0 END as error_data" +
            "    </when>" +
            "    <when test='type == \"5\"'>" +
            "      COALESCE(t.elect_value_pre, 0) as pre_cur_data, " +
            "      COALESCE(t1.elect_value, 0) as yes_data, " +
            "      COALESCE(t1.elect_value_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.elect_value_pre > 0 THEN (ABS(COALESCE(t1.elect_value_pre, 0) - COALESCE(t1.elect_value, 0)) / t1.elect_value_pre * 100)::numeric(18, 2) ELSE 0 END as error_data" +
            "    </when>" +
            "    <when test='type == \"3\"'>" +
            "      COALESCE(t.water_level_pre, 0) as pre_cur_data, " +
            "      COALESCE(t1.water_level, 0) as yes_data, " +
            "      COALESCE(t1.water_level_pre, 0) as pre_yes_data, " +
            "      CASE WHEN t1.water_level_pre > 0 THEN (ABS(COALESCE(t1.water_level_pre, 0) - COALESCE(t1.water_level, 0)) / t1.water_level_pre * 100)::numeric(18, 2) ELSE 0 END as error_data" +
            "    </when>" +
            "    <otherwise>null</otherwise>" +
            "  </choose>" +
            "FROM (" +
            "  SELECT " +
            "    system_type, " +
            "    water_value, " +
            "    elect_value, " +
            "    water_level, " +
            "    water_value_pre, " +
            "    elect_value_pre, " +
            "    water_level_pre " +
            "  FROM history_data_pre " +
            "  WHERE cur_date = CURRENT_DATE AND system_type = #{systemType}" +
            ") t " +
            "JOIN (" +
            "  SELECT " +
            "    system_type, " +
            "    water_value, " +
            "    elect_value, " +
            "    water_level, " +
            "    water_value_pre, " +
            "    elect_value_pre, " +
            "    water_level_pre " +
            "  FROM history_data_pre " +
            "  WHERE cur_date = CURRENT_DATE - INTERVAL '1 day' AND system_type = #{systemType}" +
            ") t1 ON t.system_type = t1.system_type " +
            "</script>")
    List<EnergyPreTopDataDTO> getTopData(@Param("systemType") String systemType, @Param("type") String type);

    @Results({
            @Result(column = "cur_date", property = "curDate"),
            @Result(column = "cur_data", property = "curData"),
            @Result(column = "pre_data", property = "preData"),
            @Result(column = "error_data", property = "errorData")
    })
    @Select("<script>" +
            "SELECT " +
            "  hdp.cur_date, " +
            "  <choose>" +
            "    <when test='type == \"2\"'>COALESCE(hdp.water_value, 0)</when>" +
            "    <when test='type == \"5\"'>COALESCE(hdp.elect_value, 0)</when>" +
            "    <when test='type == \"3\"'>COALESCE(hdp.water_level, 0)</when>" +
            "    <otherwise>0</otherwise>" +
            "  </choose> as cur_data, " +
            "  <choose>" +
            "    <when test='type == \"2\"'>COALESCE(hdp.water_value_pre, 0)</when>" +
            "    <when test='type == \"5\"'>COALESCE(hdp.elect_value_pre, 0)</when>" +
            "    <when test='type == \"3\"'>COALESCE(hdp.water_level_pre, 0)</when>" +
            "    <otherwise>0</otherwise>" +
            "  </choose> as pre_data, " +
            "  CASE " +
            "    <when test='type == \"2\"'>" +
            "      WHEN hdp.water_value_pre > 0 THEN (ABS(COALESCE(hdp.water_value_pre, 0) - COALESCE(hdp.water_value, 0)) / hdp.water_value_pre * 100)::numeric(18, 2) ELSE 0 " +
            "    </when>" +
            "    <when test='type == \"5\"'>" +
            "      WHEN hdp.elect_value_pre > 0 THEN (ABS(COALESCE(hdp.elect_value_pre, 0) - COALESCE(hdp.elect_value, 0)) / hdp.elect_value_pre * 100)::numeric(18, 2) ELSE 0 " +
            "    </when>" +
            "    <when test='type == \"3\"'>" +
            "      WHEN hdp.water_level_pre > 0 THEN (ABS(COALESCE(hdp.water_level_pre, 0) - COALESCE(hdp.water_level, 0)) / hdp.water_level_pre * 100)::numeric(18, 2) ELSE 0 " +
            "    </when>" +
            "  END as error_data " +
            "FROM history_data_pre hdp " +
            "WHERE hdp.system_type = #{systemType} " +
            "  AND hdp.cur_date BETWEEN #{beginDate}::timestamp AND #{endDate}::timestamp " +
            "ORDER BY hdp.cur_date" +
            "</script>")
    List<EnergyPreEchartDataDTO> getEnergyPre(@Param("systemType") String systemType,
                                              @Param("beginDate") String beginDate,
                                              @Param("endDate") String endDate,
                                              @Param("type") String deviceType);

}
