package com.mh.system.mapper.energy;

import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.entity.ChillersEntity;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DataMonth;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.math.BigDecimal;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 项目总览
 * @date 2025-03-19 16:37:22
 */
@Mapper
public interface OverviewMapper {

    @Select("<script>" +
            "select sum(calc_value) as calc_value from data_year where " +
            "  cur_time >= DATE_TRUNC('year', CURRENT_DATE)" +
            " AND cur_time &lt; DATE_TRUNC('year', CURRENT_DATE) + INTERVAL '1 year'" +
            " <if test='deviceNums != null and deviceNums.size() > 0'>" +
            " AND device_num IN " +
            "   <foreach collection='deviceNums' item='item' open='(' separator=',' close=')'>" +
            "     #{item.mtNum} " +
            "   </foreach>" +
            " </if>" +
            "</script>")
    BigDecimal queryByDeviceNum(@Param("deviceNums") List<CollectionParamsManage> deviceNums);

    @Select("<script>" +
            "WITH months AS ( " +
            "  SELECT  " +
            "    to_char(generate_series( " +
            "      date_trunc('month', CURRENT_DATE) - INTERVAL '11 months',  " +
            "      date_trunc('month', CURRENT_DATE),  " +
            "      '1 month' " +
            "    ), 'YYYY-MM') AS month " +
            ") " +
            "SELECT  " +
            "  m.month as name, " +
            "  COALESCE(SUM(dm.calc_value), 0) AS value " +
            "FROM months m " +
            "LEFT JOIN data_month dm  " +
            "  ON to_char(dm.cur_time, 'YYYY-MM') = m.month " +
            " <if test='deviceNums != null and deviceNums.size() > 0'>" +
            " AND dm.device_num IN " +
            "   <foreach collection='deviceNums' item='item' open='(' separator=',' close=')'>" +
            "     #{item.mtNum} " +
            "   </foreach>" +
            " </if>" +
            "  AND dm.cur_time >= date_trunc('month', CURRENT_DATE) - INTERVAL '11 months' " +
            "WHERE m.month &lt;= to_char(CURRENT_DATE, 'YYYY-MM') " +
            "GROUP BY m.month " +
            "ORDER BY m.month; " +
            "</script>")
    List<ColumnFilter> energyAnalysis(@Param("deviceNums") List<CollectionParamsManage> deviceNums);

    @Select("<script>" +
            "select sum(cur_value) from collect_param_manage where id in " +
            "   <foreach collection='deviceNums' item='item' open='(' separator=',' close=')'>" +
            "     #{item.id} " +
            "   </foreach>" +
            "</script>")
    BigDecimal selectEER(@Param("deviceNums") List<CollectionParamsManage> deviceNums);
}
