package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.dto.DeviceMonitorDTO;
import com.mh.common.core.domain.dto.HotWaterControlDTO;
import com.mh.common.core.domain.dto.HotWaterNowDataDTO;
import com.mh.common.core.domain.dto.PumpInfoDTO;
import com.mh.common.core.domain.entity.ChillersEntity;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceLedger;
import com.mh.common.core.domain.vo.*;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.device.DataProcessMapper;
import com.mh.system.mapper.device.DeviceLedgerMapper;
import com.mh.system.mapper.device.GatewayManageMapper;
import com.mh.system.service.device.ICollectionParamsManageService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备采集参数配置
 * @date 2025-01-13 16:33:47
 */
@Slf4j
@Service
public class CollectionParamsManageServiceImpl implements ICollectionParamsManageService {

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Resource
    private DeviceLedgerMapper deviceLedgerMapper;

    @Resource
    private GatewayManageMapper gatewayManageMapper;

    @Resource
    private DataProcessMapper dataProcessMapper;

    @Override
    public List<CollectionParamsManage> selectCollectionParamsManageList(CollectionParamsManage communicationParams) {
        if (communicationParams == null) {
            return List.of();
        }
        QueryWrapper<CollectionParamsManage> queryWrapper = new QueryWrapper<>();
        // 是否启用
        if (communicationParams.getIsUse() != null) {
            queryWrapper.eq("is_use", communicationParams.getIsUse());
        }
        // 仪表类型
        if (communicationParams.getMtType() != null) {
            queryWrapper.eq("mt_type", communicationParams.getMtType());
        }
        // 表号
        if (communicationParams.getMtNum() != null) {
            queryWrapper.like("mt_num", communicationParams.getMtNum());
        }
        // 设备台账
        if (communicationParams.getDeviceLedgerId() != null) {
            queryWrapper.eq("device_ledger_id", communicationParams.getDeviceLedgerId());
        }
        // 根据系统类型查询对应系统的状态
        if (!StringUtils.isEmpty(communicationParams.getSystemType())) {
            queryWrapper.eq("system_type", communicationParams.getSystemType());
        }
        // 根据点位参数类型
        if (!StringUtils.isEmpty(communicationParams.getParamType())) {
            queryWrapper.eq("param_type", communicationParams.getParamType());
        }
        // 根据备注查询
        if (!StringUtils.isEmpty(communicationParams.getRemark())) {
            queryWrapper.like("remark", communicationParams.getRemark());
        }
        // 添加时间范围
        if (communicationParams.getParams() != null && !communicationParams.getParams().isEmpty()) {
            String beginTimeStr = (String) communicationParams.getParams().get("beginTime");
            String endTimeStr = (String) communicationParams.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        queryWrapper.orderByDesc("cur_time");
        return collectionParamsManageMapper.selectList(queryWrapper);
    }

    @Override
    public CollectionParamsManage selectCollectionParamsManageById(String cpmId) {
        return collectionParamsManageMapper.selectById(cpmId);
    }

    @Override
    public int insertCollectionParamsManage(CollectionParamsManage communicationParams) {
        if (communicationParams != null) {
            return collectionParamsManageMapper.insert(communicationParams);
        }
        return 0;
    }

    @Override
    public int updateCollectionParamsManage(CollectionParamsManage communicationParams) {
        if (communicationParams != null) {
            return collectionParamsManageMapper.updateById(communicationParams);
        }
        return 0;
    }

    @Override
    public int deleteCommunicationByIds(String[] cpmIds) {
        if (cpmIds != null && cpmIds.length > 0) {
            for (String cpmId : cpmIds) {
                collectionParamsManageMapper.deleteById(cpmId);
            }
            return cpmIds.length;
        }
        return 0;
    }

    @Override
    public List<CollectionParamsManage> queryAllDevice() {
        return collectionParamsManageMapper.selectList(null);
    }

    @Override
    public List<CollectionParamsManage> queryCollectionParamsByMtType(String mtType) {
        QueryWrapper<CollectionParamsManage> queryWrapper = new QueryWrapper<>();
//        if ("0".equals(mtType)) {
//            queryWrapper.eq("mt_type", mtType);
//        } else {
//            queryWrapper.ne("mt_type", mtType);
//        }
        queryWrapper.eq("is_use", 0);
        queryWrapper.orderByAsc("order_num");
        return collectionParamsManageMapper.selectList(queryWrapper);
    }

    @Override
    public void updateCollectionParamsManages(ArrayList<CollectionParamsManage> chillersEntities) {
        // 批量更新
        chillersEntities.forEach(chillerEntity -> {
            log.info("chillerEntity: {}", chillerEntity.toString());
            if (chillerEntity.getId() != null && !StringUtils.isEmpty(chillerEntity.getId())) {
                collectionParamsManageMapper.updateCurValueById(chillerEntity.getId(), chillerEntity.getCurValue(), chillerEntity.getCurTime());
            }
        });
        // 根据台账id更新台账在线情况
        // 根据chillerEntity使用stream流获取台账id分组数据
        chillersEntities.stream().map(CollectionParamsManage::getDeviceLedgerId).distinct().forEach(deviceLedgerId -> {
            // 根据台账id 更新 device_ledger 在线情况,0:在线，1：离线
            if (!StringUtils.isEmpty(deviceLedgerId)) {
                deviceLedgerMapper.updateOnlineStatus(deviceLedgerId);
            }
        });
        // 根据网关id更新网关在线情况
        chillersEntities.stream().map(CollectionParamsManage::getGatewayId).distinct().forEach(gatewayId -> {
            // 根据网关id 更新 gateway_manage 在线情况,0:在线，1：离线
            gatewayManageMapper.updateOnlineStatus(gatewayId);
        });
    }

    @Override
    public List<PumpInfoDTO> selectPumpListInfo(String registerAddr, String mtType) {
        CollectionParamsManage collectionParamsManage = new CollectionParamsManage();
        collectionParamsManage.setRemark(registerAddr);
        collectionParamsManage.setMtType(mtType);
        List<CollectionParamsManage> collectionParamsManages = selectCollectionParamsManageList(collectionParamsManage);
        // 根据设备台账id，分离出对应的设备信息
        List<PumpInfoDTO> pumpInfoDTOS = collectionParamsManages.stream().map(collectionParamsManage1 -> {
            PumpInfoDTO pumpInfoDTO = new PumpInfoDTO();
            // 根据设备台账id，获取设备名称
            DeviceLedger deviceLedger = deviceLedgerMapper.selectById(collectionParamsManage1.getDeviceLedgerId());
            pumpInfoDTO.setName(deviceLedger.getDeviceName());
            // 根据别称进行赋值
            String otherName = collectionParamsManage1.getOtherName();
            if (otherName.contains("开关状态")) {
                pumpInfoDTO.setSwitchStatus(collectionParamsManage1.getCurValue().toString());
            } else if (otherName.contains("控制状态")) {
                pumpInfoDTO.setControlStatus(collectionParamsManage1.getCurValue().toString());
            } else if (otherName.contains("报警状态")) {
                pumpInfoDTO.setAlertStatus(collectionParamsManage1.getCurValue().toString());
            }
            return pumpInfoDTO;
        }).toList();
        if (!pumpInfoDTOS.isEmpty()) {
            return pumpInfoDTOS;
        }
        return List.of();
    }

    @Override
    public List<ChillersEntity> lineData(String registerAddr, String mtType) {
        // 根据device_code, register_ address 查询出对应的官网压力，水温
        // 截取当前时间yyyy
        String now = DateUtils.getTime();
        String tableName = "chillers_data_min" + now.substring(0, 4);
        String registerId = collectionParamsManageMapper.selectIdByMtCodeAndType(registerAddr, mtType);
        List<ChillersEntity> chillersEntities = dataProcessMapper.selectLineDataByCodeAndRegisterAddr(tableName, registerId);
        if (chillersEntities.isEmpty()) {
            return List.of();
        }
        return chillersEntities;
    }

    @Override
    public List<DeviceMonitorDTO> selectMonitorListBySystemType(String systemType) {
        // 根据系统类型查询出对应的设备
        List<CollectionParamsManageVO> collectionParamsManages = collectionParamsManageMapper.selectBySystemType(systemType);
        if (collectionParamsManages.isEmpty()) {
            return List.of();
        }
        // 得出的collectionParamsManages进行数据赋值，如果paramType="1',则是运行状态，则赋值给runStatus,
        // 如果paramType="5",则是故障报警，则赋值给alarmStatus
        // 如果paramType="6",则是手自动状态，则赋值给handOrAuto
        // 如果paramType="4",则是频率反馈，则赋值给frequency
        // 使用Stream API处理数据
        Map<String, List<DeviceMonitorVO>> groupedByDeviceType = collectionParamsManages.stream()
                .filter(p -> p.getDeviceLedgerId() != null && StringUtils.isNotEmpty(p.getDeviceLedgerId()))
                .collect(Collectors.groupingBy(CollectionParamsManageVO::getId))
                .values()
                .stream()
                .map(params -> {
                    DeviceMonitorVO deviceMonitorVO = new DeviceMonitorVO();
                    for (CollectionParamsManageVO param : params) {
                        deviceMonitorVO.setId(param.getId());
                        deviceMonitorVO.setDeviceLedgerId(param.getDeviceLedgerId());
                        deviceMonitorVO.setDeviceName(param.getDeviceName());
                        deviceMonitorVO.setDeviceType(param.getMtType());
                        deviceMonitorVO.setCollectName(param.getOtherName());
                        deviceMonitorVO.setCollectTime(param.getCurTime());
                        // 判断如果是压力，放大1000倍，并保留一位小数
                        if (param.getParamType().equals("13")) {
                            BigDecimal bigDecimal = param.getCurValue();
                            bigDecimal = bigDecimal.multiply(new BigDecimal(1000));
                            param.setCurValue(bigDecimal.setScale(1, RoundingMode.HALF_UP));
                        }
                        deviceMonitorVO.setCollectValue(String.valueOf(param.getCurValue().setScale(1, RoundingMode.HALF_UP)));
                        deviceMonitorVO.setParamType(param.getParamType());
                        deviceMonitorVO.setOrderNum(param.getCpmOrderNum());
                    }
                    return deviceMonitorVO;
                })
                .collect(Collectors.groupingBy(DeviceMonitorVO::getDeviceType));

        // 根据deviceType进行分组，赋值给List<DeviceMonitorDTO>得出结果集
        List<DeviceMonitorDTO> result = groupedByDeviceType.entrySet().stream()
                .map(entry -> {
                    DeviceMonitorDTO deviceMonitorDTO = new DeviceMonitorDTO();
                    deviceMonitorDTO.setName(entry.getKey());
                    deviceMonitorDTO.setValues(entry.getValue());
                    return deviceMonitorDTO;
                })
                .collect(Collectors.toList());

        return result;
    }

    @Override
    public List<DeviceOperateMonitorVO> operationDeviceList(String systemType) {
        // 根据系统类型查询出对应的设备
        List<CollectionParamsManageVO> collectionParamsManages = collectionParamsManageMapper.selectBySystemType(systemType);
        if (collectionParamsManages.isEmpty()) {
            return List.of();
        }
        // 根据deviceLedgerId进行分组，然后得出deviceLedgerId，deviceName赋值给DeviceOperateMonitorVO的id和name
        // 按设备台账ID分组
        Map<String, List<CollectionParamsManageVO>> groupedByDevice = collectionParamsManages.stream()
                .sorted(Comparator.comparing(CollectionParamsManageVO::getDlOrderNum))
                .filter(p -> p.getDeviceLedgerId() != null && StringUtils.isNotEmpty(p.getDeviceLedgerId()))
                .collect(Collectors.groupingBy(CollectionParamsManageVO::getDeviceLedgerId));

        // 转换分组结果为监控VO列表
        List<DeviceOperateMonitorVO> monitorData = groupedByDevice.values().stream()
                .map(group -> {
                    // 取组内第一个元素获取公共信息（假设同设备台账ID的设备名称相同）
                    CollectionParamsManageVO firstItem = group.get(0);

                    DeviceOperateMonitorVO vo = new DeviceOperateMonitorVO();
                    vo.setId(firstItem.getDeviceLedgerId());
                    vo.setName(firstItem.getDeviceName());
                    vo.setOrderNum(firstItem.getDlOrderNum());

                    // 可补充其他需要聚合的字段
                    // vo.setStatus(calculateStatus(group));
                    return vo;
                })
                .sorted(Comparator.comparing(DeviceOperateMonitorVO::getOrderNum))
                .toList();
        // 在遍历monitorData,根据deviceLedgerId查询出collectionParamsManages集合对应的设备id,otherName对应name,curValue对应value,curTime对应time,paramType对应paramType,然后拼接到monitorData中children中
        // 处理子节点数据
        monitorData.forEach(mainVO -> {
            // 根据主设备ID过滤参数
            List<DeviceOperateMonitorVO> children = collectionParamsManages.stream()
                    .filter(p -> mainVO.getId().equals(p.getDeviceLedgerId()))
                    .map(param -> {
                        DeviceOperateMonitorVO child = new DeviceOperateMonitorVO();
                        child.setId(param.getId());
                        child.setName(param.getOtherName());
                        child.setValue(String.valueOf(param.getCurValue()));
                        child.setTime(param.getCurTime());
                        child.setParamType(param.getParamType());
                        child.setOrderNum(param.getCpmOrderNum());
                        return child;
                    })
                    .sorted(Comparator.comparing(DeviceOperateMonitorVO::getOrderNum))
                    .collect(Collectors.toList());
            mainVO.setChildren(children);
        });
        return monitorData;
    }

    @Override
    public List<ColumnFilter> monitorPublic(String systemType, String floorId) {
        List<Map<String, Object>> mapList = collectionParamsManageMapper.selectBySystemTypeAndBuildingId(systemType, floorId);
        // stream流判断mapList中的数据curTime是否为今天日期，假如不是今天日期，则重新赋值value为"采集失败"，然后响应赋值给List<ColumnFilter>
        return mapList.stream()
                .map(map -> {
                    // 1. 获取当前记录的 curTime 和 value
                    Object curTimeObj = map.get("curTime");

                    // 2. 判断 curTime 是否为今天
                    boolean isToday = false;
                    try {
                        if (curTimeObj != null) {
                            // 假设 curTime 是 Date 类型（根据实际数据库类型调整）
                            Date curTime = (Date) curTimeObj;
                            LocalDate recordDate = curTime.toInstant()
                                    .atZone(ZoneId.systemDefault())
                                    .toLocalDate();
                            LocalDate today = LocalDate.now();
                            isToday = recordDate.isEqual(today);
                        }
                    } catch (Exception e) {
                        // 处理日期解析异常（如类型不匹配）
                        log.error("日期解析失败: {}", curTimeObj, e);
                    }

                    // 3. 如果非今天或 curTime 为空，设置 value 为采集失败
                    if (!isToday || curTimeObj == null) {
                        map.put("value", "采集失败");
                    } else {
                        // 4.1 如果是今天，设置 value 为采集到的值
                        map.put("value", new BigDecimal(String.valueOf(map.get("value"))).setScale(0, RoundingMode.HALF_UP).toString());
                    }

                    // 4. 转换为 ColumnFilter 对象（假设构造函数为 ColumnFilter(name, value)）
                    return new ColumnFilter(
                            (String) map.get("name"),
                            (String) map.get("value")
                    );
                })
                .collect(Collectors.toList());
    }

    @Override
    public List<HotWaterNowDataDTO> monitorList(String systemType, String floorId) {
        // 处理热泵设备（deviceType=11）
        List<DeviceMonitorVO> hotPumps = collectionParamsManageMapper.selectHotWaterBySystemTypeAndBuildingIdAndDeviceType(systemType, floorId, "11");
        Map<String, List<DeviceMonitorVO>> hotPumpGroups = hotPumps.stream()
                .collect(Collectors.groupingBy(DeviceMonitorVO::getId));

        List<HotWaterNowDataDTO> result = hotPumpGroups.values().stream().map(group -> {
            HotWaterNowDataDTO dto = new HotWaterNowDataDTO();
            group.forEach(vo -> {
                dto.setId(vo.getId());
                dto.setPumpId(vo.getId());
                dto.setPumpName(vo.getDeviceName());
                dto.setCurDate(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", vo.getCollectTime()));
                dto.setCurDate(DateUtils.parseDateToStr("yyyy-MM-dd HH:mm:ss", vo.getCollectTime()));
                dto.setOrderNum(vo.getOrderNum());

                switch (vo.getParamType()) {
                    case "2":  // 运行状态
                        dto.setRunState(vo.getCollectValue());
                        break;
                    case "5":  // 故障状态
                        dto.setIsFault(vo.getCollectValue());
                        break;
                    case "14": // 温度设置
                        dto.setTempSet(vo.getCollectValue());
                        break;
                    case "12": // 水温
                        dto.setWaterTemp(vo.getCollectValue());
                        break;
                }
            });
            return dto;
        }).collect(Collectors.toList());

        // 处理供水泵（deviceType=10）
        List<DeviceMonitorVO> waterPumps = collectionParamsManageMapper.selectHotWaterBySystemTypeAndBuildingIdAndDeviceType(systemType, floorId, "10");
        // 判断是否存在collectValue>0的记录
        boolean hasValidCollectValue = waterPumps.stream()
                .anyMatch(vo -> "1".equals(vo.getParamType())
                        && vo.getCollectValue() != null
                        && new BigDecimal(vo.getCollectValue()).intValue() > 0);

        // 根据判断结果设置状态值
        result.forEach(dto -> dto.setUpWaterState(hasValidCollectValue ? "1" : "0"));

        // 处理水箱液位（deviceType=12）
        List<DeviceMonitorVO> waterLevels = collectionParamsManageMapper.selectHotWaterBySystemTypeAndBuildingIdAndDeviceType(systemType, floorId, "12");

        // 计算levelSet平均值（paramType=10）
        Double levelSetAvg = waterLevels.stream()
                .filter(vo -> vo.getParamType().equals("10"))
                .mapToDouble(vo -> {
                    try {
                        return Double.parseDouble(vo.getCollectValue());
                    } catch (NumberFormatException e) {
                        log.error("数值转换失败：设备ID={} 值={}", vo.getId(), vo.getCollectValue());
                        return 0.0; // 返回默认值
                    }
                })
                .average()
                .orElse(0.0);

        // 计算waterLevel平均值（paramType=11）
        Double waterLevelAvg = waterLevels.stream()
                .filter(vo -> vo.getParamType().equals("11"))
                .mapToDouble(vo -> {
                    try {
                        return Double.parseDouble(vo.getCollectValue());
                    } catch (NumberFormatException e) {
                        log.error("数值转换失败：设备ID={} 值={}", vo.getId(), vo.getCollectValue());
                        return 0.0; // 返回默认值
                    }
                })
                .average()
                .orElse(0.0);

        // 设置平均值到所有DTO（根据业务需求调整）
        result.forEach(dto -> {
            dto.setLevelSet(String.format("%.2f", levelSetAvg));
            dto.setWaterLevel(String.format("%.2f", waterLevelAvg));
        });
        // result再根据orderNum排序
        return result.stream()
                .sorted(Comparator.comparingInt(HotWaterNowDataDTO::getOrderNum))
                .collect(Collectors.toList());
    }

    @Override
    public List<HotWaterControlDTO> operateList(String systemType, String floorId) {
        List<HotWaterControlListVO> hotWaterControlListVOS = collectionParamsManageMapper.selectHotWaterBySystemTypeAndBuildingId(systemType, floorId);
        if (hotWaterControlListVOS.isEmpty()) {
            return List.of();
        }

        return hotWaterControlListVOS.stream()
//                .peek(vo -> System.out.println(vo.getHiOrderNum()))
                .collect(Collectors.groupingBy(HotWaterControlListVO::getHouseId))
                .entrySet().stream()
                .map(houseEntry -> {
                    HotWaterControlDTO dto = new HotWaterControlDTO();
                    dto.setId(houseEntry.getKey());
                    dto.setName(houseEntry.getValue().getFirst().getHouseName());
                    dto.setOrderNum(houseEntry.getValue().getFirst().getHiOrderNum());

                    List<HotWaterControlVO> children = houseEntry.getValue().stream()
                            .collect(Collectors.groupingBy(HotWaterControlListVO::getDlId))
                            .entrySet().stream()
                            .map(dlEntry -> {
                                HotWaterControlVO vo = new HotWaterControlVO();
                                List<HotWaterControlListVO> dlItems = dlEntry.getValue();
                                vo.setId(dlEntry.getKey());
                                if (StringUtils.isEmpty(dlItems.getFirst().getDeviceName())) {
                                    // 如果设备名称为空，则使用房间号
                                    vo.setName(dto.getName());
                                } else {
                                    vo.setName(dlItems.getFirst().getDeviceName());
                                }
                                vo.setOrderNum(dlItems.getFirst().getDlOrderNum());

                                dlItems.forEach(item -> {
                                    switch (item.getParamType()) {
                                        case "1":
                                            // 运行状态
                                            vo.setRunningStatus(item.getCurValue().intValue());
                                            vo.setRunningStatusId(item.getCpmId());
                                            break;
                                        case "2":
                                            vo.setSwitchStatus(item.getCurValue().intValue());
                                            vo.setSwitchStatusId(item.getCpmId());
                                            break;
                                        case "3":
                                            vo.setFrequencySet(item.getCurValue());
                                            vo.setFrequencySetId(item.getCpmId());
                                            break;
                                        case "4":
                                            vo.setFrequency(item.getCurValue());
                                            vo.setFrequencyId(item.getCpmId());
                                            break;
                                        case "5":
                                            vo.setAlarmStatus(item.getCurValue().intValue());
                                            vo.setAlarmStatusId(item.getCpmId());
                                            break;
                                        case "6":
                                            vo.setHandAutomaticSwitch(item.getCurValue().intValue());
                                            vo.setHandAutomaticSwitchId(item.getCpmId());
                                            break;
                                        case "8":
                                            vo.setOpenSwitch(item.getCurValue().intValue());
                                            vo.setOpenSwitchId(item.getCpmId());
                                            break;
                                        case "9":
                                            vo.setCloseSwitch(item.getCurValue().intValue());
                                            vo.setCloseSwitchId(item.getCpmId());
                                            break;
                                        case "10":
                                            vo.setWaterLevelSet(item.getCurValue());
                                            vo.setWaterLevelSetId(item.getCpmId());
                                            break;
                                        case "11":
                                            vo.setWaterLevel(item.getCurValue());
                                            vo.setCurTime(item.getCurTime());
                                            vo.setWaterLevelId(item.getCpmId());
                                            break;
                                        case "12":
                                            vo.setTemp(item.getCurValue());
                                            vo.setTempId(item.getCpmId());
                                            break;
                                        case "13":
                                            vo.setPressure(item.getCurValue());
                                            vo.setPressureId(item.getCpmId());
                                            break;
                                        case "14":
                                            vo.setTempSet(item.getCurValue());
                                            vo.setTempSetId(item.getCpmId());
                                            break;
                                        case "15":
                                            vo.setPressureSet(item.getCurValue());
                                            vo.setPressureSetId(item.getCpmId());
                                            break;
                                        case "34":
                                            vo.setDelayTimeSet(item.getCurValue());
                                            vo.setDelayTimeSetId(item.getCpmId());
                                            break;
                                        case "35":
                                            vo.setDiffValueSet(item.getCurValue());
                                            vo.setDiffValueSetId(item.getCpmId());
                                            break;
                                        case "36":
                                            vo.setCounterSet(item.getCurValue());
                                            vo.setCounterSetId(item.getCpmId());
                                            break;
                                        case "37":
                                            vo.setSwitchTimeSet(item.getCurValue());
                                            vo.setSwitchTimeSetId(item.getCpmId());
                                            break;
                                        case "38":
                                            vo.setFaultResetStatus(item.getCurValue().intValue());
                                            vo.setFaultResetStatusId(item.getCpmId());
                                            break;
                                        case "39":
                                            vo.setEmergencyStopStatus(item.getCurValue().intValue());
                                            vo.setEmergencyStopStatusId(item.getCpmId());
                                            break;
                                        default:
                                            // 处理未识别的paramType或忽略
                                            break;
                                    }
                                });
                                return vo;
                            })
                            .sorted(Comparator.comparingInt(HotWaterControlVO::getOrderNum))
                            .collect(Collectors.toList());

                    dto.setChildren(children);
                    return dto;
                })
                // 添加排序操作
                .sorted(Comparator.comparingInt(HotWaterControlDTO::getOrderNum))
                .collect(Collectors.toList());
    }

    @Override
    public List<DeviceMonitorDTO> selectMonitorListBySystemTypeAndHouseId(String systemType, String houseId) {
        // 根据系统类型查询出对应的设备
        List<CollectionParamsManageVO> collectionParamsManages = collectionParamsManageMapper.selectBySystemTypeAndHouseId(systemType, houseId);
        if (collectionParamsManages.isEmpty()) {
            return List.of();
        }
        // 得出的collectionParamsManages进行数据赋值，如果paramType="1',则是运行状态，则赋值给runStatus,
        // 如果paramType="5",则是故障报警，则赋值给alarmStatus
        // 如果paramType="6",则是手自动状态，则赋值给handOrAuto
        // 如果paramType="4",则是频率反馈，则赋值给frequency
        // 使用Stream API处理数据
        Map<String, List<DeviceMonitorVO>> groupedByDeviceType = collectionParamsManages.stream()
                .collect(Collectors.groupingBy(CollectionParamsManageVO::getId))
                .values()
                .stream()
                .map(params -> {
                    DeviceMonitorVO deviceMonitorVO = new DeviceMonitorVO();
                    for (CollectionParamsManageVO param : params) {
                        deviceMonitorVO.setId(param.getId());
                        deviceMonitorVO.setDeviceLedgerId(param.getDeviceLedgerId());
                        deviceMonitorVO.setDeviceName(param.getDeviceName());
                        deviceMonitorVO.setDeviceType(param.getMtType());
                        deviceMonitorVO.setCollectName(param.getOtherName());
                        deviceMonitorVO.setCollectTime(param.getCurTime());
                        deviceMonitorVO.setCollectValue(String.valueOf(param.getCurValue()));
                        deviceMonitorVO.setParamType(param.getParamType());
                    }
                    return deviceMonitorVO;
                })
                .collect(Collectors.groupingBy(DeviceMonitorVO::getDeviceType));

        // 根据deviceType进行分组，赋值给List<DeviceMonitorDTO>得出结果集
        List<DeviceMonitorDTO> result = groupedByDeviceType.entrySet().stream()
                .map(entry -> {
                    DeviceMonitorDTO deviceMonitorDTO = new DeviceMonitorDTO();
                    deviceMonitorDTO.setName(entry.getKey());
                    deviceMonitorDTO.setValues(entry.getValue());
                    return deviceMonitorDTO;
                })
                .collect(Collectors.toList());

        return result;
    }

}
