package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.dto.SysPerformanceDTO;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.SysParams;
import com.mh.common.core.domain.entity.WeatherData;
import com.mh.common.core.domain.vo.DeviceOperateMonitorVO;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.WeatherUtil;
import com.mh.system.mapper.SysParamsMapper;
import com.mh.system.mapper.WeatherDataMapper;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.device.ICoolingSystemMonitorService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 冷源系统监控实现类
 * @date 2025-03-30 10:28:53
 */
@Slf4j
@Service
public class CoolingSystemMonitorServiceImpl implements ICoolingSystemMonitorService {

    private final SysParamsMapper sysParamsMapper;

    private final WeatherDataMapper weatherDataMapper;

    private final CollectionParamsManageMapper collectionParamsManageMapper;

    public CoolingSystemMonitorServiceImpl(SysParamsMapper sysParamsMapper, WeatherDataMapper weatherDataMapper, CollectionParamsManageMapper collectionParamsManageMapper) {
        this.sysParamsMapper = sysParamsMapper;
        this.weatherDataMapper = weatherDataMapper;
        this.collectionParamsManageMapper = collectionParamsManageMapper;
    }

    @Override
    public HashMap<String, Object> getSystemMode(String systemType, String paramType) {
        HashMap<String, Object> result = new HashMap<>();
        CollectionParamsManage collectionParamsManage = collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", paramType));
        // 判断当前系统启动状态
        List<CollectionParamsManage> collectionParamsManages = collectionParamsManageMapper.selectList(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", "30"));
        if (null != collectionParamsManage && null != collectionParamsManage.getCurValue()) {
            // 判断启动模式是什么
            if (collectionParamsManage.getCurValue().compareTo(new BigDecimal("1")) == 0) {
                // 如果是1，说明是"一键开关模式"
                result.put("autoType", "一键开关模式");
                // 获取一键启动标识
                CollectionParamsManage collectionParamsManage1 = collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", "23"));
                // 判断当前是开还是关闭
                getStartOrStopStatus(result, collectionParamsManages, collectionParamsManage1);
            } else if (collectionParamsManage.getCurValue().compareTo(new BigDecimal("2")) == 0) {
                // 如果是2，说明是"定时模式"
                result.put("autoType", "定时模式");
                // 获取定时启动标识标识
                CollectionParamsManage collectionParamsManage1 = collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", "29"));
                // 判断当前是开还是关闭
                getStartOrStopStatus(result, collectionParamsManages, collectionParamsManage1);
            }
        }
        // 查询报警内容
        List<CollectionParamsManage> alarmList = collectionParamsManageMapper.selectList(new QueryWrapper<CollectionParamsManage>()
                .eq("system_type", systemType)
                .eq("param_type", "5")
                .eq("cur_value", 1));
        result.put("alarmList", alarmList);
        return result;
    }

    private void getStartOrStopStatus(HashMap<String, Object> result, List<CollectionParamsManage> collectionParamsManages, CollectionParamsManage collectionParamsManage1) {
        if (null != collectionParamsManage1 && null != collectionParamsManage1.getCurValue()) {
            String startOrStopStatus = "";
            if (collectionParamsManage1.getCurValue().compareTo(new BigDecimal("1")) == 0) {
                // 如果是1，说明是"开启",匹配系统启动状态是开启的
                CollectionParamsManage statusParam = collectionParamsManages.stream().filter(val -> val.getOtherName().contains("一键启动")).findFirst().get();
                switch (statusParam.getCurValue().intValue()) {
                    case 0:
                        startOrStopStatus = "";
                        break;
                    case 1:
                        startOrStopStatus = "开阀中......";
                        break;
                    case 2:
                        startOrStopStatus = "阀开失败检测报警，开启冷却泵......";
                        break;
                    case 3:
                        startOrStopStatus = "开启冷却塔风机中......";
                        break;
                    case 4:
                        startOrStopStatus = "开启冷冻泵中......";
                        break;
                    case 5:
                        startOrStopStatus = "开启主机中......";
                        break;
                    default:
                        break;
                }
            } else if (collectionParamsManage1.getCurValue().compareTo(new BigDecimal("0")) == 0) {
                // 如果是0，说明是"关闭"
                // 如果是0，说明是"开启",匹配系统启动状态是关闭的
                CollectionParamsManage statusParam = collectionParamsManages.stream().filter(val -> val.getOtherName().contains("一键关机")).findFirst().get();
                switch (statusParam.getCurValue().intValue()) {
                    case 0:
                        startOrStopStatus = "";
                        break;
                    case 1:
                        startOrStopStatus = "关主机中......";
                        break;
                    case 2:
                        startOrStopStatus = "关闭冷却塔风机中......";
                        break;
                    case 3:
                        startOrStopStatus = "关闭冷却泵中......";
                        break;
                    case 4:
                        startOrStopStatus = "关闭冷冻泵中......";
                        break;
                    case 5:
                        startOrStopStatus = "关闭冷却阀门中......";
                        break;
                    case 6:
                        startOrStopStatus = "关闭冷冻阀门中......";
                        break;
                    default:
                        break;
                }
            }
            result.put("startOrStopStatus", startOrStopStatus);
        }
    }

    @Override
    public CollectionParamsManage getOneKeyButton(String systemType, String paramType) {
        CollectionParamsManage collectionParamsManage = collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", paramType));
        // 判断启动模式是什么
        if (null != collectionParamsManage && null != collectionParamsManage.getCurValue()) {
            // 判断启动模式是什么
            if (collectionParamsManage.getCurValue().compareTo(new BigDecimal("1")) == 0) {
                // 如果是1，说明是"一键开关模式"
                return collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", "23"));
            } else if (collectionParamsManage.getCurValue().compareTo(new BigDecimal("2")) == 0) {
                // 如果是2，说明是"定时模式"
                return collectionParamsManageMapper.selectOne(new QueryWrapper<CollectionParamsManage>().eq("system_type", systemType).eq("param_type", "29"));
            }
        }
        return null;
    }

    @Override
    public SysPerformanceDTO getSysPerformance() {
        SysPerformanceDTO sysPerformanceDTO = new SysPerformanceDTO();
        // 查询总表的数据
        QueryWrapper<CollectionParamsManage> queryWrapper = new QueryWrapper<CollectionParamsManage>()
                .eq("system_type", "0")
                .eq("terminal_device_type", "15")
                .in("param_type", Arrays.asList("16", "17"));
        List<CollectionParamsManage> collectionParamsManages = collectionParamsManageMapper.selectList(queryWrapper);
        // 遍历获取数据
        if (null != collectionParamsManages && !collectionParamsManages.isEmpty()) {
            for (CollectionParamsManage value : collectionParamsManages) {
                // 判断paramTYpe,如果16：是电表，如果是17：是冷量计
                if ("16".equals(value.getParamType())) {
                    // 电表
                    // 在判断cur_time是当天值，grade，40:累计值，140：瞬时值
                    if (DateUtils.isSameDay(new Date(), value.getCurTime())) {
                        // 如果是当天值，判断grade，40:累计值，140：瞬时值
                        if (value.getGrade() == 40) {
                            // 累计值
                            sysPerformanceDTO.setYearPower(value.getCurValue().subtract(value.getMtInitValue()));
                        } else if (value.getGrade() == 140) {
                            // 瞬时值
                            sysPerformanceDTO.setRealPower(value.getCurValue());
                        }
                    } else {
                        // 如果不是当天值，判断grade，40:累计值，140：瞬时值
                        BigDecimal bigDecimal = new BigDecimal("0.0");
                        if (value.getGrade() == 40) {
                            // 累计值
                            sysPerformanceDTO.setYearPower(bigDecimal);
                        } else if (value.getGrade() == 140) {
                            // 瞬时值
                            sysPerformanceDTO.setRealPower(bigDecimal);
                        }
                    }
                } else if ("17".equals(value.getParamType())) {
                    // 冷量计
                    // 在判断cur_time是当天值，grade，40:累计值，140：瞬时值
                    if (DateUtils.isSameDay(new Date(), value.getCurTime())) {
                        // 如果是当天值，判断grade，40:累计值，140：瞬时值
                        if (value.getGrade() == 40) {
                            // 累计值
                            sysPerformanceDTO.setYearCold(value.getCurValue().subtract(value.getMtInitValue()));
                        } else if (value.getGrade() == 140) {
                            // 瞬时值
                            sysPerformanceDTO.setRealCold(value.getCurValue());
                        }
                    } else {
                        // 如果不是当天值，判断grade，40:累计值，140：瞬时值
                        BigDecimal bigDecimal = new BigDecimal("0.0");
                        if (value.getGrade() == 40) {
                            // 累计值
                            sysPerformanceDTO.setYearCold(bigDecimal);
                        } else if (value.getGrade() == 140) {
                            // 瞬时值
                            sysPerformanceDTO.setRealCold(bigDecimal);
                        }
                    }
                }
            }
            // 开始计算瞬时EER和累计EER
            BigDecimal yearPower = sysPerformanceDTO.getYearPower();
            BigDecimal yearCold = sysPerformanceDTO.getYearCold();
            BigDecimal realPower = sysPerformanceDTO.getRealPower();
            BigDecimal realCold = sysPerformanceDTO.getRealCold();

            // 计算瞬时EER
            if (realPower != null && realCold != null) {
                if (realPower.compareTo(BigDecimal.ZERO) == 0) {
                    sysPerformanceDTO.setRealEER(BigDecimal.ZERO.setScale(1));  // 除数为0时赋值为0.0
                } else {
                    BigDecimal eer = realCold.divide(realPower, 1, RoundingMode.HALF_UP);
                    sysPerformanceDTO.setRealEER(eer);
                }
            }

            // 计算累计EER
            if (yearPower != null && yearCold != null) {
                if (yearPower.compareTo(BigDecimal.ZERO) == 0) {
                    sysPerformanceDTO.setYearEER(BigDecimal.ZERO.setScale(1));  // 除数为0时赋值为0.0
                } else {
                    BigDecimal yearEer = yearCold.divide(yearPower, 1, RoundingMode.HALF_UP);
                    sysPerformanceDTO.setYearEER(yearEer);
                }
            }
        }
        return sysPerformanceDTO;
    }

    @Override
    public Map<String, String> getWeatherData() {
        Map<String, String> result = new HashMap<>();
        // 查询跟当前时间yyyy-MM-dd HH相同的数据
        WeatherData weatherData = weatherDataMapper.selectWeatherDataByDate(DateUtils.getTime());
        if (weatherData != null) {
            WeatherUtil weatherUtil = new WeatherUtil();
            // 室外温度
            int temperature = weatherData.getTemperature();
            result.putIfAbsent("temperature", temperature + "");

            // 室外湿度
            int humidity = weatherData.getHumidity();
            result.putIfAbsent("humidity", humidity + "");

            // 室外压力，采用标准大气压 101325 Pa
            String pressure = "101.325";
//            result.putIfAbsent("pressure", new BigDecimal("101.325") + "");

            // 室外含湿
            BigDecimal humidityRatio = weatherUtil.returnMoistureContent(String.valueOf(temperature),
                    String.valueOf(new BigDecimal(humidity).divide(new BigDecimal("100")).setScale(1, RoundingMode.HALF_UP)),
                    pressure);
            result.put("humidityRatio", humidityRatio.setScale(1, RoundingMode.HALF_UP).toString());

            // 室外焓值
            BigDecimal enthalpy = weatherUtil.returnEnthalpy(String.valueOf(temperature),
                    String.valueOf(new BigDecimal(humidity).divide(new BigDecimal("100")).setScale(1, RoundingMode.HALF_UP)),
                    pressure);
            result.put("enthalpy", enthalpy.setScale(1, RoundingMode.HALF_UP).toString());

            // 室外露点温度
            BigDecimal dewPointTemp = weatherUtil.calculateDewPoint(temperature, humidity);
            result.put("dewPointTemp", dewPointTemp.setScale(1, RoundingMode.HALF_UP).toString());

            // 室外湿球温度
            BigDecimal wetBulbTemp = weatherUtil.returnComputePsi(temperature, (double) humidity / 100, Double.parseDouble(pressure) * 1000);
            result.put("wetBulbTemp", wetBulbTemp.setScale(1, RoundingMode.HALF_UP).toString());
        }
        return result;
    }

    @Override
    public Map<String, String> getRunTime() {
        List<SysParams> sysParams = sysParamsMapper.selectSysParamsList();
        Map<String, String> result = new HashMap<>();
        if (!sysParams.isEmpty()) {
            // 获取运行时间
            Date operateStartTime = sysParams.get(0).getOperateStartTime();
            // 比较当前时间跟运行时间的相差多少天
            long days = (new Date().getTime() - operateStartTime.getTime()) / (1000 * 60 * 60 * 24);
            result.put("runTime", days + "");
        } else {
            result.put("runTime", "0");
        }
        return result;
    }
}
