package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.DeviceInOutManage;
import com.mh.common.core.domain.entity.SparePartsManage;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.DeviceInOutManageMapper;
import com.mh.system.mapper.device.SparePartsManageMapper;
import com.mh.system.service.device.IDeviceInOutManageService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 配件设备出入库流水管理
 * @date 2025-01-09 15:39:45
 */
@Service
public class DeviceInOutManageServiceImpl implements IDeviceInOutManageService {

    @Resource
    private DeviceInOutManageMapper deviceInOutManageMapper;

    @Resource
    private SparePartsManageMapper sparePartsManageMapper;

    @Override
    public List<DeviceInOutManage> selectDeviceInOutManageList(DeviceInOutManage deviceInOutManage) {
        if (deviceInOutManage == null) {
            return List.of();
        }
        QueryWrapper<DeviceInOutManage> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(deviceInOutManage.getSparePartsId())) {
            queryWrapper.eq("spare_parts_id", deviceInOutManage.getSparePartsId());
        }
        if (!StringUtils.isEmpty(deviceInOutManage.getSpareName())) {
            queryWrapper.like("spare_name", deviceInOutManage.getSpareName());
        }
        if (deviceInOutManage.getOperaType() != null) {
            queryWrapper.eq("opera_type", deviceInOutManage.getOperaType());
        }
        // 出入库时间范围
        // 添加时间范围
        if (deviceInOutManage.getParams() != null && !deviceInOutManage.getParams().isEmpty())  {
            String beginTimeStr = (String) deviceInOutManage.getParams().get("beginTime");
            String endTimeStr = (String) deviceInOutManage.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        queryWrapper.orderByDesc("create_time");
        return deviceInOutManageMapper.selectList(queryWrapper);
    }

    @Override
    public DeviceInOutManage selectDeviceInOutManageById(String spareId) {
        return deviceInOutManageMapper.selectById(spareId);
    }

    @Transactional
    @Override
    public int insertDeviceInOutManage(List<DeviceInOutManage> deviceInOutManageList, String createBy) {
        Date date = new Date();
        int result = 0;
        for (DeviceInOutManage deviceInOutManage : deviceInOutManageList) {
            deviceInOutManage.setCreateBy(createBy);
            deviceInOutManage.setCreateTime(date);
            // 获取当前库存量
            String sparePartsId = deviceInOutManage.getSparePartsId();
            SparePartsManage sparePartsManage = sparePartsManageMapper.selectById(sparePartsId);
            int inventoryLevels = 0;
            if (sparePartsManage != null) {
                inventoryLevels = sparePartsManage.getInventoryLevels();
            }
            // 修改配件的库存：operaType=0：入库；operaType=1：出库
            int intOutNum = deviceInOutManage.getIntOutNum();
            if (deviceInOutManage.getOperaType() == 0) {
                inventoryLevels = inventoryLevels + intOutNum;
            } else if (deviceInOutManage.getOperaType() == 1) {
                inventoryLevels = inventoryLevels - intOutNum;
            }
            sparePartsManageMapper.updateInventoryLevels(deviceInOutManage.getSparePartsId(), inventoryLevels, deviceInOutManage.getCreateBy());
            deviceInOutManage.setInventoryLevels(inventoryLevels);
            result = result + deviceInOutManageMapper.insert(deviceInOutManage);
        }
        return result;
    }

    @Override
    public int updateDeviceInOutManage(DeviceInOutManage deviceInOutManage) {
        return 0;
    }

    @Override
    public int deleteDeviceInOutManageByIds(String[] spareIds) {
        return 0;
    }
}
