package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DeviceLedger;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.device.DeviceLedgerMapper;
import com.mh.system.service.device.IDeviceLedgerService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备台账管理
 * @date 2025-01-10 16:55:28
 */
@Service
public class DeviceLedgerServiceImpl implements IDeviceLedgerService {

    @Resource
    private DeviceLedgerMapper deviceLedgerMapper;

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Override
    public void updateDeviceLedgerStatus() {
        // 先从采集点为中查询，看看当天是否有数据上来
        List<String> deviceLedgerList = collectionParamsManageMapper.OffLine();
        // 开始根据id更新设备台账状态
        for (String deviceLedgerId : deviceLedgerList) {
            deviceLedgerMapper.updateOnlineOrOfflineStatus(deviceLedgerId, 1);
        }
    }

    @Override
    public List<DeviceLedger> selectDeviceLedgerList(DeviceLedger ledgerInfo) {
        if (ledgerInfo == null) {
            return List.of();
        }
        QueryWrapper<DeviceLedger> queryWrapper = new QueryWrapper<>();
        // 设备名称
        if (!StringUtils.isEmpty(ledgerInfo.getDeviceName())) {
            queryWrapper.like("device_name", ledgerInfo.getDeviceName());
        }
        // 资产编号
        if (!StringUtils.isEmpty(ledgerInfo.getAssetNum())) {
            queryWrapper.like("asset_num", ledgerInfo.getAssetNum());
        }
        // 设备类型
        if (!StringUtils.isEmpty(ledgerInfo.getDeviceType())) {
            queryWrapper.eq("device_type", ledgerInfo.getDeviceType());
        }
        // 出入库时间范围
        if (ledgerInfo.getParams() != null && !ledgerInfo.getParams().isEmpty())  {
            String beginTimeStr = (String) ledgerInfo.getParams().get("beginTime");
            String endTimeStr = (String) ledgerInfo.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        queryWrapper.orderByAsc("order_num");
        return deviceLedgerMapper.selectList(queryWrapper);
    }

    @Override
    public DeviceLedger selectDeviceLedgerById(String ledgerId) {
        return deviceLedgerMapper.selectById(ledgerId);
    }

    @Override
    public int insertDeviceLedger(DeviceLedger deviceLedger) {
        return deviceLedgerMapper.insert(deviceLedger);
    }

    @Override
    public int updateDeviceLedger(DeviceLedger deviceLedger) {
        return deviceLedgerMapper.updateById(deviceLedger);
    }

    @Override
    public int deleteDeviceLedgerByIds(String[] ledgerIds) {
        if (ledgerIds != null && ledgerIds.length > 0) {
            for (String cmpId : ledgerIds) {
                deviceLedgerMapper.deleteById(cmpId);
            }
            return ledgerIds.length;
        }
        return 0;
    }
}
