package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.config.MHConfig;
import com.mh.common.constant.Constants;
import com.mh.common.core.domain.entity.DeviceQrManage;
import com.mh.common.utils.file.FileToMultipartConverter;
import com.mh.common.utils.file.FileUploadUtils;
import com.mh.common.utils.file.MimeTypeUtils;
import com.mh.common.utils.qrcode.QrCodeUtil;
import com.mh.system.mapper.device.DeviceQrManageMapper;
import com.mh.system.service.device.IDeviceQrManageService;
import jakarta.annotation.Resource;
import org.apache.commons.io.FileUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 二维码管理
 * @date 2025-01-09 17:13:21
 */
@Service
public class DeviceQrManageServiceImpl implements IDeviceQrManageService {

    @Resource
    private DeviceQrManageMapper deviceQrManageMapper;

    @Override
    public List<DeviceQrManage> selectDeviceQrManageList(DeviceQrManage deviceQrManage) {
        if (deviceQrManage == null) {
            return List.of();
        }
        QueryWrapper<DeviceQrManage> queryWrapper = new QueryWrapper<>();
        if (deviceQrManage.getStatus() != null) {
            queryWrapper.eq("status", deviceQrManage.getStatus());
        }
        queryWrapper.orderByAsc("create_time");
        return deviceQrManageMapper.selectList(queryWrapper);
    }

    @Override
    public DeviceQrManage selectDeviceQrManageById(String qrId) {
        return deviceQrManageMapper.selectById(qrId);
    }

    @Override
    public int updateDeviceQrManage(DeviceQrManage deviceQrManage) {
        // 判断是解绑还是绑定
        if (deviceQrManage.getStatus() == 0 && deviceQrManage.getBindTime() == null)  {
            // 绑定
            deviceQrManage.setBindTime(new Date());
        } else {
            // 解绑
            deviceQrManage.setBindTime(null);
            deviceQrManage.setDeviceLedgerId(null);
            deviceQrManage.setDeviceName(null);
        }
        return deviceQrManageMapper.updateById(deviceQrManage);
    }

    @Override
    public int insertDeviceQrManage(DeviceQrManage deviceQrManage) {
        return deviceQrManageMapper.insert(deviceQrManage);
    }

    @Transactional
    @Override
    public int deleteDeviceQrManageByIds(String[] qrIds) {
        if (qrIds != null) {
            for (String qrId : qrIds) {
                deviceQrManageMapper.deleteById(qrId);
            }
            return qrIds.length;
        }
        return 0;
    }

    @Transactional
    @Override
    public int createQrCode(Integer qrNum, String createBy) throws Exception {
        // 生成动态文字
        for (int i = 0; i < qrNum; i++) {
            String qrCode = UUID.randomUUID().toString().replaceAll("-", "");
            DeviceQrManage deviceQrManage = new DeviceQrManage();
            String serialNum = "mh_" + qrCode;
            deviceQrManage.setSerialNum(serialNum);
            deviceQrManage.setStatus(1);
            deviceQrManage.setCreateTime(new Date());
            deviceQrManage.setCreateBy(createBy);
            String currentPath = System.getProperty("user.dir");
            QrCodeUtil.save(serialNum, null, currentPath+ Constants.PROJECT_IMG_PATH +serialNum+".png");
            // 从文件中读取二维码图片
            File file = FileUtils.getFile(currentPath+ Constants.PROJECT_IMG_PATH +serialNum+".png");
            // File转成MultipartFile
            FileToMultipartConverter fileToMultipartConverter = new FileToMultipartConverter(file, "png");
            String avatar = FileUploadUtils.upload(MHConfig.getAvatarPath(), fileToMultipartConverter, MimeTypeUtils.IMAGE_EXTENSION);
            deviceQrManage.setPicContent(avatar);
            // 删除掉原始文件
            FileUtils.deleteQuietly(file);
            deviceQrManageMapper.insert(deviceQrManage);
        }
        return 0;
    }
}
