package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.MaintainInfo;
import com.mh.common.core.domain.entity.MaintainSum;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.MaintainInfoMapper;
import com.mh.system.service.device.IMaintainInfoService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备维护管理服务类
 * @date 2025-01-08 17:37:53
 */
@Service
public class MaintainInfoServiceImpl implements IMaintainInfoService {

    @Resource
    private MaintainInfoMapper maintainInfoMapper;

    @Override
    public List<MaintainInfo> selectMaintainInfoList(MaintainInfo maintainInfo) {
        if (maintainInfo == null) {
            return List.of();
        }
        QueryWrapper<MaintainInfo> queryWrapper = new QueryWrapper<>();
        // 维保类型
        if (maintainInfo.getMaintainType() != null) {
            queryWrapper.eq("maintain_type", maintainInfo.getMaintainType());
        }
        if (!StringUtils.isEmpty(maintainInfo.getBuildingId())) {
            queryWrapper.eq("building_id", maintainInfo.getBuildingId());
        }
        // 维保人员
        if (!StringUtils.isEmpty(maintainInfo.getMaintainPeople())) {
            queryWrapper.like("maintain_people", maintainInfo.getMaintainPeople());
        }
        // 设备类型
        if (!StringUtils.isEmpty(maintainInfo.getDeviceType())) {
            queryWrapper.eq("device_type", maintainInfo.getDeviceType());
        }
        // 创建时间（格式为 yyyy-MM-dd）
        // 添加时间范围
        if (maintainInfo.getParams() != null && !maintainInfo.getParams().isEmpty())  {
            String beginTimeStr = (String) maintainInfo.getParams().get("beginTime");
            String endTimeStr = (String) maintainInfo.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        queryWrapper.orderByDesc("create_time");
        return maintainInfoMapper.selectList(queryWrapper);
    }

    @Override
    public MaintainInfo selectMaintainInfoById(String maintainId) {
        return maintainInfoMapper.selectById(maintainId);
    }

    @Override
    public int insertMaintainInfo(MaintainInfo maintainInfo) {
        return maintainInfoMapper.insert(maintainInfo);
    }

    @Override
    public int updateMaintainInfo(MaintainInfo maintainInfo) {
        return maintainInfoMapper.updateById(maintainInfo);
    }

    @Override
    public int deleteMaintainInfoByIds(String[] maintainIds) {
        if (maintainIds != null) {
            for (String maintainId : maintainIds) {
                maintainInfoMapper.deleteById(maintainId);
            }
            return maintainIds.length;
        }
        return 0;
    }

    @Override
    public MaintainSum queryMaintainSum(String buildingId, String curDate) {
        return maintainInfoMapper.queryMaintainSum(buildingId, curDate);
    }

    @Override
    public void createMaintainTask(String curDate) {
        maintainInfoMapper.proMaintainSum(curDate);
    }
}
