package com.mh.system.service.device.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.SparePartsManage;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.SparePartsManageMapper;
import com.mh.system.service.device.ISparePartsManageService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 备件管理
 * @date 2025-01-09 11:47:05
 */
@Service
public class SparePartsManageServiceImpl implements ISparePartsManageService {

    @Resource
    private SparePartsManageMapper sparePartsManageMapper;

    @Override
    public List<SparePartsManage> selectSparePartsManageList(SparePartsManage sparePartsManage) {
        if (sparePartsManage == null) {
            return List.of();
        }
        QueryWrapper<SparePartsManage> queryWrapper = new QueryWrapper<>();
        // 序列号
        if (!StringUtils.isEmpty(sparePartsManage.getSerialNum())) {
            queryWrapper.like("serial_num", sparePartsManage.getSerialNum());
        }
        // 配件名称
        if (!StringUtils.isEmpty(sparePartsManage.getSpareName())) {
            queryWrapper.like("spare_name", sparePartsManage.getSpareName());
        }
        // 入库时间范围
        // 添加时间范围
        if (sparePartsManage.getParams() != null && !sparePartsManage.getParams().isEmpty())  {
            String beginTimeStr = (String) sparePartsManage.getParams().get("beginTime");
            String endTimeStr = (String) sparePartsManage.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        queryWrapper.orderByDesc("create_time");
        return sparePartsManageMapper.selectList(queryWrapper);
    }

    @Override
    public SparePartsManage selectSparePartsManageById(String spareId) {
        return sparePartsManageMapper.selectById(spareId);
    }

    @Override
    public int insertSparePartsManage(SparePartsManage sparePartsManage) {
        return sparePartsManageMapper.insert(sparePartsManage);
    }

    @Override
    public int updateSparePartsManage(SparePartsManage sparePartsManage) {
        return sparePartsManageMapper.updateById(sparePartsManage);
    }

    @Override
    public int deleteSparePartsManageByIds(String[] spareIds) {
        if (spareIds != null) {
            for (String spareId : spareIds) {
                sparePartsManageMapper.deleteById(spareId);
            }
            return spareIds.length;
        }
        return 0;
    }
}
