package com.mh.system.service.energy.impl;

import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.ConsumptionAnalyze;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.common.utils.DateUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.energy.EnergyQueryMapper;
import com.mh.system.service.energy.IEnergyQueryService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 能源查询服务实现类
 * @date 2023-12-13 15:54:39
 */
@Slf4j
@Service
public class EnergyQueryServiceImpl implements IEnergyQueryService {

    @Resource
    private EnergyQueryMapper energyQueryMapper;

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Override
    public AjaxResult sysQuery(EnergyQueryVO vo) {
        DateUtils.sysEnergyDateChange(vo);
        // 获取参数
        AtomicReference<String> lastTableName = new AtomicReference<>("data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        // 判断是否有总表
        boolean haveMeter = collectionParamsManageMapper.selectSummary(40, "5") != 0;
        boolean haveCloud = collectionParamsManageMapper.selectSummary(40, "6") != 0;
        List<ConsumptionAnalyze> consumptionAnalyzeEntities = null;
        // 表格数据
        if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
            // 单表
            consumptionAnalyzeEntities = energyQueryMapper.queryOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), null, haveMeter, haveCloud, vo.getSystemType());
        } else {
            lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
            curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
            if (lastTableName.get().equalsIgnoreCase(curTableName.get())) {
                // 单表
                consumptionAnalyzeEntities = energyQueryMapper.queryOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), null, haveMeter, haveCloud, vo.getSystemType());
            } else {
                // 多表
                consumptionAnalyzeEntities = energyQueryMapper.queryManyTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), null, haveMeter, haveCloud, vo.getSystemType());
            }
        }
        if (null == consumptionAnalyzeEntities || consumptionAnalyzeEntities.size() == 0) {
            return AjaxResult.success();
        }
        // 分组并按时间排序操作，拿到冷量记和电表数据
        Map<String, List<ConsumptionAnalyze>> collect = consumptionAnalyzeEntities.stream()
                .parallel()
                .collect(Collectors.groupingBy(ConsumptionAnalyze::getDeviceType, HashMap::new, Collectors
                        .collectingAndThen(Collectors.toList(),
                                list -> list.stream().sorted(Comparator.comparing(ConsumptionAnalyze::getTimeStr)).collect(Collectors.toList()))));
        List<ConsumptionAnalyze> meterData = new ArrayList<>();
        List<ConsumptionAnalyze> coldData = new ArrayList<>();
        for (Map.Entry<String, List<ConsumptionAnalyze>> nmap : collect.entrySet()) {
            // 获取电表的值
            if (nmap.getKey().equalsIgnoreCase("meter")) {
                meterData = nmap.getValue();
            }
            // 获取冷量计的值
            if (nmap.getKey().equalsIgnoreCase("cloud")) {
                coldData = nmap.getValue();
            }
        }
        String[] copArr = new String[meterData.size()];
        String[] timeStrArr = meterData.stream()
                .map(ConsumptionAnalyze::getTimeStr)
                .toArray(String[]::new);
        String[] meterArr = meterData.stream()
                .map(ConsumptionAnalyze::getCurValue)
                .toArray(String[]::new);
        String[] coldArr = getArr(coldData, timeStrArr);
        // 计算COP=制冷量/耗电量
        for (int i = 0; i < meterData.size(); i++) {
            try {
                double cold = Math.round(Double.parseDouble(coldArr[i]) * 100) / 100.0;
                double meter = Math.round(Double.parseDouble(meterArr[i]) * 100) / 100.0;
                double cop = Math.round((meter==0 ? 0.00 : cold/ meter) * 100) / 100.0;
                copArr[i] = String.valueOf(cop);
            } catch (NumberFormatException e) {
                log.error("处理累计能耗异常==>",e);
                throw new RuntimeException(e);
            }
        }
        // 表格数据
        Map<String, Object> map = new HashMap<>();
        int pageNum = vo.getPageNum();
        int pageSize = vo.getPageSize();
        if (pageNum == 0) {
            map.put("coldArr", coldArr);
            map.put("meterArr", meterArr);
            map.put("copArr", copArr);
            map.put("timeStrArr", timeStrArr);
        } else {
            int startIndex = (pageNum-1)*pageSize;
            int endIndex = Math.min(pageNum * pageSize, coldArr.length);
            if (startIndex > endIndex) {
                return AjaxResult.success();
            }
            map.put("coldArr", Arrays.copyOfRange(coldArr, startIndex , endIndex));
            map.put("meterArr", Arrays.copyOfRange(meterArr, startIndex , endIndex));
            map.put("copArr", Arrays.copyOfRange(copArr, startIndex , endIndex));
            map.put("timeStrArr", Arrays.copyOfRange(timeStrArr, startIndex , endIndex));
        }

        // 组装赋值
        List<Map<String, Object>> listData = new ArrayList<>();
        Map<String, Object> cold = new HashMap<>();
        cold.put("cold", map.get("coldArr"));
        listData.add(cold);
        Map<String, Object> meter = new HashMap<>();
        meter.put("meter", map.get("meterArr"));
        listData.add(meter);
        Map<String, Object> cop = new HashMap<>();
        cop.put("cop", map.get("copArr"));
        listData.add(cop);
        String[] titleArr = new String[]{"cold", "meter", "cop"};
        Map<String, Object> titles = new HashMap<>();
        titles.put("titleArr", titleArr);
        listData.add(titles);
        Map<String, Object> timeStr = new HashMap<>();
        timeStr.put("timeStrArr", map.get("timeStrArr"));
        listData.add(timeStr);
        return AjaxResult.success(listData);
    }

    private static String[] getArr(List<ConsumptionAnalyze> copLineData, String[] lineTimeStrArr) {
        String[] lineCopArr = new String[lineTimeStrArr.length];
        for (int i = 0; i < lineTimeStrArr.length; i++) {
            int j = i;
            Optional<ConsumptionAnalyze> first = copLineData.stream().filter(s -> lineTimeStrArr[j].equalsIgnoreCase(s.getTimeStr())).findFirst();
            if (first.isPresent()) {
                lineCopArr[i] = first.get().getCurValue();
            } else {
                lineCopArr[i] = "0.00";
            }
        }
        return lineCopArr;
    }

    @Override
    public AjaxResult deviceTypeQuery(EnergyQueryVO page) {
        return null;
    }
}
