package com.mh.system.service.energy.impl;

import com.alibaba.fastjson2.JSONObject;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.ColumnData;
import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.dto.EnergyConsumptionDTO;
import com.mh.common.core.domain.entity.ConsumptionAnalyze;
import com.mh.common.core.domain.entity.DeviceTypeEnergy;
import com.mh.common.core.domain.vo.EnergyConsumptionVO;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.EnergyThreadPoolService;
import com.mh.common.utils.StringUtils;
import com.mh.common.utils.bean.BeanUtils;
import com.mh.system.mapper.energy.EnergyAnalyzeMapper;
import com.mh.system.mapper.energy.EnergyMapper;
import com.mh.system.service.ISysDictDataService;
import com.mh.system.service.energy.IEnergyService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project NewZhujiang_Server
 * @description 系统能耗分析
 * @date 2023/7/18 14:08:10
 */
@Slf4j
@Service
public class EnergyServiceImpl implements IEnergyService {

    @Resource
    private EnergyMapper energyMapper;

    @Resource
    private EnergyAnalyzeMapper energyAnalyzeMapper;

    @Resource
    private ISysDictDataService iSysDictDataService;

    private static String[] getArrChillerLine(List<DeviceTypeEnergy> copLineData, String[] lineTimeStrArr) {
        String[] lineCopArr = new String[lineTimeStrArr.length];
        for (int i = 0; i < lineTimeStrArr.length; i++) {
            int j = i;
            Optional<DeviceTypeEnergy> first = copLineData.stream().filter(s -> lineTimeStrArr[j].equalsIgnoreCase(s.getTimeStr())).findFirst();
            if (first.isPresent()) {
                lineCopArr[i] = first.get().getCalcValue();
            } else {
                lineCopArr[i] = "0.00";
            }
        }
        return lineCopArr;
    }

    @Override
    public AjaxResult chillerLine(EnergyQueryVO vo) {
        DateUtils.sysEnergyDateChange(vo);
        // 获取参数
        List<ColumnFilter> paramsMap = vo.getSearchParams();
        if (paramsMap.isEmpty()) {
            return AjaxResult.success(new ArrayList<>());
        }
        List<Object> params = paramsMap.stream().map(ColumnFilter::getValue).collect(Collectors.toList());
        AtomicReference<String> lastTableName = new AtomicReference<>("chillers_data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("chillers_data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        List<DeviceTypeEnergy> deviceTypeEnergyEntities = null;
        // 表格数据
        if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
            // 单表
            deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
        } else {
            lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
            curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
            if (lastTableName.get().equalsIgnoreCase(curTableName.get())) {
                // 单表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
            } else {
                // 多表
                deviceTypeEnergyEntities = energyAnalyzeMapper.queryDeviceManyTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), DateUtils.getTimeLen(vo.getTimeType()), params, vo.getSystemType());
            }
        }
        if (null == deviceTypeEnergyEntities || deviceTypeEnergyEntities.isEmpty()) {
            return AjaxResult.success(new ArrayList<>());
        }
        // 分组拿出各个设备
        Map<String, List<DeviceTypeEnergy>> deviceNumMap = deviceTypeEnergyEntities.stream()
                .sorted(Comparator.comparing(DeviceTypeEnergy::getTimeStr))
                .collect(Collectors.groupingBy(DeviceTypeEnergy::getDeviceNum));

        List<String> title = new ArrayList<>();
        List<String> titleStr = new ArrayList<>();
        List<String> timeStr = new ArrayList<>();
        List<ColumnData> columnData = new ArrayList<>();
        // 组装对应需要查询的数据
        int i = 0;
        for (Map.Entry<String, List<DeviceTypeEnergy>> stringListEntry : deviceNumMap.entrySet()) {
            title.add(stringListEntry.getKey());
            List<DeviceTypeEnergy> value = stringListEntry.getValue();
            ColumnData column = new ColumnData();
            column.setName(stringListEntry.getKey());
            if (value.size() > 1) {
                titleStr.add(value.getFirst().getRemark());
            }
            for (DeviceTypeEnergy deviceTypeEnergy : value) {
                if (i == 0) {
                    timeStr.add(deviceTypeEnergy.getTimeStr());
                } else {
                    break;
                }
            }
            int pageNum = Integer.parseInt(String.valueOf(vo.getPageNum()));
            int pageSize = Integer.parseInt(String.valueOf(vo.getPageSize()));
            if (pageNum != 0) {
                int startIndex = (pageNum-1)*pageSize;
                int endIndex = Math.min(pageNum * pageSize, timeStr.size());
                value = value.subList(startIndex, endIndex);
                column.setValue(getArrChillerLine(value, timeStr.subList(startIndex, endIndex).toArray(new String[0])));
            } else {
                column.setValue(getArrChillerLine(value, timeStr.toArray(new String[0])));
            }
            columnData.add(column);
            i++;
        }
        int pageNum = Integer.parseInt(String.valueOf(vo.getPageNum()));
        int pageSize = Integer.parseInt(String.valueOf(vo.getPageSize()));
        Map<String, Object> map = new JSONObject();
        map.put("title", title);
        map.put("titleStr", titleStr);
        if (pageNum == 0) {
            map.put("timeStr", timeStr);
            map.put("dataList", columnData);
        } else {
            int startIndex = (pageNum-1) * pageSize;
            int endIndex = Math.min(pageNum * pageSize, timeStr.size());
            if (startIndex > endIndex) {
                return AjaxResult.success(new ArrayList<>(map.entrySet()));
            }
            map.put("timeStr", timeStr.subList(startIndex, endIndex));
            map.put("dataList", columnData);
        }
        return AjaxResult.success(new ArrayList<>(map.entrySet()));
    }

    @Override
    public AjaxResult device(EnergyConsumptionVO vo) {
        AtomicReference<String> lastTableName = new AtomicReference<>("data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        ThreadPoolExecutor executor = EnergyThreadPoolService.getInstance();
        CountDownLatch latch = new CountDownLatch(2);
        List<Future<Map<String, Object>>> futures = new ArrayList<>();
        futures.add(executor.submit(() -> {
            List<ConsumptionAnalyze> consumptionAnalyzeEntities;
            if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
                // 单表
                consumptionAnalyzeEntities = energyMapper.queryDeviceOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), getTimeLen(vo.getTimeType()), vo.getDeviceType());
            } else {
                // 多表
                lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
                curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
                consumptionAnalyzeEntities = energyMapper.queryDeviceManyTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), getTimeLen(vo.getTimeType()), vo.getDeviceType(), vo.getSystemType());
            }
            if (null == consumptionAnalyzeEntities || consumptionAnalyzeEntities.size() == 0) {
                latch.countDown();
                return null;
            }
            // 组装赋值
            String[] timeStrArr = consumptionAnalyzeEntities.stream()
                    .map(ConsumptionAnalyze::getTimeStr)
                    .toArray(String[]::new);
            String[] meterArr = consumptionAnalyzeEntities.stream()
                    .map(ConsumptionAnalyze::getCurValue)
                    .toArray(String[]::new);
            Map<String, Object> map = new HashMap<>();
            map.put("timeStrArr", timeStrArr);
            map.put("meterArr", meterArr);
            latch.countDown();
            return map;
        }));
        futures.add(executor.submit(() -> {
            // 折线图，都查询min表
            // 多表
            String lastTable = "data_min" + vo.getStartTime().substring(0,4);
            String curTable = "data_min" + vo.getEndTime().substring(0,4);
            List<ConsumptionAnalyze> consumptionAnalyzeEntities = energyMapper.queryDeviceLineManyTable(vo.getStartTime(), vo.getEndTime(), lastTable, curTable, vo.getDeviceType(), vo.getSystemType());
            if (null == consumptionAnalyzeEntities || consumptionAnalyzeEntities.size() == 0) {
                latch.countDown();
                return null;
            }
            // 组装赋值
            String[] timeStrArr = consumptionAnalyzeEntities.stream()
                    .map(ConsumptionAnalyze::getTimeStr)
                    .toArray(String[]::new);
            String[] meterArr = consumptionAnalyzeEntities.stream()
                    .map(ConsumptionAnalyze::getCurValue)
                    .toArray(String[]::new);
            Map<String, Object> map = new HashMap<>();
            map.put("timeStrLineArr", timeStrArr);
            map.put("meterLineArr", meterArr);
            latch.countDown();
            return map;
        }));

        try {
            latch.await();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }

        Map<String, Object> allData = new HashMap<>();
        // 获取数据
        for (Future<Map<String, Object>> future : futures) {
            try {
                Map<String, Object> map = future.get();
                if (null != map) {
                    allData.putAll(map);
                }
            } catch (InterruptedException | ExecutionException e) {
                throw new RuntimeException(e);
            }
        }

        EnergyConsumptionDTO energyConsumptionDTO = new EnergyConsumptionDTO();
        String[] titleArr = new String[]{"meter", "lineMeter"};
        energyConsumptionDTO.setTitles(titleArr);
        energyConsumptionDTO.setTimes((String[]) allData.get("timeStrArr"));
        energyConsumptionDTO.setLineTimes((String[]) allData.get("timeStrLineArr"));
        List<Map<String, Object>> listData = new ArrayList<>();
        Map<String, Object> meter = new HashMap<>();
        meter.put("meter",allData.get("meterArr"));
        listData.add(meter);
        Map<String, Object> lineMeter = new HashMap<>();
        lineMeter.put("lineMeter",allData.get("meterLineArr"));
        listData.add(lineMeter);
        energyConsumptionDTO.setData(listData);
        return AjaxResult.success(energyConsumptionDTO);
    }

    @Override
    public AjaxResult sys(EnergyConsumptionVO vo) {
        AtomicReference<String> lastTableName = new AtomicReference<>("data_" + vo.getTimeType());
        AtomicReference<String> curTableName = new AtomicReference<>("data_" + vo.getTimeType());
        String timeType = vo.getTimeType();
        EnergyConsumptionDTO energyConsumptionDTO = new EnergyConsumptionDTO();
        String[] titleArr = new String[]{"cold", "meter", "cop", "lineCop", "lineInstantaneousCold", "lineInstantaneousMeter"};
        ThreadPoolExecutor executor = EnergyThreadPoolService.getInstance();
        CountDownLatch latch = new CountDownLatch(2);
        List<Future<Map<String, Object>>> futures = new ArrayList<>();
        futures.add(executor.submit(() -> {
            List<ConsumptionAnalyze> consumptionAnalyzeEntities = null;
            // 表格数据
            if ("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType)) {
                // 单表
                consumptionAnalyzeEntities = energyMapper.queryOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), getTimeLen(vo.getTimeType()), null, vo.getSystemType());
            } else {
                lastTableName.set(lastTableName + vo.getStartTime().substring(0, 4));
                curTableName.set(curTableName + vo.getEndTime().substring(0, 4));
                if (lastTableName.get().equalsIgnoreCase(curTableName.get())) {
                    // 单表
                    consumptionAnalyzeEntities = energyMapper.queryOneTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), getTimeLen(vo.getTimeType()), null, vo.getSystemType());
                } else {
                    // 多表
                    consumptionAnalyzeEntities = energyMapper.queryManyTable(vo.getStartTime(), vo.getEndTime(), lastTableName.get(), curTableName.get(), getTimeLen(vo.getTimeType()), null, vo.getSystemType());
                }
            }
            if (null == consumptionAnalyzeEntities || consumptionAnalyzeEntities.isEmpty()) {
                latch.countDown();
                return null;
            }
            // 分组并按时间排序操作，拿到冷量记和电表数据
            Map<String, List<ConsumptionAnalyze>> collect = consumptionAnalyzeEntities.stream()
                    .parallel()
                    .collect(Collectors.groupingBy(ConsumptionAnalyze::getDeviceType, HashMap::new, Collectors
                            .collectingAndThen(Collectors.toList(),
                                    list -> list.stream().sorted(Comparator.comparing(ConsumptionAnalyze::getTimeStr)).collect(Collectors.toList()))));
            List<ConsumptionAnalyze> meterData = new ArrayList<>();
            List<ConsumptionAnalyze> coldData = new ArrayList<>();
            for (Map.Entry<String, List<ConsumptionAnalyze>> nmap : collect.entrySet()) {
                // 获取电表的值
                if (nmap.getKey().equalsIgnoreCase("meter")) {
                    meterData = nmap.getValue();
                }
                // 获取冷量计的值
                if (nmap.getKey().equalsIgnoreCase("cloud")) {
                    coldData = nmap.getValue();
                }
            }
            String[] copArr = new String[meterData.size()];
            String[] timeStrArr = meterData.stream()
                    .map(ConsumptionAnalyze::getTimeStr)
                    .toArray(String[]::new);
            String[] meterArr = meterData.stream()
                    .map(ConsumptionAnalyze::getCurValue)
                    .toArray(String[]::new);
            String[] coldArr = getArr(coldData, timeStrArr);
            // 计算COP=制冷量/耗电量
            for (int i = 0; i < meterData.size(); i++) {
                try {
                    double cold = Math.round(Double.parseDouble(coldArr[i]) * 100) / 100.0;
                    double meter = Math.round(Double.parseDouble(meterArr[i]) * 100) / 100.0;
                    double cop = Math.round((meter==0 ? 0.00 : cold/ meter) * 100) / 100.0;
                    copArr[i] = String.valueOf(cop);
                } catch (NumberFormatException e) {
                    log.error("处理累计能耗异常==>",e);
                    throw new RuntimeException(e);
                }
            }
            // 表格数据
            Map<String, Object> map = new HashMap<>();
            map.put("coldArr", coldArr);
            map.put("meterArr", meterArr);
            map.put("copArr", copArr);
            map.put("timeStrArr", timeStrArr);
            latch.countDown();
            return map;
        }));
        futures.add(executor.submit(() -> {
            // 折线图数据，都是查询的分钟表，最低颗粒度
            List<ConsumptionAnalyze> lineData;
            String lastTable = "data_min" + vo.getStartTime().substring(0, 4);
            String curTable = "data_min" + vo.getEndTime().substring(0, 4);
            String lastCopTable = "data_min_cop";
            String curCopTable = "data_min_cop";
            if (lastTable.equalsIgnoreCase(curTable)) {
                // 查询单表
                lineData = energyMapper.queryLineDataSysByOne(curTable, curCopTable, vo.getStartTime(), vo.getEndTime());
            } else {
                // 查询多表
                lineData = energyMapper.queryLineDataSysByMany(lastTable, curTable, lastCopTable, curCopTable, vo.getStartTime(), vo.getEndTime());
            }
            if (null == lineData) {
                latch.countDown();
                return null;
            }
            // 分组并按时间排序操作，拿到冷量记，电表，cop数据
            Map<String, List<ConsumptionAnalyze>> collectLine = lineData.stream()
                    .parallel()
                    .collect(Collectors.groupingBy(ConsumptionAnalyze::getDeviceType, HashMap::new, Collectors
                            .collectingAndThen(Collectors.toList(),
                                    list -> list.stream().sorted(Comparator.comparing(ConsumptionAnalyze::getTimeStr)).collect(Collectors.toList()))));
            List<ConsumptionAnalyze> efrColdLineData = new ArrayList<>();
            List<ConsumptionAnalyze> efrMeterLineData = new ArrayList<>();
            for (Map.Entry<String, List<ConsumptionAnalyze>> nmap : collectLine.entrySet()) {
                // 获取瞬时冷量计的值
                if (nmap.getKey().equalsIgnoreCase("efrCloud")) {
                    efrColdLineData = nmap.getValue();
                }
                // 获取瞬时功率的值
                if (nmap.getKey().equalsIgnoreCase("efrMeter")) {
                    efrMeterLineData = nmap.getValue();
                }
            }
            String[] lineTimeStrArr = efrMeterLineData.stream()
                    .map(ConsumptionAnalyze::getTimeStr)
                    .toArray(String[]::new);
            String[] lineInstantaneousMeterArr = getArr(efrMeterLineData, lineTimeStrArr);
            String[] lineEfrColdArr = getArr(efrColdLineData, lineTimeStrArr);
            String[] lineCopArr = new String[efrMeterLineData.size()];
            // 计算瞬时COP=制冷量/耗电量
            for (int i = 0; i < efrMeterLineData.size(); i++) {
                try {
                    double cold = Math.round(Double.parseDouble(lineEfrColdArr[i]) * 100) / 100.0;
                    double meter = Math.round(Double.parseDouble(lineInstantaneousMeterArr[i]) * 100) / 100.0;
                    double cop = Math.round((meter==0 ? 0.00 : cold/ meter) * 100) / 100.0;
                    lineCopArr[i] = String.valueOf(cop);
                } catch (NumberFormatException e) {
                    log.error("处理累计能耗异常==>",e);
                    throw new RuntimeException(e);
                }
            }
            // 折现图数据
            Map<String, Object> map = new HashMap<>();
            map.put("lineTimeStrArr", lineTimeStrArr);
            map.put("lineCopArr", lineCopArr);
            map.put("lineInstantaneousColdArr", lineEfrColdArr);
            map.put("lineInstantaneousMeterArr", lineInstantaneousMeterArr);
            latch.countDown();
            return map;
        }));

        try {
            latch.await();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }

        Map<String, Object> allData = new HashMap<>();
        // 获取数据
        for (Future<Map<String, Object>> future : futures) {
            try {
                Map<String, Object> map = future.get();
                if (null != map) {
                    allData.putAll(map);
                }
            } catch (InterruptedException | ExecutionException e) {
                throw new RuntimeException(e);
            }
        }

        // 组装赋值
        energyConsumptionDTO.setTitles(titleArr);
        energyConsumptionDTO.setTimes((String[]) allData.get("timeStrArr"));
        energyConsumptionDTO.setLineTimes((String[]) allData.get("lineTimeStrArr"));
        List<Map<String, Object>> listData = new ArrayList<>();
        Map<String, Object> cold = new HashMap<>();
        cold.put("cold", allData.get("coldArr"));
        listData.add(cold);
        Map<String, Object> meter = new HashMap<>();
        meter.put("meter", allData.get("meterArr"));
        listData.add(meter);
        Map<String, Object> cop = new HashMap<>();
        cop.put("cop", allData.get("copArr"));
        listData.add(cop);
        Map<String, Object> coldLine = new HashMap<>();
        coldLine.put("lineCold", allData.get("lineColdArr"));
        listData.add(coldLine);
        Map<String, Object> meterLine = new HashMap<>();
        meterLine.put("lineMeter", allData.get("lineMeterArr"));
        listData.add(meterLine);
        Map<String, Object> copLine = new HashMap<>();
        copLine.put("lineCop",allData.get("lineCopArr"));
        listData.add(copLine);
        Map<String, Object> instantaneousColdLine = new HashMap<>();
        instantaneousColdLine.put("lineInstantaneousCold",allData.get("lineInstantaneousColdArr"));
        listData.add(instantaneousColdLine);
        Map<String, Object> instantaneousMeterLine = new HashMap<>();
        instantaneousMeterLine.put("lineInstantaneousMeter",allData.get("lineInstantaneousMeterArr"));
        listData.add(instantaneousMeterLine);
        energyConsumptionDTO.setData(listData);
        return AjaxResult.success(energyConsumptionDTO);
    }

    private static String[] getArr(List<ConsumptionAnalyze> copLineData, String[] lineTimeStrArr) {
        String[] lineCopArr = new String[lineTimeStrArr.length];
        for (int i = 0; i < lineTimeStrArr.length; i++) {
            int j = i;
            Optional<ConsumptionAnalyze> first = copLineData.stream().filter(s -> lineTimeStrArr[j].equalsIgnoreCase(s.getTimeStr())).findFirst();
            if (first.isPresent()) {
                lineCopArr[i] = first.get().getCurValue();
            } else {
                lineCopArr[i] = "0.00";
            }
        }
        return lineCopArr;
    }

    @Override
    public AjaxResult yoy(EnergyConsumptionVO vo) {
        // 获取今年的值
        List<ConsumptionAnalyze> consumptionAnalyzeEntities;
        String lastTableName = "data_"+vo.getTimeType();
        String curTableName = "data_"+vo.getTimeType();
        String startTime = vo.getStartTime();
        String endTime = vo.getEndTime();
        String deviceType = vo.getParamType();
        if ("meter".equals(deviceType)) {
            deviceType = "5";
        } else if ("cloud".equals(deviceType)) {
            deviceType = "6";
        }
        if ("cop".equalsIgnoreCase(vo.getParamType())) {
            return calcCopYoyData(vo);
//            lastTableName = "data_min_cop";
//            curTableName = "data_min_cop";
//            deviceType = "system";
        } else {
            lastTableName = lastTableName+startTime.substring(0,4);
            curTableName = curTableName+endTime.substring(0,4);
        }
        String timeType = vo.getTimeType();
        if (!(lastTableName.equalsIgnoreCase(curTableName)) && ("hour".equalsIgnoreCase(timeType) || "day".equalsIgnoreCase(timeType))) {
            // 多表
            // 获取上一期的时间
            startTime = DateUtils.yoyDate(startTime);
            endTime = DateUtils.yoyDate(endTime);
            String yoyLastTableName = lastTableName.substring(0, lastTableName.length() - 4) + startTime.substring(0,4);
            String yoyCurTableName = curTableName.substring(0, lastTableName.length() - 4) + endTime.substring(0,4);
            consumptionAnalyzeEntities = energyMapper.queryManyTableYoy(vo.getStartTime(), vo.getEndTime(), lastTableName, curTableName,
                    startTime, endTime, yoyLastTableName, yoyCurTableName, getTimeLen(vo.getTimeType()), deviceType, vo.getTimeType(), vo.getSystemType());
        } else  {
            // 单表
            startTime = DateUtils.yoyDate(startTime);
            endTime = DateUtils.yoyDate(endTime);
            if (!"cop".equalsIgnoreCase(vo.getParamType())) {
                String lastTable = lastTableName.substring(0, lastTableName.length() - 4);
                if (("month".equalsIgnoreCase(timeType) || "year".equalsIgnoreCase(timeType))) {
                    lastTableName = lastTable;
                    curTableName = curTableName.substring(0, curTableName.length() - 4);
                } else {
                    lastTableName = lastTable + startTime.substring(0, 4);
                }
            } else {
                deviceType = "system";
            }
            consumptionAnalyzeEntities = energyMapper.queryOneTableYoy(vo.getStartTime(), vo.getEndTime(), lastTableName, curTableName,
                    startTime, endTime, getTimeLen(vo.getTimeType()), deviceType, vo.getTimeType(), vo.getSystemType());
        }
        return getConsumptionDTOHttpResult(consumptionAnalyzeEntities, "yoy", getTimeLen(vo.getTimeType()));
    }

    private AjaxResult calcCopYoyData(EnergyConsumptionVO vo) {
        // 获取冷量读数
        vo.setParamType("6");
        AjaxResult cloudData = yoy(vo);
        EnergyConsumptionDTO cloudEnergyData = (EnergyConsumptionDTO) cloudData.get("data");
        if (cloudEnergyData == null) {
            return AjaxResult.error();
        }
        // 获取电量读数
        vo.setParamType("5");
        AjaxResult meterData = yoy(vo);
        EnergyConsumptionDTO meterEnergyData = (EnergyConsumptionDTO) meterData.get("data");;
        if (meterEnergyData == null) {
            return AjaxResult.error();
        }
        // 计算COP值
        return getEnergyConsumptionDTOHttpResult(meterEnergyData, cloudEnergyData, "yoy");
    }

    private static AjaxResult getEnergyConsumptionDTOHttpResult(EnergyConsumptionDTO meterEnergyData, EnergyConsumptionDTO cloudEnergyData, String type) {
        // 计算COP值
        EnergyConsumptionDTO energyConsumptionDTO = new EnergyConsumptionDTO();
        //
        // 获取冷量数据
        List<Map<String, Object>> cloudListData = cloudEnergyData.getData();
        String[] curStr = (String[]) cloudListData.getFirst().get("curValue");
        String[] cloudCurData = new String[curStr.length];
        String[] cloudLastData = new String[curStr.length];
        for (Map<String, Object> cloudListDatum : cloudListData) {
            if (cloudListDatum.get("curValue") != null) {
                cloudCurData = (String[]) cloudListDatum.get("curValue");
                if (cloudCurData == null) {
                    cloudCurData = new String[curStr.length];
                }
            }
            if (cloudListDatum.get("lastValue") != null) {
                cloudLastData = (String[]) cloudListDatum.get("curValue");
                if (cloudLastData == null) {
                    cloudLastData = new String[curStr.length];
                }
            }
        }
        // 获取电量数据
        List<Map<String, Object>> meterListData = meterEnergyData.getData();
        String[] meterCurStr = (String[]) meterListData.getFirst().get("curValue");
        String[] meterCurData = new String[meterCurStr.length];
        String[] meterLastData = new String[meterCurStr.length];
        for (Map<String, Object> meterListDatum : meterListData) {
            if (meterListDatum.get("curValue") != null) {
                meterCurData = (String[]) meterListDatum.get("curValue");
                if (meterCurData == null) {
                    meterCurData = new String[meterCurStr.length];
                }
            }
            if (meterListDatum.get("lastValue") != null) {
                meterLastData = (String[]) meterListDatum.get("curValue");
                if (meterLastData == null) {
                    meterLastData = new String[meterCurStr.length];
                }
            }
        }

        // 初始化数据参数，当其中某个参数为空时，初始化数据为0
        String[] timeA = cloudEnergyData.getTimes();
        String[] timeB = meterEnergyData.getTimes();
        List<String> timeAList = new ArrayList<>(Arrays.asList(timeA));
        List<String> cloudCurDataList = new ArrayList<>(Arrays.asList(cloudCurData));
        List<String> cloudLastDataList = new ArrayList<>(Arrays.asList(cloudLastData));
        List<String> timeBList = new ArrayList<>(Arrays.asList(timeB));
        List<String> meterCurDataList = new ArrayList<>(Arrays.asList(meterCurData));
        List<String> meterLastDataList = new ArrayList<>(Arrays.asList(meterLastData));

        if (timeAList.size() < timeBList.size()) {
            for (String time : timeBList) {
                if (!timeAList.contains(time)) {
                    timeAList.add(time);
                    cloudCurDataList.add("0");
                }
            }
            meterEnergyData.setTimes(timeAList.toArray(new String[0]));
            cloudCurData = cloudCurDataList.toArray(new String[0]);
            cloudLastData = cloudLastDataList.toArray(new String[0]);
        } else if (timeBList.size() < timeAList.size()) {
            for (String time : timeAList) {
                if (!timeBList.contains(time)) {
                    timeBList.add(time);
                    meterCurDataList.add("0");
                    meterLastDataList.add("0");
                }
            }
            meterEnergyData.setTimes(timeBList.toArray(new String[0]));
            meterCurData = meterCurDataList.toArray(new String[0]);
            meterLastData = meterLastDataList.toArray(new String[0]);
        }
        energyConsumptionDTO.setTitles(meterEnergyData.getTitles());
        energyConsumptionDTO.setTimes(meterEnergyData.getTimes());
        energyConsumptionDTO.setLineTimes(meterEnergyData.getLineTimes());

        // 直接两个相除，保留两位小数
        if (cloudCurData.length == meterCurData.length) {
            String[] copMap = new String[meterCurData.length];
            String[] copLastMap = new String[meterCurData.length];
            String[] yoyMap = new String[meterCurData.length];
            for (int i = 0; i < meterCurData.length; i++) {
                String curValue = StringUtils.isBlank(cloudCurData[i]) ? "0" : cloudCurData[i];
                String lastValue = cloudLastData == null || StringUtils.isBlank(cloudLastData[i]) ? "0" : cloudLastData[i];
                String curMeterValue = StringUtils.isBlank(meterCurData[i]) ? "0" : meterCurData[i];
                String lastMeterValue = meterLastData == null || StringUtils.isBlank(meterLastData[i]) ? "0" : meterLastData[i];
                String curCop = "0.00";
                if (StringUtils.isBlank(curMeterValue) || new BigDecimal(curMeterValue).compareTo(BigDecimal.ZERO) == 0) {
                    curCop = "0.00";
                } else {
                    curCop = new BigDecimal(curValue).divide(new BigDecimal(curMeterValue), 2, RoundingMode.HALF_UP).toString();
                }
                String lastCop = "0.00";
                if (StringUtils.isBlank(lastMeterValue) || new BigDecimal(lastMeterValue).compareTo(BigDecimal.ZERO) == 0) {
                    lastCop = "0.00";
                } else {
                    lastCop = new BigDecimal(lastValue).divide(new BigDecimal(lastMeterValue), 2, RoundingMode.HALF_UP).toString();
                }
                // 同比操作
                String yoy = "0.00%";
                if (new BigDecimal(lastCop).compareTo(BigDecimal.ZERO) == 0) {
                    yoy = "0.00%";
                } else {
                    yoy = new BigDecimal(curCop).subtract(new BigDecimal(lastCop)).divide(new BigDecimal(lastCop), 2, RoundingMode.HALF_UP).multiply(new BigDecimal(100)).toString() + "%";
                }
                copMap[i] = curCop;
                copLastMap[i] = lastCop;
                yoyMap[i] = yoy;
            }
//            energyConsumptionDTO.setData(Arrays.asList(
//                    new HashMap<String, Object>() {{
//                        put("curValue", copMap);
//                        put("lastValue", copLastMap);
//                        put("yoy", yoyMap);
//                    }}
//            ));
            List<Map<String, Object>> listData = new ArrayList<>();
            Map<String, Object> cur = new HashMap<>();
            cur.put("curValue",copMap);
            listData.add(cur);
            Map<String, Object> last = new HashMap<>();
            last.put("lastValue",copLastMap);
            listData.add(last);
            Map<String, Object> yoy = new HashMap<>();
            yoy.put(type,yoyMap);
            listData.add(yoy);
            energyConsumptionDTO.setData(listData);
        }
        return AjaxResult.success(energyConsumptionDTO);
    }

    @Override
    public AjaxResult mom(EnergyConsumptionVO vo) {
        List<ConsumptionAnalyze> consumptionAnalyzeEntities = null;
        // 开始时间和结束时间都往前-1月或者年
        String startTime = DateUtils.momDate(vo.getStartTime(), vo.getTimeType(), "start");
        String endTime = DateUtils.momDate(vo.getEndTime(), vo.getTimeType(), "end");
        String deviceType = vo.getParamType();
        if ("meter".equals(deviceType)) {
            deviceType = "5";
        } else if ("cloud".equals(deviceType)){
            deviceType = "6";
        }
        // 判断查询时间类型是month，year
        if ("month".equalsIgnoreCase(vo.getTimeType()) || "year".equalsIgnoreCase(vo.getTimeType())) {
            String tableName = "data_"+vo.getTimeType();
            // 计算COP值(后期有一个表存储)
            if (!StringUtils.isBlank(vo.getParamType()) && vo.getParamType().equalsIgnoreCase("cop")) {
//                tableName = "data_min_cop";
//                deviceType = "system";
                return calcCopMomData(vo);
            }
            consumptionAnalyzeEntities = energyMapper.queryMonthAndYearMom(vo.getStartTime(), vo.getEndTime(), startTime, endTime, tableName, getTimeLen(vo.getTimeType()), deviceType, vo.getTimeType(), vo.getSystemType());
        } else {
            // 判斷是否需要進行連表查詢
            String tableName1 = "data_"+vo.getTimeType()+startTime.substring(0, 4);
            String tableName2 = "data_"+vo.getTimeType()+vo.getEndTime().substring(0, 4);
            // 计算COP值(后期有一个表存储)
            if (!StringUtils.isBlank(vo.getParamType()) && vo.getParamType().equalsIgnoreCase("cop")) {
//                tableName1 = "data_min_cop";
//                tableName2 = "data_min_cop";
//                deviceType = "system";
                return calcCopMomData(vo);
            }
            if (tableName1.equalsIgnoreCase(tableName2)) {
                // 单表查询
                consumptionAnalyzeEntities = energyMapper.queryOneTableMom(vo.getStartTime(), vo.getEndTime(), startTime, endTime, tableName1, getTimeLen(vo.getTimeType()), deviceType, vo.getTimeType(), vo.getSystemType());
            } else {
                // 多表查询
                consumptionAnalyzeEntities = energyMapper.queryManyTableMom(vo.getStartTime(), vo.getEndTime(), startTime, endTime, tableName1, tableName2, getTimeLen(vo.getTimeType()), deviceType, vo.getTimeType(), vo.getSystemType());
            }
        }
        return getConsumptionDTOHttpResult(consumptionAnalyzeEntities, "mom", getTimeLen(vo.getTimeType()));
    }

    private AjaxResult calcCopMomData(EnergyConsumptionVO vo) {
        // 获取冷量读数
        vo.setParamType("6");
        AjaxResult cloudData = mom(vo);
        EnergyConsumptionDTO cloudEnergyData = (EnergyConsumptionDTO) cloudData.get("data");
        if (cloudEnergyData == null) {
            return AjaxResult.error();
        }
        // 获取电量读数
        vo.setParamType("5");
        AjaxResult meterData = mom(vo);
        EnergyConsumptionDTO meterEnergyData = (EnergyConsumptionDTO) meterData.get("data");
        if (meterEnergyData == null) {
            return AjaxResult.error();
        }

        return getEnergyConsumptionDTOHttpResult(meterEnergyData, cloudEnergyData, "mom");
    }

    @Override
    public AjaxResult yoyMom(EnergyConsumptionVO vo) {
        // 多线程分别求出yoy和mom
        ThreadPoolExecutor executor = EnergyThreadPoolService.getInstance();
        CountDownLatch latch = new CountDownLatch(2);
        List<Future<Map<String, Object>>> futures = new ArrayList<>();
        futures.add(executor.submit(() -> {
            Map<String, Object> map = new HashMap<>();
            EnergyConsumptionVO vo1 = new EnergyConsumptionVO();
            BeanUtils.copyProperties(vo, vo1);
            map.put("yoy", yoy(vo1));
            latch.countDown();
            return map;
        }));
        futures.add(executor.submit(() -> {
            Map<String, Object> map = new HashMap<>();
            EnergyConsumptionVO vo1 = new EnergyConsumptionVO();
            BeanUtils.copyProperties(vo, vo1);
            map.put("mom", mom(vo1));
            latch.countDown();
            return map;
        }));

        try {
            latch.await();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        }
        Map<String, Object> allData = new HashMap<>();
        // 获取数据
        for (Future<Map<String, Object>> future : futures) {
            try {
                Map<String, Object> map = future.get();
                if (null != map) {
                    allData.putAll(map);
                }
            } catch (InterruptedException | ExecutionException e) {
                throw new RuntimeException(e);
            }
        }
        // 封装数据
        if (allData.containsKey("yoy") && allData.containsKey("mom")) {
            AjaxResult yoy = (AjaxResult) allData.get("yoy");
            AjaxResult mom = (AjaxResult) allData.get("mom");
            if (yoy.isSuccess() && mom.isSuccess()) {
                EnergyConsumptionDTO result = new EnergyConsumptionDTO();
                EnergyConsumptionDTO yoyData = (EnergyConsumptionDTO) yoy.get("data");
                EnergyConsumptionDTO momData = (EnergyConsumptionDTO) mom.get("data");
                if (yoyData.getData()==null || momData.getData()==null) {
                    return AjaxResult.success(result);
                }
                result.setTitles(new String[]{"curValue", "lastValue", "yoy", "mom"});
                result.setTimes(yoyData.getTimes());
                result.setLineTimes(yoyData.getLineTimes());
                List<Map<String, Object>> data = new ArrayList<>();
                // 同比数据
                Map<String, Object> lastValueMap = new HashMap<>();
                Map<String, Object> yoyMap = new HashMap<>();
                for (int i = 0; i < yoyData.getData().size(); i++) {
                    if (yoyData.getData().get(i).containsKey("lastValue")) {
                        lastValueMap.put("lastValue", yoyData.getData().get(i).get("lastValue"));
                        data.add(lastValueMap);
                    }
                    if (yoyData.getData().get(i).containsKey("yoy")) {
                        yoyMap.put("yoy", yoyData.getData().get(i).get("yoy"));
                        data.add(yoyMap);
                    }
                }
                // 环比数据
                Map<String, Object> curValMap = new HashMap<>();
                Map<String, Object> momMap = new HashMap<>();
                for (int i = 0; i < momData.getData().size(); i++) {
                    if (momData.getData().get(i).containsKey("curValue")) {
                        curValMap.put("curValue", momData.getData().get(i).get("curValue"));
                        data.add(curValMap);
                    }
                    if (momData.getData().get(i).containsKey("mom")) {
                        momMap.put("mom", momData.getData().get(i).get("mom"));
                        data.add(momMap);
                    }
                }
                result.setData(data);
                return AjaxResult.success(result);
            }
        }
        return AjaxResult.success(null);
    }

    private static AjaxResult getConsumptionDTOHttpResult(List<ConsumptionAnalyze> consumptionAnalyzeEntities, String compareType, String len) {
        EnergyConsumptionDTO energyConsumptionDTO = new EnergyConsumptionDTO();
        if (null == consumptionAnalyzeEntities || consumptionAnalyzeEntities.size() == 0) {
            return AjaxResult.success(energyConsumptionDTO);
        }
        String[] lastValue = new String[consumptionAnalyzeEntities.size()];
        String[] curValue = new String[consumptionAnalyzeEntities.size()];
        String[] yoyValue = new String[consumptionAnalyzeEntities.size()];
        String[] timeStrArr = new String[consumptionAnalyzeEntities.size()];
        String[] titleArr = new String[]{"curValue", "lastValue", compareType};
        for (int i = 0; i < consumptionAnalyzeEntities.size(); i++) {
            ConsumptionAnalyze entity = consumptionAnalyzeEntities.get(i);
            lastValue[i] = entity.getLastValue();
            curValue[i] = entity.getCurValue();
            if ("yoy".equalsIgnoreCase(compareType)) {
                yoyValue[i] = entity.getYoy();
            }
            if ("mom".equalsIgnoreCase(compareType)) {
                yoyValue[i] = entity.getMom();
            }
            timeStrArr[i] = entity.getTimeStr().substring(0, Integer.parseInt(len));
        }
        energyConsumptionDTO.setTitles(titleArr);
        energyConsumptionDTO.setTimes(timeStrArr);
        List<Map<String, Object>> listData = new ArrayList<>();
        Map<String, Object> cur = new HashMap<>();
        cur.put("curValue",curValue);
        listData.add(cur);
        Map<String, Object> last = new HashMap<>();
        last.put("lastValue",lastValue);
        listData.add(last);
        Map<String, Object> yoy = new HashMap<>();
        yoy.put(compareType,yoyValue);
        listData.add(yoy);
        energyConsumptionDTO.setData(listData);
        return AjaxResult.success(energyConsumptionDTO);
    }

    private static String getTimeLen(String timeType) {
        String timeLen = "";
        switch (timeType) {
            case "hour":
                timeLen = "13";
                break;
            case "day":
                timeLen = "10";
                break;
            case "month":
                timeLen = "7";
                break;
            case "year":
                timeLen = "4";
                break;
        }
        return timeLen;
    }

    @Override
    public String queryCoolingData(String tableName, String timeString) {
        return energyMapper.queryCoolingData(tableName, timeString);
    }
}
