package com.mh.system.service.energy.impl;

import com.mh.common.core.domain.dto.EnergyPreEchartDataDTO;
import com.mh.common.core.domain.dto.EnergyPreTopDataDTO;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.energy.HistoryDataPreMapper;
import com.mh.system.service.energy.IHistoryDataPreService;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 数据预测
 * @date 2025-03-25 15:36:58
 */
@Service
public class HistoryDataPreServiceImpl implements IHistoryDataPreService {

    private final HistoryDataPreMapper historyDataPreMapper;

    public HistoryDataPreServiceImpl(HistoryDataPreMapper historyDataPreMapper) {
        this.historyDataPreMapper = historyDataPreMapper;
    }

    @Override
    public List<?> getTopData(String systemType, String deviceType) {
        return historyDataPreMapper.getTopData(systemType, deviceType);
    }

    @Override
    public List<?> getEnergyPre(String systemType, String beginDate, String endDate, String deviceType) {
        if (StringUtils.isBlank(beginDate) || StringUtils.isBlank(endDate)) {
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
            // 获取当前日期
            LocalDate now = LocalDate.now();
            // 向前推30天
            LocalDate startDate = now.minusDays(30);
            beginDate = startDate.format(formatter);
            // 结束日期是当前日期
            endDate = now.format(formatter);
        }
        if (StringUtils.isBlank(systemType) || StringUtils.isBlank(deviceType)) {
            return List.of();
        }
        List<EnergyPreEchartDataDTO> energyPre = historyDataPreMapper.getEnergyPre(systemType, beginDate, endDate, deviceType);
        if (energyPre.isEmpty()) {
            return List.of();
        }
        String[] curDate = energyPre.stream()
                .map(EnergyPreEchartDataDTO::getCurDate)
                .toArray(String[]::new); // 使用stream和map转换每个对象的特定字段为JSON字符串，然后转换成数组
        String[] curData = energyPre.stream()
                .map(EnergyPreEchartDataDTO::getCurData)
                .toArray(String[]::new); // 使用stream和map转换每个对象的特定字段为JSON字符串，然后转换成数组
        String[] preData = energyPre.stream()
                .map(EnergyPreEchartDataDTO::getPreData)
                .toArray(String[]::new); // 使用stream和map转换每个对象的特定字段为JSON字符串，然后转换成数组
        String[] errorData = energyPre.stream()
                .map(EnergyPreEchartDataDTO::getErrorData)
                .toArray(String[]::new); // 使用stream和map转换每个对象的特定字段为JSON字符串，然后转换成数组
        List<HashMap<String, Object>> resultList = new ArrayList<>();
        HashMap<String, Object> resultHashMap = new HashMap<>();
        resultHashMap.put("curDate", curDate);
        resultHashMap.put("curData", curData);
        resultHashMap.put("preData", preData);
        resultHashMap.put("errorData", errorData);
        resultList.add(resultHashMap);
        return resultList;
    }

}
