package com.mh.system.service.mqtt.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.MqttSubscription;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.mqtt.MqttSubscriptionMapper;
import com.mh.system.service.mqtt.IMqttSubscriptionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description mqtt订阅实现类
 * @date 2025-02-14 13:59:27
 */
@Service
public class MqttSubscriptionServiceImpl implements IMqttSubscriptionService {

    private final MqttSubscriptionMapper mqttSubscriptionMapper;

    @Autowired
    public MqttSubscriptionServiceImpl(MqttSubscriptionMapper mqttSubscriptionMapper) {
        this.mqttSubscriptionMapper = mqttSubscriptionMapper;
    }

    @Override
    public List<MqttSubscription> selectMqttSubList(MqttSubscription mqttSubscription) {
        if (mqttSubscription == null) {
            return List.of();
        }
        QueryWrapper<MqttSubscription> queryWrapper = new QueryWrapper<>();
        // 订阅主题
        if (!StringUtils.isEmpty(mqttSubscription.getTopic())) {
            queryWrapper.like("topic", mqttSubscription.getTopic());
        }
        // 状态
        if (!StringUtils.isEmpty(mqttSubscription.getStatus())) {
            queryWrapper.eq("status", mqttSubscription.getStatus());
        }
        queryWrapper.orderByDesc("create_time");
        return mqttSubscriptionMapper.selectList(queryWrapper);
    }

    @Override
    public MqttSubscription selectMqttSubById(String msId) {
        return mqttSubscriptionMapper.selectById(msId);
    }

    @Override
    public int insertMqttSub(MqttSubscription mqttSubscription) {
        return mqttSubscriptionMapper.insert(mqttSubscription);
    }

    @Override
    public int updateMqttSub(MqttSubscription mqttSubscription) {
        return mqttSubscriptionMapper.updateById(mqttSubscription);
    }

    @Override
    public int deleteMqttSubByIds(String[] msIds) {
        if (msIds != null && msIds.length > 0) {
            for (String msId : msIds) {
                mqttSubscriptionMapper.deleteById(msId);
            }
            return msIds.length;
        }
        return 0;
    }
}
