package com.mh.system.service.operation.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.AlarmCode;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.operation.AlarmCodeMapper;
import com.mh.system.service.operation.IAlarmCodeService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 报警编码页面请求实现类
 * @date 2025-01-14 16:53:08
 */
@Service
public class AlarmCodeServiceImpl implements IAlarmCodeService {

    @Resource
    private AlarmCodeMapper alarmCodeMapper;

    @Override
    public List<AlarmCode> selectAlarmCodeList(AlarmCode alarmCode) {
        if (alarmCode == null) {
            return List.of();
        }
        QueryWrapper<AlarmCode> queryWrapper = new QueryWrapper<>();
        // 消息编码
        if (!StringUtils.isEmpty(alarmCode.getMsgCode())) {
            queryWrapper.like("msg_code", alarmCode.getMsgCode());
        }
        // 采集类别
        if (!StringUtils.isEmpty(alarmCode.getCollectionType())) {
            queryWrapper.eq("collection_type", alarmCode.getCollectionType());
        }
        // 事件类型
        if (!StringUtils.isEmpty(alarmCode.getEventType())) {
            queryWrapper.eq("event_type", alarmCode.getEventType());
        }
        // 报警等级
        if (!StringUtils.isEmpty(alarmCode.getAlarmLevel())) {
            queryWrapper.eq("alarm_level", alarmCode.getAlarmLevel());
        }
        queryWrapper.orderByDesc("create_time");
        return alarmCodeMapper.selectList(queryWrapper);
    }

    @Override
    public AlarmCode selectAlarmCodeById(String acId) {
        if (StringUtils.isEmpty(acId)) {
            return null;
        }
        return alarmCodeMapper.selectById(acId);
    }

    @Override
    public int insertAlarmCode(AlarmCode alarmCode) {
        if (alarmCode != null) {
            return alarmCodeMapper.insert(alarmCode);
        }
        return 0;
    }

    @Override
    public int updateAlarmCode(AlarmCode alarmCode) {
        if (alarmCode != null) {
            return alarmCodeMapper.updateById(alarmCode);
        }
        return 0;
    }

    @Override
    public int deleteAlarmCodeByIds(String[] acIds) {
        if (acIds != null && acIds.length > 0) {
            for (String acId : acIds) {
                alarmCodeMapper.deleteById(acId);
            }
            return acIds.length;
        }
        return 0;
    }

    @Override
    public AlarmCode selectAlarmCodeByAlarmType(String eventType) {
        if (StringUtils.isEmpty(eventType)) {
            return null;
        }
        QueryWrapper<AlarmCode> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("event_type", eventType);
        return alarmCodeMapper.selectOne(queryWrapper);
    }
}
