package com.mh.system.service.operation.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.AlarmRecords;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.operation.AlarmRecordsMapper;
import com.mh.system.service.operation.IAlarmRecordsService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 报警记录实现类
 * @date 2025-02-21 14:13:13
 */
@Service
public class AlarmRecordsServiceImpl implements IAlarmRecordsService {

    @Resource
    private AlarmRecordsMapper alarmRecordsMapper;

    @Override
    public AlarmRecords selectIsExist(AlarmRecords alarmRecords) {
        QueryWrapper<AlarmRecords> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("alarm_type", alarmRecords.getAlarmType());
        queryWrapper.eq("event_type", alarmRecords.getEventType());
        queryWrapper.eq("alarm_level", alarmRecords.getAlarmLevel());
        queryWrapper.eq("ledger_id", alarmRecords.getLedgerId());
        queryWrapper.eq("cpm_id", alarmRecords.getCpmId());

        // 添加PostgreSQL特有的时间格式判断
        queryWrapper.apply("to_char(create_time, 'YYYY-MM-DD HH24') = {0}",
                new SimpleDateFormat("yyyy-MM-dd HH").format(alarmRecords.getCreateTime()));

        return alarmRecordsMapper.selectOne(queryWrapper);
    }

    @Override
    public void insertAlarmRecord(AlarmRecords alarmRecords) {
        alarmRecordsMapper.insert(alarmRecords);
    }

    @Override
    public void updateAlarmRecord(AlarmRecords alarmRecords) {
        alarmRecordsMapper.updateById(alarmRecords);
    }

    @Override
    public List<AlarmRecords> selectAlarmRecordsList(AlarmRecords alarmRecords) {
        if (alarmRecords == null) {
            return List.of();
        }
        QueryWrapper<AlarmRecords> queryWrapper = new QueryWrapper<>();
        // 报警类型
        if (!StringUtils.isEmpty(alarmRecords.getAlarmType())) {
            queryWrapper.eq("alarm_type", alarmRecords.getAlarmType());
        }
        // 事件类型
        if (!StringUtils.isEmpty(alarmRecords.getEventType())) {
            queryWrapper.eq("event_type", alarmRecords.getEventType());
        }
        // 报警等级
        if (!StringUtils.isEmpty(alarmRecords.getAlarmLevel())) {
            queryWrapper.eq("alarm_level", alarmRecords.getAlarmLevel());
        }
        // 设备名称
        if (!StringUtils.isEmpty(alarmRecords.getDeviceName())) {
            queryWrapper.like("device_name", alarmRecords.getDeviceName());
        }
        // 报警时间范围
        if (alarmRecords.getParams() != null && !alarmRecords.getParams().isEmpty())  {
            String beginTimeStr = (String) alarmRecords.getParams().get("beginTime");
            String endTimeStr = (String) alarmRecords.getParams().get("endTime");

            if (StringUtils.isNotEmpty(beginTimeStr) && StringUtils.isNotEmpty(endTimeStr)) {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
                LocalDate beginTime = LocalDate.parse(beginTimeStr, formatter);
                LocalDate endTime = LocalDate.parse(endTimeStr, formatter);

                queryWrapper.between("create_time", java.sql.Date.valueOf(beginTime), java.sql.Date.valueOf(endTime));
            }
        }
        // 台账id
        if (!StringUtils.isEmpty(alarmRecords.getLedgerId())) {
            queryWrapper.eq("ledger_id", alarmRecords.getLedgerId());
        }
        // 仪表参数id
        if (!StringUtils.isEmpty(alarmRecords.getCpmId())) {
            queryWrapper.eq("cpm_id", alarmRecords.getCpmId());
        }
        // 仪表参数名称
        if (!StringUtils.isEmpty(alarmRecords.getCpmName())) {
            queryWrapper.like("cpm_name", alarmRecords.getCpmName());
        }
        // 报警内容
        if (!StringUtils.isEmpty(alarmRecords.getContent())) {
            queryWrapper.like("content", alarmRecords.getContent());
        }
        queryWrapper.orderByDesc("create_time");
        return alarmRecordsMapper.selectList(queryWrapper);
    }

    @Override
    public AlarmRecords selectAlarmRecordsById(String acdId) {
        return alarmRecordsMapper.selectById(acdId);
    }

    @Override
    public int deleteAlarmRecordsByIds(String[] acdIds) {
        if (acdIds != null && acdIds.length > 0) {
            for (String acdId : acdIds) {
                alarmRecordsMapper.deleteById(acdId);
            }
            return acdIds.length;
        }
        return 0;
    }
}
