package com.mh.system.service.operation.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.AlarmRules;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.operation.AlarmRulesMapper;
import com.mh.system.service.operation.IAlarmRulesService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 报警编码页面请求实现类
 * @date 2025-01-14 16:53:08
 */
@Service
public class AlarmRulesServiceImpl implements IAlarmRulesService {

    @Resource
    private AlarmRulesMapper alarmRulesMapper;

    @Override
    public List<AlarmRules> selectAlarmRulesList(AlarmRules alarmRules) {
        if (alarmRules == null) {
            return List.of();
        }
        QueryWrapper<AlarmRules> queryWrapper = new QueryWrapper<>();
        // 报警类型
        if (!StringUtils.isEmpty(alarmRules.getAlarmType())) {
            queryWrapper.eq("alarm_type", alarmRules.getAlarmType());
        }
        // 事件类型
        if (!StringUtils.isEmpty(alarmRules.getEventType())) {
            queryWrapper.eq("event_type", alarmRules.getEventType());
        }
        // 报警等级
        if (!StringUtils.isEmpty(alarmRules.getAlarmLevel())) {
            queryWrapper.eq("alarm_level", alarmRules.getAlarmLevel());
        }
        queryWrapper.orderByDesc("create_time");
        return alarmRulesMapper.selectList(queryWrapper);
    }

    @Override
    public AlarmRules selectAlarmRulesById(String acId) {
        if (StringUtils.isEmpty(acId)) {
            return null;
        }
        return alarmRulesMapper.selectById(acId);
    }

    @Override
    public int insertAlarmRules(AlarmRules alarmRules) {
        if (alarmRules != null) {
            return alarmRulesMapper.insert(alarmRules);
        }
        return 0;
    }

    @Override
    public int updateAlarmRules(AlarmRules alarmRules) {
        if (alarmRules != null) {
            return alarmRulesMapper.updateById(alarmRules);
        }
        return 0;
    }

    @Override
    public int deleteAlarmRulesByIds(String[] acIds) {
        if (acIds != null && acIds.length > 0) {
            for (String acId : acIds) {
                alarmRulesMapper.deleteById(acId);
            }
            return acIds.length;
        }
        return 0;
    }

    @Override
    public List<AlarmRules> selectAlarmRulesListByStatus(int status) {
        QueryWrapper<AlarmRules> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("status", status);
        queryWrapper.orderByDesc("create_time");
        return alarmRulesMapper.selectList(queryWrapper);
    }
}
