package com.mh.system.service.operation.impl;

import com.alibaba.fastjson2.JSONObject;
import com.mh.common.constant.Constants;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.OrderEntity;
import com.mh.common.core.domain.entity.PolicyManage;
import com.mh.common.model.request.AdvantechDatas;
import com.mh.common.model.response.AdvantechResponse;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.policy.PolicyManageMapper;
import com.mh.system.service.operation.IOperationDeviceService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 设备操作实现类
 * @date 2025-02-14 09:32:34
 */
@Slf4j
@Service
public class OperationDeviceServiceImpl implements IOperationDeviceService {

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Resource
    private PolicyManageMapper policyManageMapper;

    @Override
    public String operationDevice(List<OrderEntity> changeValues) {
        // 拼接发送的报文
        AdvantechResponse<AdvantechDatas> sendData = new AdvantechResponse<>();
        try {
            List<AdvantechDatas> advantechDatas = getAdvantechDatas(changeValues);
            if (null == advantechDatas) {
                return Constants.FAIL;
            }
            sendData.setW(advantechDatas);
            sendData.setTs(DateUtils.dateToTsStr());
        } catch (Exception e) {
            log.error("发送mqtt控制设备指令失败", e);
            return Constants.FAIL;
        }
        return JSONObject.toJSONString(sendData);
    }

    public List<AdvantechDatas> getAdvantechDatas(List<OrderEntity> changeValues) {
        List<AdvantechDatas> advantechDatas = new ArrayList<>();
        for (OrderEntity changeValue : changeValues) {
            // 获取报文
            String message = changeValue.getParam();
            // 获取报文类型
            Integer type = changeValue.getType();
            if (type != null) {
                if (type == 1) {
                    // 设置延时开关时间，参数需要*1000
                    message = String.valueOf(new BigDecimal(message).multiply(new BigDecimal("1000")).intValue());
                }
            }
            // 获取报文其他信息
            String otherName = changeValue.getOtherName();
            if (StringUtils.isEmpty(otherName)) {
                // 从数据库id获取设备名称
                CollectionParamsManage collectionParamsManage = collectionParamsManageMapper.selectById(changeValue.getId());
                if (null != collectionParamsManage) {
                    otherName = collectionParamsManage.getOtherName();
                } else {
                    return null;
                }
                // 从策略管理中获取数据，判断是否需要放大10的n次方倍
                PolicyManage policyManage = policyManageMapper.selectOntByCpmId(changeValue.getId());
                if (null != policyManage) {
                    message = String.valueOf(new BigDecimal(message).multiply(BigDecimal.valueOf(Math.pow(10, policyManage.getDigits()))).intValue());
                }
            }
            // 发送报文
            AdvantechDatas data = new AdvantechDatas();
            data.setTag(otherName);
            data.setValue(Integer.valueOf(message));
            advantechDatas.add(data);
        }
        return advantechDatas;
    }
}
