package com.mh.system.service.overview.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.ColumnFilter;
import com.mh.common.core.domain.dto.BigScreenOverviewAndBasicDTO;
import com.mh.common.core.domain.dto.OverviewEnergyDTO;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.DataMonth;
import com.mh.common.core.domain.entity.SysParams;
import com.mh.common.core.domain.vo.EnergyConsumptionVO;
import com.mh.common.utils.DateUtils;
import com.mh.system.mapper.SysParamsMapper;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.energy.BigScreenMapper;
import com.mh.system.service.overview.IBigScreenService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 大屏实现类
 * @date 2025-03-21 11:47:12
 */
@Service
public class BigScreenServiceImpl implements IBigScreenService {

    private final CollectionParamsManageMapper collectionParamsManageMapper;

    private final BigScreenMapper bigScreenMapper;

    private final SysParamsMapper sysParamsMapper;

    public BigScreenServiceImpl(CollectionParamsManageMapper collectionParamsManageMapper, BigScreenMapper bigScreenMapper, SysParamsMapper sysParamsMapper) {
        this.collectionParamsManageMapper = collectionParamsManageMapper;
        this.bigScreenMapper = bigScreenMapper;
        this.sysParamsMapper = sysParamsMapper;
    }

    @Override
    public List<?> overview(EnergyConsumptionVO vo) {
        // 判断时间类型
        String tableName = getTableName(vo);
        // 查询总表的数据采集参数
        String[] paramTypes = {"16", "17", "18", "19"};
        BigScreenOverviewAndBasicDTO bigScreenOverviewAndBasicDTO = new BigScreenOverviewAndBasicDTO();
        for (String paramType : paramTypes) {
            int mtIsSum = queryCollectionParams(paramType, 40);
            BigDecimal useValue = bigScreenMapper.selectOverviewData(tableName, paramType, vo.getStartTime(), vo.getEndTime(), mtIsSum, 40, null);
            switch (paramType) {
                case "16":
                    bigScreenOverviewAndBasicDTO.setTotalEle(useValue);
                    break;
                case "17":
                    bigScreenOverviewAndBasicDTO.setTotalCold(useValue);
                    break;
                case "18":
                    bigScreenOverviewAndBasicDTO.setTotalWater(useValue);
                    break;
                case "19":
                    bigScreenOverviewAndBasicDTO.setTotalGas(useValue);
                    break;
                default:
                    break;
            }
        }
        // 换算标准煤
        // 用电转换，公式：标准煤量（kgce）= 用电量（kWh）×0.1229
        bigScreenOverviewAndBasicDTO.setEleStandardCoal(bigScreenOverviewAndBasicDTO.getTotalEle().multiply(new BigDecimal("0.1229")));
        // 用冷转换，公式：标准煤量（kgce）= 3.6*用冷量（kW）➗29.3076
        bigScreenOverviewAndBasicDTO.setColdStandardCoal(bigScreenOverviewAndBasicDTO.getTotalCold().multiply(new BigDecimal("3.6")).divide(new BigDecimal("29.3076")));
        // 用水转换，公式：标准煤量（kgce）=用水量（吨）×0.0857
        bigScreenOverviewAndBasicDTO.setWaterStandardCoal(bigScreenOverviewAndBasicDTO.getTotalWater().multiply(new BigDecimal("0.0857")));
        // 用气转换，公式：标准煤量（kgce）=用蒸汽量（吨）×0.0948
        bigScreenOverviewAndBasicDTO.setGasStandardCoal(bigScreenOverviewAndBasicDTO.getTotalGas().multiply(new BigDecimal("0.0948")));
        // 获取建筑面积
        List<SysParams> sysParams = sysParamsMapper.selectSysParamsList();
        if (!sysParams.isEmpty()) {
            bigScreenOverviewAndBasicDTO.setBuildingArea(new BigDecimal(sysParams.get(0).getBuildingArea()));
            // 计算单位面积能耗
            // 用电转换，公式：单位面积能耗（kWh/m2）=用电量（kWh）➗建筑面积（m2）
            bigScreenOverviewAndBasicDTO.setEleUnitArea(bigScreenOverviewAndBasicDTO.getTotalEle().divide(bigScreenOverviewAndBasicDTO.getBuildingArea()));
            // 用冷转换，公式：单位面积能耗（kWh/m2）=用冷量（kW）➗建筑面积（m2）
            bigScreenOverviewAndBasicDTO.setColdUnitArea(bigScreenOverviewAndBasicDTO.getTotalCold().divide(bigScreenOverviewAndBasicDTO.getBuildingArea()));
            // 用水转换，公式：单位面积能耗（kWh/m2）=用水量（吨）➗建筑面积（m2）
            bigScreenOverviewAndBasicDTO.setWaterUnitArea(bigScreenOverviewAndBasicDTO.getTotalWater().divide(bigScreenOverviewAndBasicDTO.getBuildingArea()));
            // 用气转换，公式：单位面积能耗（kWh/m2）=用蒸汽量（吨）➗建筑面积（m2）
            bigScreenOverviewAndBasicDTO.setGasUnitArea(bigScreenOverviewAndBasicDTO.getTotalGas().divide(bigScreenOverviewAndBasicDTO.getBuildingArea()));
        }
        return List.of(bigScreenOverviewAndBasicDTO);
    }

    private int queryCollectionParams(String paramType, int grade) {
        QueryWrapper<CollectionParamsManage> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("param_type", paramType)
                .eq("grade", grade);

        // 先查询总表
        queryWrapper.eq("mt_is_sum", 0);
        Long count = collectionParamsManageMapper.selectCount(queryWrapper);
        if (count > 0) {
            return 0;
        }

        // 查询分表
        return 1;
    }

    @Override
    public List<?> subSystem(EnergyConsumptionVO vo) {
        // 判断时间类型
        String tableName = getTableName(vo);
        // 查询总表的数据采集参数
        String[] paramTypes = {"16", "17"};
        if (vo.getSystemType().equals("1")) {
            paramTypes = new String[]{"16", "18"};
        }
        HashMap<String, BigDecimal> mapData = new HashMap<>();
        for (String paramType : paramTypes) {
            int mtIsSum = queryCollectionParams(paramType, 40);
            BigDecimal useValue = bigScreenMapper.selectOverviewData(tableName, paramType, vo.getStartTime(), vo.getEndTime(), mtIsSum, 40, vo.getSystemType());
            switch (paramType) {
                case "16":
                    mapData.put("totalEle", useValue);
                    break;
                case "17":
                    mapData.put("totalCold", useValue);
                    break;
                case "18":
                    mapData.put("totalWater", useValue);
                    break;
                default:
                    break;
            }
        }
        // 判断冷量为0或者电量为0，直接赋值0.00
        if (vo.getSystemType().equals("0")) {
            if (mapData.get("totalCold").compareTo(BigDecimal.ZERO) == 0 || mapData.get("totalEle").compareTo(BigDecimal.ZERO) == 0) {
                mapData.put("eer", BigDecimal.ZERO);
            } else {
                mapData.put("eer", mapData.get("totalCold").divide(mapData.get("totalEle"), 2, BigDecimal.ROUND_HALF_UP));
            }
        } else if (vo.getSystemType().equals("1")) {
            if (mapData.get("totalWater").compareTo(BigDecimal.ZERO) == 0 || mapData.get("totalEle").compareTo(BigDecimal.ZERO) == 0) {
                mapData.put("eer", BigDecimal.ZERO);
            } else {
                mapData.put("eer", mapData.get("totalEle").divide(mapData.get("totalWater"), 2, BigDecimal.ROUND_HALF_UP));
            }
        }
        return List.of(mapData);
    }

    private static String getTableName(EnergyConsumptionVO vo) {
        String tableName = "data_min" + vo.getStartTime().substring(0, 4);
        switch (vo.getTimeType()) {
            case "year":
                tableName = "data_year";
                break;
            case "month":
                tableName = "data_month";
                break;
            case "day":
                tableName = "data_min" + vo.getStartTime().substring(0, 4);
                break;
            default:
                break;
        }
        return tableName;
    }

    @Override
    public List<?> systemTrend(EnergyConsumptionVO vo) {
        String tableName = "data_hour" + vo.getStartTime().substring(0, 4);
        String dateFormat = "yyyy-MM-dd HH";
        switch (vo.getTimeType()) {
            case "year":
                tableName = "data_month";
                dateFormat = "yyyy-MM";
                break;
            case "month":
                tableName = "data_day";
                dateFormat = "yyyy-MM-dd";
                break;
            case "day":
                tableName = "data_hour" + vo.getStartTime().substring(0, 4);
                break;
            default:
                break;
        }
        String paramType = "16";
        int mtIsSum = queryCollectionParams(paramType, 40);
        List<DataMonth> dataList = bigScreenMapper.selectDataByRangeTime(tableName, paramType, vo.getStartTime(), vo.getEndTime(), mtIsSum, 40, null);
        List<OverviewEnergyDTO> result = new ArrayList<>();
        OverviewEnergyDTO ele = new OverviewEnergyDTO();
        if (dataList == null || dataList.isEmpty()) {
            return List.of(result);
        }
        // dataList使用stream流根据时间分组排序，dataMonth.calcValue求和，并根据时间升序排序
        String finalDateFormat = dateFormat;
        Map<String, List<DataMonth>> groupedData = dataList.stream()
                .collect(Collectors.groupingBy(val -> DateUtils.dateToString(val.getCurTime(), finalDateFormat)));
        // 根据时间分组，求和取总和，再根据时间升序排序
        // 使用LinkedHashMap保持时间顺序
        LinkedHashMap<String, Double> eleMap = dataList.stream()
                .sorted(Comparator.comparing(val->DateUtils.dateToString(val.getCurTime(), finalDateFormat))) // 按时间字段排序
                .collect(Collectors.groupingBy(
                        val->DateUtils.dateToString(val.getCurTime(), finalDateFormat),
                        LinkedHashMap::new,
                        Collectors.summingDouble(value ->
                                value.getCalcValue().doubleValue())
                ));

        // 提取时间序列和数值序列
        String[] timeArray = eleMap.keySet().toArray(new String[0]);
        String[] valueArray = eleMap.values().stream()
                .map(d -> String.format("%.2f", d)) // 保留两位小数
                .toArray(String[]::new);

        // 设置到对象中
        ele.setName("耗电趋势");
        ele.setUnit("kwh");
        ele.setTimeStr(timeArray);
        ele.setData(valueArray);
        result.add(ele);
        return result;
    }

}
