package com.mh.system.service.space.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.AreaInfo;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.space.AreaInfoMapper;
import com.mh.system.service.space.IAreaInfoService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 区域管理实现类
 * @date 2025-02-19 16:46:38
 */
@Service
public class AreaInfoServiceImpl implements IAreaInfoService {
    
    @Resource
    private AreaInfoMapper areaInfoMapper;
    
    @Override
    public List<AreaInfo> selectAreaInfoList(AreaInfo areaInfo) {
        if (areaInfo == null) {
            return List.of();
        }
        QueryWrapper<AreaInfo> queryWrapper = new QueryWrapper<>();
        // 区域名称
        if (!StringUtils.isEmpty(areaInfo.getAreaName())) {
            queryWrapper.like("area_name", areaInfo.getAreaName());
        }
        // 备注
        if (!StringUtils.isEmpty(areaInfo.getRemark())) {
            queryWrapper.like("remark", areaInfo.getRemark());
        }
        // 系统各类型
        if (!StringUtils.isEmpty(areaInfo.getSystemType())) {
            queryWrapper.eq("system_type", areaInfo.getSystemType());
        }
        queryWrapper.orderByAsc("order_num");
        return areaInfoMapper.selectList(queryWrapper);
    }

    @Override
    public AreaInfo selectAreaInfoById(String areaInfo) {
        return areaInfoMapper.selectById(areaInfo);
    }

    @Override
    public int insertAreaInfo(AreaInfo areaInfo) {
        return areaInfoMapper.insert(areaInfo);
    }

    @Override
    public int updateAreaInfo(AreaInfo areaInfo) {
        areaInfo.setUpdateTime(new Date());
        return areaInfoMapper.updateById(areaInfo);
    }

    @Override
    public int deleteAreaInfoByIds(String[] areaInfos) {
        if (areaInfos != null && areaInfos.length > 0) {
            for (String areaInfo : areaInfos) {
                areaInfoMapper.deleteById(areaInfo);
            }
            return areaInfos.length;
        }
        return 0;
    }
}
