package com.mh.system.service.space.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.entity.AreaInfo;
import com.mh.common.core.domain.entity.BuildingInfo;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.space.BuildingInfoMapper;
import com.mh.system.service.space.IBuildingInfoService;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 区域管理实现类
 * @date 2025-02-19 16:46:38
 */
@Service
public class BuildingInfoServiceImpl implements IBuildingInfoService {

    @Resource
    private BuildingInfoMapper buildingInfoMapper;

    @Override
    public List<BuildingInfo> selectBuildingInfoList(BuildingInfo buildingInfo) {
        if (buildingInfo == null) {
            return List.of();
        }
        QueryWrapper<BuildingInfo> queryWrapper = new QueryWrapper<>();
        // 区域id
        if (!StringUtils.isEmpty(buildingInfo.getAreaId())) {
            queryWrapper.eq("area_id", buildingInfo.getAreaId());
        }
        // 楼栋名称
        if (!StringUtils.isEmpty(buildingInfo.getBuildingName())) {
            queryWrapper.like("building_name", buildingInfo.getBuildingName());
        }
        // 备注
        if (!StringUtils.isEmpty(buildingInfo.getRemark())) {
            queryWrapper.like("remark", buildingInfo.getRemark());
        }
        queryWrapper.orderByAsc("order_num");
        return buildingInfoMapper.selectList(queryWrapper);
    }

    @Override
    public BuildingInfo selectBuildingInfoById(String buildingId) {
        return buildingInfoMapper.selectById(buildingId);
    }

    @Override
    public int insertBuildingInfo(BuildingInfo buildingInfo) {
        if (StringUtils.isEmpty(buildingInfo.getAreaId())) {
            throw new RuntimeException("没有对应的区域id");
        }
        return buildingInfoMapper.insert(buildingInfo);
    }

    @Override
    public int updateBuildingInfo(BuildingInfo buildingInfo) {
        if (StringUtils.isEmpty(buildingInfo.getAreaId())) {
            throw new RuntimeException("没有对应的区域id");
        }
        buildingInfo.setUpdateTime(new Date());
        return buildingInfoMapper.updateById(buildingInfo);
    }

    @Override
    public int deleteBuildingInfoByIds(String[] buildingIds) {
        if (buildingIds != null && buildingIds.length > 0) {
            for (String buildingId : buildingIds) {
                buildingInfoMapper.deleteById(buildingId);
            }
            return buildingIds.length;
        }
        return 0;
    }
}
