package com.mh.system.service.space.impl;

import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.CpmSpaceRelation;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.space.CpmSpaceRelationMapper;
import com.mh.system.service.space.ICpmSpaceRelationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 区域、楼栋、楼层、房间跟采集参数关联管理
 * @date 2025-02-19 16:37:19
 */
@Service
public class CpmSpaceRelationServiceImpl implements ICpmSpaceRelationService {

    private final CpmSpaceRelationMapper cpmSpaceRelationMapper;

    private final CollectionParamsManageMapper collectionParamsManageMapper;

    @Autowired
    public CpmSpaceRelationServiceImpl(CpmSpaceRelationMapper cpmSpaceRelationMapper, CollectionParamsManageMapper collectionParamsManageMapper) {
        this.cpmSpaceRelationMapper = cpmSpaceRelationMapper;
        this.collectionParamsManageMapper = collectionParamsManageMapper;
    }

    @Override
    public List<CollectionParamsManage> selectCpmSpaceRelationList(CpmSpaceRelation cpmSpaceRelation) {
        // 判断房间号id是否为空
        if (cpmSpaceRelation == null) {
            return List.of();
        }
        if (!StringUtils.isEmpty(cpmSpaceRelation.getHouseId())) {
            return collectionParamsManageMapper.selectByHouseId(cpmSpaceRelation.getHouseId());
        } else if (!StringUtils.isEmpty(cpmSpaceRelation.getFloorId() )) {
            return collectionParamsManageMapper.selectByFloorId(cpmSpaceRelation.getFloorId());
        } else if (!StringUtils.isEmpty(cpmSpaceRelation.getBuildingId() )) {
            return collectionParamsManageMapper.selectByBuildingId(cpmSpaceRelation.getBuildingId());
        } else  if (!StringUtils.isEmpty(cpmSpaceRelation.getAreaId() )) {
            return collectionParamsManageMapper.selectByAreaId(cpmSpaceRelation.getAreaId());
        } else {
            return List.of();
        }
    }

    @Override
    public CpmSpaceRelation selectCpmSpaceRelationById(String cpmSpaceRelationId) {
        return cpmSpaceRelationMapper.selectById(cpmSpaceRelationId);
    }

    @Override
    public int insertCpmSpaceRelation(CpmSpaceRelation cpmSpaceRelation) {
        return cpmSpaceRelationMapper.insert(cpmSpaceRelation);
    }

    @Override
    public int updateCpmSpaceRelation(CpmSpaceRelation cpmSpaceRelation) {
        return cpmSpaceRelationMapper.updateById(cpmSpaceRelation);
    }

    @Override
    public int deleteCpmSpaceRelationByIds(String[] buildingIds) {
        if (buildingIds != null && buildingIds.length > 0) {
            for (String buildingId : buildingIds) {
                cpmSpaceRelationMapper.deleteById(buildingId);
            }
            return buildingIds.length;
        }
        return 0;
    }
}
