package com.mh.system.service.space.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mh.common.core.domain.SpaceTreeSelect;
import com.mh.common.core.domain.entity.FloorInfo;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.space.FloorInfoMapper;
import com.mh.system.service.space.IFloorInfoService;
import com.mh.system.service.space.IHouseInfoService;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 楼层实现类
 * @date 2025-02-19 17:28:13
 */
@Service
public class FloorInfoServiceImpl implements IFloorInfoService {

    @Resource
    private FloorInfoMapper floorInfoMapper;

    @Autowired
    private IHouseInfoService houseInfoService;

    @Override
    public List<FloorInfo> selectFloorInfoList(FloorInfo floorInfo) {
        if (floorInfo == null) {
            return List.of();
        }
        QueryWrapper<FloorInfo> queryWrapper = new QueryWrapper<>();
        // 区域id
        if (!StringUtils.isEmpty(floorInfo.getAreaId())) {
            queryWrapper.eq("area_id", floorInfo.getAreaId());
        }
        // 楼栋id
        if (!StringUtils.isEmpty(floorInfo.getBuildingId())) {
            queryWrapper.like("building_id", floorInfo.getBuildingId());
        }
        // 楼层名称
        if (!StringUtils.isEmpty(floorInfo.getFloorName())) {
            queryWrapper.like("floor_name", floorInfo.getFloorName());
        }
        // 备注
        if (!StringUtils.isEmpty(floorInfo.getRemark())) {
            queryWrapper.like("remark", floorInfo.getRemark());
        }
        queryWrapper.orderByAsc("order_num");
        return floorInfoMapper.selectList(queryWrapper);
    }

    @Override
    public FloorInfo selectFloorInfoById(String floorId) {
        return floorInfoMapper.selectById(floorId);
    }

    @Override
    public int insertFloorInfo(FloorInfo floorInfo) {
        getAreaId(floorInfo);
        return floorInfoMapper.insert(floorInfo);
    }

    @Override
    public int updateFloorInfo(FloorInfo floorInfo) {
        getAreaId(floorInfo);
        floorInfo.setUpdateTime(new Date());
        return floorInfoMapper.updateById(floorInfo);
    }

    private void getAreaId(FloorInfo floorInfo) {
        if(StringUtils.isEmpty(floorInfo.getAreaId())) {
            // 获取完整的空间树结构
            List<SpaceTreeSelect> spaceTree = houseInfoService.buildTree(null);

            // 递归查找目标楼栋节点
            SpaceTreeSelect buildingNode = findBuildingNode(spaceTree, floorInfo.getBuildingId());

            if(buildingNode != null && buildingNode.getParentId() != null) {
                floorInfo.setAreaId(buildingNode.getParentId());
            } else {
                throw new RuntimeException("未找到楼栋ID对应的区域信息");
            }
        }
        if(StringUtils.isEmpty(floorInfo.getBuildingId())) {
            throw new RuntimeException("没有对应的楼栋id");
        }
    }

    // 递归查找楼栋节点方法
    private SpaceTreeSelect findBuildingNode(List<SpaceTreeSelect> nodes, String targetBuildingId) {
        for (SpaceTreeSelect node : nodes) {
            if (node.getNodeType() == 2 && targetBuildingId.equals(node.getId())) {
                return node;
            }
            if (node.getChildren() != null && !node.getChildren().isEmpty()) {
                SpaceTreeSelect found = findBuildingNode(node.getChildren(), targetBuildingId);
                if (found != null) {
                    return found;
                }
            }
        }
        return null;
    }

    @Override
    public int deleteFloorInfoByIds(String[] floorIds) {
        if (floorIds != null && floorIds.length > 0) {
            for (String floorId : floorIds) {
                floorInfoMapper.deleteById(floorId);
            }
            return floorIds.length;
        }
        return 0;
    }
}
